/**
 * webEdition CMS
 *
 * $Rev: 15378 $
 * $Author: mokraemer $
 * $Date: 2018-06-20 14:14:41 +0200 (Wed, 20 Jun 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

CodeMirror.defineMode("text/weTmpl", function (config, parserConfig) {
	var webeditionOverlay = {
		startState: function () {
			return {
				'insideTag': false,
				'tagName': "",
				'attrName': "",
				'typeName': "",
				'open': false,
				'close': false,
				'attrActive': false,
				'comment': false
			};
		},
		token: function (stream, state) {
			var quot, value, val, ch, attrName, comma = false;
			if (state.insideTag) {
				if (state.close) {
					if (stream.skipTo(">")) {
						stream.next();
					}
					state.insideTag = false;
					state.typeName = "";
					return "weCloseTag weTag";
				}
				if (state.open) {
					if (stream.eatSpace()) {
						return null;
					}
					if (state.attrActive) {
						stream.next();//consume =
						quot = false;
						value = "";
						while ((ch = stream.next()) !== null && ch !== undefined) {
							switch (ch) {
								case "\\":
									if (stream.peek() === "\"") {
										stream.next();
									}
									continue;
								case ",":
									if (!quot) {
										continue;
									}
									comma = true;
								case "\"":
									if (!quot) {
										quot = true;
										comma = false;
										continue;
									}
									//reset if closing ", keep if was comma
									quot = (comma ? quot : false);
									state.attrActive = false;
									val = value;
									value = "";//reset if comma
									switch (state.attrName) {
										case "type":
											state.typeName = value;
											return null;
										case "id":
										case "pid":
											if (state.typeName === "") {
												switch (state.tagName) {
													case "object":
														state.typeName = "object";
														break;
													case 'ifTemplate':
														state.typeName = "template";
														break;
													default:
														state.typeName = "document";
												}
											}
											return ((val - 0 == val) ? ("number we" + state.typeName + "ID-" + val) + " WEID" : "string");
										case "objectid":
											return ((val - 0 == val) ? ("number weobjectID-" + val) + " WEID" : "string");
										case "triggerid":
										case "startid":
										case "parentid":
										case "documentid":
										case "docid":
										case "thumbnailid":
											return ((val - 0 == val) ? ("number wedocumentID-" + val) + " WEID" : "string");
										case "workflowuserid":
										case "userid":
											return ((val - 0 == val) ? ("number weuserID-" + val) + " WEID" : "string");
										case "classid":
											return ((val - 0 == val) ? ("number weclassID-" + val) + " WEID" : "string");
										default:
											return null;

									}
								default:
									value += ch;
							}
						}
					} else {
						attrName = "";
						state.attrActive = true;
						while ((ch = stream.next()) !== null && ch !== undefined) {
							switch (ch) {
								default:
									attrName += ch;
									stream.eatSpace();
									if (stream.peek() === "=") {
										state.attrName = attrName;
										return "weTagAttribute weTag_" + state.tagName + "_" + attrName;
									}
									continue;
								case "/":
									if (stream.skipTo('>')) {
										stream.next();
									}
									/* falls through */
								case ">":
									state.insideTag = false;
									return "weTag " + attrName;
							}
						}
					}
				}
				stream.skipToEnd();
				return null;
			} else {
				state.open = stream.match("<we:");
				state.close = !state.open && stream.match("</we:");
				state.attrActive = false;
				if (state.open || state.close) {
					state.insideTag = true;
					state.tagName = "";
					state.typeName = "";
					state.attrName = "";

					while ((ch = stream.next()) !== null && ch !== undefined) {
						switch (ch) {
							default:
								state.tagName += ch;
								continue;
							case '/':
								if (state.open && (stream.eatSpace() || stream.peek() === ">")) {
									stream.next();
									state.insideTag = false;
									return "weSelfClose weTag weTag_" + state.tagName;
								}
								/* falls through */
							case '>':
								state.insideTag = false;
								if (state.open) {
									if (state.tagName === "comment") {
										state.comment = true;
									}
									return "weOpenTag weTag weTag_" + state.tagName;
								}
								if (state.tagName === "comment") {
									state.comment = false;
								}
								return "weCloseTag weTag";

							case ' ':
								stream.eatSpace();
								if (stream.peek() === ">" || stream.peek() === "/") {
									continue;
								}
								return "weOpenTag weTag weTag_" + state.tagName;
						}
					}
				}
				if (state.comment) {
					if (stream.match(/.*<\/we:comment/, true)) {
						stream.backUp(12);
						state.comment = false;
					} else {
						stream.skipToEnd();
					}
					return 'comment';
				}

				if (!stream.eol() && stream.peek() === "<") {
					stream.next();
				}
				if (!stream.skipTo("<")) {
					stream.skipToEnd();
				}
				return null;
			}
		}
	};
	return CodeMirror.overlayMode(CodeMirror.getMode(config, parserConfig.backdrop || "application/x-httpd-php"), webeditionOverlay);
});
