/* global WE, top */

/**
 webEdition CMS
 *
 * $Rev: 15369 $
 * $Author: mokraemer $
 * $Date: 2018-06-19 12:12:44 +0200 (Tue, 19 Jun 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function Fileupload_controller_abstract(uploader) {
	this.uploader = uploader;

	this.isMultiUpload = false;
	this.isPreset = false; // to uploader
	this.cmdFileselectOnclick = '';
	this.elemFileDragClasses = 'we_file_drag';
	this.countDraghover = 0;

	this.ON = {
		UPLOAD_START: 'upload_start',
		UPLOAD_FINISH: 'upload_finish',
		UPLOAD_CANCEL: 'upload_cancel',
		FILE_BEFORE_ADD: 'file_before_add',
		FILE_ADD: 'file_add',
		FILE_REMOVE: 'file_remove',
		FILE_SEND: 'file_send',
		FILE_RESPONSE_OK: 'file_response_ok',
		FILE_RESPONSE_NOK: 'file_response_nok',
		CHUNK_RESPONSE_OK: 'chunk_response_ok',
		CHUNK_RESPONSE_NOK: 'chunk_response_nok',
		GUI_INIT: 'gui_init',
		GUI_RESET: 'gui_reset',
		IMAGE_POSTPROCESS: 'img_postprocess'
	};

	this.STATE = {
		READY: 'ready',
		NOTREADY: 'notready',
		UPLOADING: 'uploading',
		UPLOADED: 'uploaded',
		FINISHED: 'finished',
		CANCELLED: 'cancelled',
		EMPTY: 'empty',
		RESET: 'reset',
		IMG_NOTPROCESSED: 'img_not_processed',
		IMG_PROCESSED: 'img_processed',
		IMG_DONOTEDIT: 'img_donotedit',
		ITEM_UPLOADING: 'item_uploaded',
		ITEM_UPLOAD_NOK: 'item_upload_nok',
		ITEM_UPLOADED: 'item_uploading',
		ITEM_SIZE_OK: 'item_size_ok',
		ITEM_SIZE_NOK: 'item_size_nok',
		ITEM_TYPE_OK: 'item_type_ok',
		ITEM_TYPE_NOK: 'item_type_nok'
	};

	this.init = function (conf) {
		this.sender = this.uploader.sender; // on init all components are initialized
		this.view = this.uploader.view;
		this.imageEdit = this.uploader.imageEdit;
		this.utils = this.uploader.utils;

		this.isPreset = conf.isPreset ? conf.isPreset : this.isPreset;
		this.isMultiUpload = !!conf.isMultiUpload;
		this.cmdFileselectOnclick = conf.cmdFileselectOnclick ? conf.cmdFileselectOnclick : this.cmdFileselectOnclick;

		this.init_sub(conf);
	};

	this.init_sub = function () {
		// to be overridden
	};

	this.onload = function () {
		var view = this.view;

		if (view.elems.fileSelect) {
			view.elems.fileSelect.addEventListener('change', this.fileSelectHandler.bind(this), false);
			view.elems.fileInputWrapper.addEventListener('click', this.fileselectOnclick.bind(this), false);
		}
		if (view.elems.fileDrag && view.isDragAndDrop) {
			view.elems.fileDrag.addEventListener('dragenter', this.fileDragHover.bind(this), false);
			view.elems.fileDrag.addEventListener('dragleave', this.fileDragLeave.bind(this), false);
			view.elems.fileDrag.addEventListener('drop', this.fileSelectHandler.bind(this), false);
			view.elems.fileDrag.style.display = 'block';
		} else {
			view.isDragAndDrop = false;
			if (view.elems.fileDrag) {
				view.elems.fileDrag.style.display = 'none';
			}
		}

		var form = this.uploader.doc.we_form;
		form.elements.fuOpts_scaleProps.addEventListener('change', this.editOptionsOnChange.bind(this));
		form.elements.fuOpts_scale.addEventListener('keyup', this.editOptionsOnChange.bind(this), false);
		form.elements.fuOpts_scaleWhat.addEventListener('change', this.editOptionsOnChange.bind(this), false);
		form.elements.fuOpts_quality.addEventListener('change', this.editOptionsOnChange.bind(this), false);

		form.getElementsByClassName('btnImgEditRefresh_global')[0].addEventListener('click', this.handleButtonEditImageGlobal.bind(this), false);

		this.onload_sub();
	};

	this.onload_sub = function () {
		// to be overridden
	};

	this.fileselectOnclick = function () {
		if (uploader.controller.cmdFileselectOnclick) {
			var tmp = uploader.controller.cmdFileselectOnclick.split(',');
			var win = this.uploader.win;

			if (win.we_cmd) {
				win.we_cmd.apply(win, tmp);
			} else { // FIXME: make sure have a function we_cmd on every opener!
				win.top.we_cmd.apply(win, tmp);
			}
		} else {

		}
	};

	this.checkIsPresetFiles = function () {
		return this.isPreset && (WE().layout.weEditorFrameController.getVisibleEditorFrame().document.presetFileupload ||
			this.uploader.win.opener.document.presetFileupload);
	};

	this.loadPresetFiles = function () {
		if (this.isPreset && WE().layout.weEditorFrameController.getVisibleEditorFrame().document.presetFileupload) {
			this.fileSelectHandler(null, true, WE().layout.weEditorFrameController.getVisibleEditorFrame().document.presetFileupload);
		} else if (this.isPreset && this.uploader.win.opener.document.presetFileupload) {
			this.fileSelectHandler(null, true, this.uploader.win.opener.document.presetFileupload);
			//this.uploader.win.setTimeout(this.fileSelectHandler.bind(this, null, true, this.uploader.win.opener.document.presetFileupload), 10);
		}
	};

	this.handleButtonEditImageGlobal = function (e) {
		this.imageEdit.currentEdit.mode = 'global';
		this.imageEdit.currentEdit.index = 0;
		this.imageEdit.currentEdit.task = 'edit';
		this.editImageButtonOnClick(e.target, -1, true);
	};

	this.handleIconHelpEnter = function (e) {
		this.editOptionsHelp(e.target, 'enter');
	};

	this.handleIconHelpLeave = function (e) {// take event type from e!
		this.editOptionsHelp(e.target, 'leave');
	};

	this.handleAlertIconEnter = function (e) {
		e.target.lastChild.style.display = 'block';
	};

	this.handleAlertIconLeave = function (e) {// take event type from e!
		e.target.lastChild.style.display = 'none';
	};

	this.handleButtonEditImageItem = function (e) {
		var btn = e.target.nodeName === 'I' ? e.target.parentNode : e.target;
		var index = btn.parentNode.attributes['data-index'].value;

		this.imageEdit.currentEdit.mode = 'item';
		this.imageEdit.currentEdit.index = index;
		this.imageEdit.currentEdit.task = 'edit';
		this.editImageButtonOnClick(btn, index, false);
	};

	this.handleButtonRotateItem = function (e) {
		var btn = e.target.nodeName === 'I' ? e.target.parentNode : e.target;
		this.itemEditOptionsOnChange(btn, parseInt(btn.parentNode.attributes['data-index'].value), false);
	};

	this.fileSelectHandler = function (e, isPreset, presetFileupload) {
		var files = [];
		files = this.selectedFiles = isPreset ? (presetFileupload.length ? presetFileupload : files) : (e.target.files || e.dataTransfer.files);

		if (files.length) {
			this.processEvents(this.ON.FILE_BEFORE_ADD);

			var container;
			if (this.sender.preparedFiles.length && (container = this.uploader.doc.getElementById('uploadFiles')) && container.lastChild) {
				this.view.scrollTopOnAddFiles = container.lastChild.offsetTop - 200;
			}

			if (!isPreset) {
				if (e.type === 'drop') {
					e.stopPropagation();
					e.preventDefault();
					this.fileDragLeave(e);
				}
			}

			this.fileselectOnclick();

			this.imageEdit.imageFilesToProcess = [];
			for (var f, i = 0; i < files.length; i++) {
				if (!this.utils.contains(this.sender.preparedFiles, this.selectedFiles[i])) {
					f = this.prepareFile(this.selectedFiles[i]);
					this.sender.preparedFiles.push(f);
					this.view.prepareItemAndAppend(f, this.sender.preparedFiles.length - 1);
					this.processEvents(this.ON.FILE_ADD, f.index, f);
				}
			}

			if (this.imageEdit.imageFilesToProcess.length) {
				this.imageEdit.currentEdit.mode = 'global';
				this.imageEdit.currentEdit.index = 0;
				this.imageEdit.currentEdit.task = 'add';
				this.imageEdit.processImages();
			}

			//IMI: REREGISTER MEMORY
		}
	};

	this.setEditorIsHot = function () {
		// to be overridden
	};

	this.processEvents = function (event, index, fileobj) {
		switch (event) {
			case this.ON.UPLOAD_START:
				this.view.setStateGUI(this.STATE.UPLOADING);
				this.view.setStateItem(this.STATE.ITEM_UPLOADING, -1);
				this.uploader.uploaderState = this.STATE.UPLOADING;
				break;
			case this.ON.UPLOAD_FINISH:
				break;
			case this.ON.UPLOAD_CANCEL:
				this.uploader.uploaderState = this.STATE.CANCELLED;
				this.sender.isCancelled = true;
				if (this.view.reloadOpener) {
					this.view.reloadOpener();
				}
				index = this.sender.currentFile ? this.sender.currentFile.index : 0;
				this.view.setInternalProgressCompleted(false, index);
				this.view.setStateGUI(this.STATE.CANCELLED);
				this.view.showMessageCancel(this.sender.singleSuccessMessages);
				break;
			case this.ON.FILE_BEFORE_ADD:
				if (!this.isMultiUpload || this.sender.isCancelled) {
					// reset gui and sender but REMAIN selection in fileSelect (otherwise we would fire ON.GUI_RESET)
					this.uploader.uploaderState = this.STATE.EMPTY;
					this.sender.resetSender();
					this.view.setStateGUI(this.STATE.RESET);
					break;
				}
				break;
			case this.ON.FILE_REMOVE:
				if (this.sender.preparedFiles.length === 0) {
					this.view.setStateGUI(this.STATE.EMPTY);
					this.uploader.uploaderState = this.STATE.EMPTY;
					break;
				}
				/* falls through */
			case this.ON.FILE_ADD:
				var ready = this.sender.getIsUploadableFiles();
				this.view.setStateGUI((ready ? this.STATE.READY : this.STATE.NOTREADY), index, fileobj);
				this.uploader.uploaderState = ready ? this.STATE.READY : this.STATE.NOTREADY;
				break;
			case this.ON.FILE_SEND:
				this.view.setInternalProgress(0, index);
				this.view.setExternalProgressText(index);
				break;
			case this.ON.FILE_RESPONSE_OK:
				this.view.setInternalProgressCompleted(true, index);
				this.uploader.doc.getElementById('div_upload_files').scrollTop = this.uploader.doc.getElementById('uploadItem_' + index).offsetTop - 364;
				break;
			case this.ON.CHUNK_RESPONSE_OK:
				var progress = (100 / fileobj.size) * fileobj.currentWeightFile;
				var totalProgress = (100 / this.sender.totalWeight) * this.sender.currentWeight;
				this.view.setInternalProgress(progress, fileobj.index);
				this.view.setExternalProgress(totalProgress);
				break;
			case this.ON.FILE_RESPONSE_NOK:
			case this.ON.CHUNK_RESPONSE_NOK:
				this.view.setStateItem(this.STATE.ITEM_UPLOAD_NOK, index, fileobj);
				this.view.setExternalProgress((100 / this.sender.totalWeight) * this.sender.currentWeight);
				this.view.setExternalProgressText(index);
				this.uploader.doc.getElementById('div_upload_files').scrollTop = this.uploader.doc.getElementById('uploadItem_' + index).offsetTop - 364;
				this.view.setStateGUI(this.STATE.CANCELLED, fileobj);
				break;
			case this.ON.GUI_INIT:
			case this.ON.GUI_RESET:
				this.uploader.uploaderState = this.STATE.EMPTY;
				this.view.elems.fileSelect.value = null;
				this.sender.resetSender();
				this.view.setStateGUI(this.STATE.RESET);
				break;
			case this.ON.IMAGE_POSTPROCESS:
				break;
		}
	};

	this.prepareFile = function (f, isUploadable) {
		var fileObj = {
			file: f,
			fileNum: 0,
			dataArray: null,
			dataUrl: null,
			isEdited: false,
			img: {// TODO: get empty object img from imageEdit
				processedOptions: {
					doEdit: false,
					from: 'general',
					scaleWhat: 'pixel_l',
					scale: 0,
					rotate: 0,
					quality: this.imageEdit.OPTS_QUALITY_NEUTRAL_VAL
				},
				editOptions: {
					doEdit: false,
					from: 'general',
					scaleWhat: 'pixel_l',
					scale: 0,
					rotate: 0,
					quality: this.imageEdit.OPTS_QUALITY_NEUTRAL_VAL
				},
				origWidth: 0,
				origHeight: 0,
				tooSmallToScale: false,
				isUploadStarted: false,
				isOrientationChecked: false,
				orientationValue: 0,
				workingCanvas: null, // maybe call it workingcanvas
				previewCanvas: null,
				actualRotation: 0,
				previewRotation: 0,
				previewImg: null, // created only for edited images: unedited images can be very large...
				originalSize: 0,
				jpgCustomSegments: null,
				pngTextChunks: null,
				focusX: 0,
				focusY: 0
			},
			size: 0,
			currentPos: 0,
			partNum: 0,
			currentWeightFile: 0,
			mimePHP: 'none',
			fileNameTemp: '',
			fileName: '',
			type: '',
			error: '',
			failedRequest: '',
			isEditable: false,
			isUploadable: false,
			isTypeOk: false,
			isTypeForbiddenByUploader: false,
			isSizeOk: false,
			isMetaToImport: false,
			getIsUploadable: function () {
				return this.isTypeOk && this.isSizeOk;
			}
		};

		fileObj.type = f.type ? f.type : 'text/plain';
		fileObj.size = f.size;

		fileObj.isTypeOk = !!this.utils.checkFileType(fileObj.type, f.name);
		if (!fileObj.isTypeOk) {
			fileObj.isTypeForbiddenByUploader = this.utils.checkIsTypeForbiddenByUploader(fileObj.type, f.name);
		}
		fileObj.isSizeOk = (fileObj.size <= this.sender.maxUploadSize || !this.sender.maxUploadSize);
		fileObj.isUploadable = fileObj.isTypeOk && fileObj.isSizeOk; //maybe replace uploadConditionOk by this

		/* mabe reactivate this later
		 var errorMsg = [
		 WE().consts.g_l.fileupload.errorNoFileSelected,
		 WE().consts.g_l.fileupload.errorFileSize,
		 WE().consts.g_l.fileupload.errorFileType,
		 WE().consts.g_l.fileupload.errorFileSizeType
		 ];
		 fileObj.error = errorMsg[fileObj.isSizeOk && fileObj.isTypeOk ? 0 : (!fileObj.isSizeOk && fileObj.isTypeOk ? 1 :
		 (fileObj.isSizeOk && !fileObj.isTypeOk.isTypeOk ? 2 : 3))];
		 */

		fileObj.isEditable = (fileObj.isTypeOk && this.imageEdit.EDITABLE_CONTENTTYPES.indexOf(fileObj.type) !== -1);
		fileObj.totalParts = Math.ceil(f.size / this.sender.chunkSize);
		fileObj.lastChunkSize = f.size % this.sender.chunkSize;
		fileObj.img.originalSize = f.size;

		if (this.imageEdit.IMPORTMETA_CONTENTTYPES.indexOf(f.type) !== -1) {
			fileObj.isMetaToImport = true;
		}

		if (fileObj.isEditable) {
			this.imageEdit.imageFilesToProcess.push(fileObj);
		}

		return fileObj;
	};

	this.fileDragHover = function (e) {
		e.preventDefault();
		this.countDraghover++;
		this.view.elems.fileDrag.className = this.elemFileDragClasses + ' we_file_drag_hover';
	};

	this.fileDragLeave = function (e) {
		e.preventDefault();
		this.countDraghover--;
		if (this.countDraghover === 0) {
			this.view.elems.fileDrag.className = this.elemFileDragClasses;
		}
	};

	this.editImageButtonOnClick = function (elem, index, general) {
		var btn = elem.nodeName === 'I' ? elem.parentNode : elem;
		btn.disabled = true;

		this.imageEdit.reeditImage(index, general);
	};

	this.editFilename = function (e) {
		var doc = this.uploader.doc;
		var elem = e.target;
		var index, item;

		switch (e.target.tagName) {
			case 'I':
				elem = elem.parentNode;
				/* falls through */
			case 'BUTTON':
				elem = elem.parentNode;
				break;
		}

		index = elem.attributes['data-index'].value;
		item = doc.getElementById('uploadItem_' + index);

		switch (elem.attributes['data-task'].value) {
			case 'editFilename':
				item.getElementsByClassName('elemShowFilename')[0].style.display = 'none';
				item.getElementsByClassName('elemEditFilename')[0].style.display = 'block';

				var inputFilename = item.getElementsByClassName('inputEditFilename')[0];
				this.tmpFilename = inputFilename.value;
				inputFilename.focus();
				break;
			case 'showFilename':
				var elemFilename = item.getElementsByClassName('elemShowFilename')[0];
				var textFilename = item.getElementsByClassName('textShowFilename')[0];

				switch (e.type) {
					case 'change':
						var filename = (item.getElementsByClassName('inputEditFilename')[0].value || this.tmpFilename) + '.' + elemFilename.attributes['data-extension'].value;

						this.view.addTextCutLeft(textFilename, filename, this.view.widthInputFilename);
						this.sender.preparedFiles[index].fileName = filename;
						break;
					case 'blur':
						item.getElementsByClassName('elemEditFilename')[0].style.display = 'none';
						elemFilename.style.display = 'block';
						break;
				}
				break;
		}
	};

	this.editOptionsOnChange = function (event) {
		var target = event.target;
		var form = target.form; // this is allways we_form
		var inputScale = form.elements.fuOpts_scale;
		//var inputQuality = form.elements.fuOpts_quality;
		var scale = inputScale.value;
		var btnRefresh = form.getElementsByClassName('btnImgEditRefresh_global')[0];

		switch (target.name) {
			case 'fuOpts_scaleWhat':
				if (scale) {
					btnRefresh.disabled = false;
					this.imageEdit.uneditImage(-1, true);
					this.view.setStateItemEdit(this.STATE.IMG_NOTPROCESSED, -1, true);
				}
				btnRefresh.disabled = this.sender.preparedFiles.length === 0 || !scale;
				break;
			case 'fuOpts_scaleProps':
				inputScale.value = target.value;
				scale = target.value;
				inputScale.focus();
				target.value = 0;
				/* falls through*/
			case 'fuOpts_scale':
				if (this.imageEdit.MAX_LONGEST !== -1) {
					if (parseInt(scale) > this.imageEdit.MAX_LONGEST) {
						WE().util.showMessage(WE().consts.g_l.fileupload.editMaxLongest.replace('##MAX_LONGEST##', this.imageEdit.MAX_LONGEST), WE().consts.message.WE_MESSAGE_INFO, this.uploader.win);
						scale = this.imageEdit.MAX_LONGEST;
						target.value = this.imageEdit.MAX_LONGEST;
					}
					if (!scale) {
						WE().util.showMessage(WE().consts.g_l.fileupload.editScaleMandatory.replace('##MAX_LONGEST##', this.imageEdit.MAX_LONGEST), WE().consts.message.WE_MESSAGE_INFO, this.uploader.win);
						scale = this.imageEdit.MAX_LONGEST;
						target.value = this.imageEdit.MAX_LONGEST;
					}
				}
				this.imageEdit.uneditImage(-1, true);
				this.view.setStateItemEdit('', -1, true);
				btnRefresh.disabled = this.sender.preparedFiles.length === 0;
				break;
			case 'fuOpts_quality':
				this.imageEdit.uneditImage(-1, true);
				this.view.setStateItemEdit('', -1, true, 'keep');
				btnRefresh.disabled = this.sender.preparedFiles.length === 0;
				break;
		}
	};

	this.itemEditOptionsOnChange = function (target, index, orientationAutoOpt) {
		index = index || 0;
		var btnRefresh = this.uploader.doc.getElementById('make_preview_' + index);
		var item = this.uploader.doc.getElementById('uploadItem_' + index);

		switch (target.name) {
			case 'fuOpts_addRotationLeft':
			case 'fuOpts_addRotationRight':
				var hiddenRotation = item.getElementsByClassName('fuOpts_rotate')[0];
				var rotation = parseInt(hiddenRotation.value);
				var newRotation = (rotation + (target.name === 'fuOpts_addRotationLeft' ? 270 : 90)) % 360;

				// FIXME: REDESIGN LANDCSAPE CORRECTION
				if (!orientationAutoOpt) {
					this.sender.preparedFiles[index].img.orientationIsProposition = false;
				}

				hiddenRotation.value = newRotation;
				this.view.previewSyncRotation(index, newRotation);

				this.imageEdit.uneditImage(index, false);
				if (!newRotation && !parseInt(this.imageEdit.imageEditOptions.scale)) { // if there is no other editing from general we set entry on doEdit=false
					this.view.setStateItemEdit('', index, false, 'flip');
				} else {
					this.view.setStateItemEdit(this.STATE.IMG_NOTPROCESSED, index, false, 'flip');
				}
				btnRefresh.disabled = false;



				break;
		}
	};

	this.editOptionsHelp = function (target, dir) {
		if (dir === 'enter') {
			var fileobj = this.sender.preparedFiles[target.getAttribute('data-index')];
			var scaleReference = fileobj.img.editOptions.scaleWhat === 'pixel_w' ? fileobj.img.origWidth : (
				fileobj.img.editOptions.scaleWhat === 'pixel_h' ? fileobj.img.origHeight : Math.max(fileobj.img.origHeight, fileobj.img.origWidth));
			var text = WE().consts.g_l.fileupload.editTargetsizeTooLarge;

			target.lastChild.innerHTML = text.replace('##ORIGSIZE##', scaleReference);
			target.lastChild.style.display = 'block';
		} else {
			target.lastChild.style.display = 'none';
		}
	};

	this.fireCmd = function (args) {
		var target = args[0].we_cmd ? args[0] : args[0].top;
		target.we_cmd.apply(target, args[1]);
	};
}

function Fileupload_controller_base(uploader) {
	this.uploader = uploader;
	Fileupload_controller_abstract.call(this, uploader);
}
Fileupload_controller_base.prototype = Object.create(Fileupload_controller_abstract.prototype);
Fileupload_controller_base.prototype.constructor = Fileupload_controller_base;

function Fileupload_controller_bindoc(uploader) {
	this.uploader = uploader;
	Fileupload_controller_abstract.call(this, uploader);

	this.elemFileDragClasses = 'we_file_drag';
	this.doSubmit = false;

	this.onload_sub = function () {

	};

	this.doBeforeLeaveEditpage = function () {
		if (!this.uploader.sender.getIsUploadableFiles()) {
			return true;
		}
		WE().util.showMessage(WE().consts.g_l.fileupload.alertUploadPending, WE().consts.message.WE_MESSAGE_ALERT, this.uploader.win);

		return false;
	};

	this.setEditorIsHot = function () {
		if (this.uploader.uiType !== 'wedoc') {
			WE().layout.weEditorFrameController.setEditorIsHot(true, WE().layout.weEditorFrameController.ActiveEditorFrameId);
		}
	};
}
Fileupload_controller_bindoc.prototype = Object.create(Fileupload_controller_abstract.prototype);
Fileupload_controller_bindoc.prototype.constructor = Fileupload_controller_bindoc;

function Fileupload_controller_import(uploader) {
	Fileupload_controller_abstract.call(this, uploader);

	this.onload_sub = function () {
		// FIXME: use state INIT_GUI to set button states
		WE().layout.button.display(this.view.elems.footer.document, 'next', false);
		WE().layout.button.display(this.view.elems.footer.document, 'upload', true);
		WE().layout.button.disable(this.view.elems.footer.document, 'upload', true);
	};

}
Fileupload_controller_import.prototype = Object.create(Fileupload_controller_abstract.prototype);
Fileupload_controller_import.prototype.constructor = Fileupload_controller_import;
