/*
 * webEdition CMS
 *
 * webEdition CMS
 * $Rev: 15093 $
 * $Author: mokraemer $
 * $Date: 2018-03-21 12:07:05 +0100 (Wed, 21 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/* global _EditorFrame, WE, CodeMirror */
'use strict';

var srcDoc = WE().util.getDynamicVar(document, 'loadVarSrcTmpl', 'data-doc');
WE().util.loadConsts(document, "tagWizzard");

var editor = null;
var weIsTextEditor = true;
var reloadContent = false;
var lastPos = null, lastQuery = null, marked = [];
var countJEditorInitAttempts = 0;
var wizardHeight = {
	open: 305,
	closed: 140
};

function initCM() {
	window.orignalTemplateContent = document.getElementById("editarea").value.replace(/\r/g, ""); //this is our reference of the original content to compare with current content
	var CMoptions = srcDoc.CMOptions;
	CMoptions.closeCharacters = new RegExp(CMoptions.closeCharacters);
	CMoptions.extraKeys = {
		Space: function (cm) {
			if (CMoptions.hasCodeCompletion) {
				CodeMirror.weHint(cm, ' ');
			} else {
				return CodeMirror.Pass;
			}
		},
		/*Tab: (cm) => {
		 if (cm.somethingSelected()) {
		 cm.indentSelection("add");
		 return;
		 }

		 if (this.config.indentWithTabs)
		 cm.replaceSelection("\t", "end", "+input");
		 else
		 cm.execCommand("insertSoftTab");
		 },
		 "Shift-Tab": (cm) => {
		 cm.indentSelection("subtract");
		 }*/


		Tab: function (cm) {
			if (cm.somethingSelected()) {
				cm.indentSelection("add");
				return;
			}
			if (!CMoptions.indentWithTabs) {
				/*var spaces = [cm.getOption("indentUnit") + 1].join(" ");
				 cm.replaceSelection(spaces);*/
				cm.execCommand("insertSoftTab");
			} else {
				return CodeMirror.Pass;
			}
		},
		"Shift-Tab": function (cm) {
			cm.indentSelection("subtract");
		},
		"'<'": function (cm) {
			if (CMoptions.hasCodeCompletion) {
				CodeMirror.weHint(cm, '<');
			} else {
				return CodeMirror.Pass;
			}
		},
		"Ctrl-Space": function (cm) {
			if (CMoptions.hasCodeCompletion) {
				CodeMirror.weHint(cm, '');
			} else {
				return CodeMirror.Pass;
			}
		}
	};
	try {
		var hlLine;
		document.getElementById("bodydiv").style.display = "block";
		editor = CodeMirror.fromTextArea(document.getElementById("editarea"), CMoptions);
		sizeEditor();
		if (srcDoc.editorHighlightCurrentLine) {
			hlLine = editor.addLineClass(0, "background", "activeline");
			//highlight current line
			editor.on("cursorActivity", function () {
				var cur = editor.getLineHandle(editor.getCursor().line);
				if (cur != hlLine) {
					editor.removeLineClass(hlLine, "background", "activeline");
					hlLine = editor.addLineClass(cur, "background", "activeline");
				}
			});
		} else { //FIX for CM which doesn't display lines beyond 27 if this line is missing....                                     ?>
			hlLine = editor.addLineClass(0, "background", "");

		}
		editor.on("change", function () {
			//this wil save content from CodeMirror2 to our original <textarea>.
			var currentTemplateCode = editor.getValue().replace(/\r/g, "\n");
			if (window.orignalTemplateContent !== currentTemplateCode) {
				document.getElementById("editarea").value = currentTemplateCode;
				_EditorFrame.setEditorIsHot(true);
			} else {
				document.getElementById("editarea").value = currentTemplateCode;
				_EditorFrame.setEditorIsHot(false);
			}
		});

	} catch (e) {
	}
}

function initDefaultEdior() {
	window.orignalTemplateContent = document.getElementById("editarea").value.replace(/\r/g, ""); //this is our reference of the original content to compare with current content
	document.getElementById("bodydiv").style.display = "block";
	sizeEditor();
	window.setTimeout(scrollToPosition, 50);
}

function sizeEditor() { // to be fixed (on 12.12.11)
	if (window.editor && window.editor.frame && window.editor.frame.nextSibling !== undefined) {
		document.getElementById("reindentButton").style.marginRight = (window.editor.frame.nextSibling.offsetWidth - 3) + "px";
	}
	var srtable = document.getElementById("srtable");
	if (srtable) {
		var wizardTable = document.getElementById("weTMPLDocEdit");
		var editorDiv = document.getElementById("editorDiv");
		var editarea = document.getElementById("editarea");
		var cm = document.getElementsByClassName("CodeMirror");

		editorDiv.style.bottom = (wizardTable ? wizardTable.offsetHeight : 0) + "px";
		editarea.style.height = srtable.offsetTop + "px";
		if (cm && cm.length) {
			cm[0].style.height = srtable.offsetTop + "px";
		}

		window.scroll(0, 0);
	}
}

// ################## Textarea specific functions #############

function getScrollPosTop() {
	var elem = document.getElementById("editarea");
	return (elem ? elem.scrollTop : 0);
}

function getScrollPosLeft() {
	var elem = document.getElementById("editarea");
	return  (elem ? elem.scrollLeft : 0);
}

function scrollToPosition() {
	var elem = document.getElementById("editarea");
	if (elem) {
		elem.scrollTop = window.parent.editorScrollPosTop;
		elem.scrollLeft = window.parent.editorScrollPosLeft;
	}
}

function wedoKeyDown(ta, ev) {
	var modifiers = (ev.altKey || ev.ctrlKey || ev.shiftKey);
	if (!modifiers && ev.keycode === 9) { // TAB
		if (ta.setSelectionRange) {
			var selectionStart = ta.selectionStart;
			var selectionEnd = ta.selectionEnd;
			ta.value = ta.value.substring(0, selectionStart) +
							"\t" +
							ta.value.substring(selectionEnd);
			ta.focus();
			ta.setSelectionRange(selectionEnd + 1, selectionEnd + 1);
			ta.focus();
			return false;
		}
		if (document.selection) {
			var selection = document.selection;
			var range = selection.createRange();
			range.text = "\t";
			return false;
		}
	}

	return true;
}
// ############ EDITOR PLUGIN ################

function setSource(source) {
	document.forms.we_form.elements['we_' + srcDoc.docName + '_txt[data]'].value = source;
	//Codemirror
	if (editor !== undefined && editor !== null && typeof editor === 'object') {
		editor.setValue(source);
	}
}

function getSource() {
	return document.forms.we_form.elements['we_' + srcDoc.docName + '_txt[data]'].value;
}

function getCharset() {
	return srcDoc.docCharSet;
}

// ############ CodeMirror Functions ################

function reindent() { // reindents code of CodeMirror2
	var start, end;
	if (editor.somethingSelected()) {
		start = editor.getCursor(true).line;
		end = editor.getCursor(false).line;
	} else {
		start = 0;
		end = editor.lineCount();
	}
	for (var i = start; i < end; ++i) {
		editor.indentLine(i, 'smart');
	}
}

function unmark() {
	for (var i = 0; i < marked.length; ++i) {
		marked[i].clear();
	}
	marked.length = 0;
}

function cmSearch(e) {
	if (e === null || e.keyCode === 13 || e.keyCode === 10) {
		search(document.getElementById("query").value, !document.getElementById("caseSens").checked);
	}
}

function cmReplace(event) {
	if (event === null || event.keyCode === 13 || event.keyCode === 10) {
		myReplace(document.getElementById("query").value, document.getElementById("replace").value, !document.getElementById("caseSens").checked);
	}
}

function search(text, caseIns) {
	unmark();
	if (!text) {
		return;
	}
	var cursor;
	for (cursor = editor.getSearchCursor(text, 0, caseIns); cursor.findNext(); ) {
		marked.push(editor.markText(cursor.from(), cursor.to(), {className: "searched"}));
	}
	if (lastQuery !== text) {
		lastPos = null;
	}
	cursor = editor.getSearchCursor(text, lastPos || editor.getCursor(), caseIns);
	if (!cursor.findNext()) {
		cursor = editor.getSearchCursor(text, 0, caseIns);
		if (!cursor.findNext()) {
			return;
		}
	}
	editor.setSelection(cursor.from(), cursor.to());
	marked.push(editor.markText(cursor.from(), cursor.to(), {className: "searchedHighlight"}));
	lastQuery = text;
	lastPos = cursor.to();
}

function myReplace(text, replaceby, caseIns) {
	if (!text) {
		return;
	}
	if (editor.getSelection() !== text) {
		search(text, caseIns);
	}
	if (editor.getSelection() !== text) {
		return;
	}
	editor.replaceSelection(replaceby);
	search(text, caseIns);
}

function executeEditButton() {
	if (document.getElementById('weTagGroupSelect').value === 'snippet_custom') {
		doAjax(document.getElementById('codesnippet_custom').value);

	} else if (document.getElementById('weTagGroupSelect').value === 'snippet_standard') {
		doAjax(document.getElementById('codesnippet_standard').value);

	} else {
		var _sel = document.getElementById('tagSelection');
		if (_sel.selectedIndex > -1) {
			edit_wetag(_sel.value);
		}
	}
}

function doAjax(value) {
	WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=snippetGetCode", 'we_cmd[1]=' + value, function (weResponse) {
		if (weResponse.DataArray.data) {
			document.getElementById('tag_edit_area').value = weResponse.DataArray.data;
		}
	});
}

function edit_wetag(tagname, insertAtCursor) {
	if (!insertAtCursor) {
		insertAtCursor = 0;
	}
	window.we_cmd("open_tag_wizzard", tagname, insertAtCursor);
}

function selectTagGroup(groupname) {
	switch (groupname) {
		case '-1':
			break;
		case "snippet_custom":
			document.getElementById('codesnippet_standard').style.display = 'none';
			document.getElementById('tagSelection').style.display = 'none';
			document.getElementById('codesnippet_custom').style.display = 'block';
			break;
		case "snippet_standard":
			document.getElementById('codesnippet_custom').style.display = 'none';
			document.getElementById('tagSelection').style.display = 'none';
			document.getElementById('codesnippet_standard').style.display = 'block';
			break;
		default:
			document.getElementById('codesnippet_custom').style.display = 'none';
			document.getElementById('codesnippet_standard').style.display = 'none';
			document.getElementById('tagSelection').style.display = 'block';
			var elem = document.getElementById("tagSelection");
			var i;
			for (i = (elem.options.length - 1); i >= 0; i--) {
				elem.options[i] = null;
			}

			for (i = 0; i < WE().consts.tagWizzard.groups[groupname].length; i++) {
				elem.options[i] = new Option(WE().consts.tagWizzard.groups[groupname][i], WE().consts.tagWizzard.groups[groupname][i]);
			}
	}
}

function openTagWizWithReturn(Ereignis) {
	if (!Ereignis) {
		Ereignis = window.event;
	}
	var keycode;
	if (Ereignis.which) {
		keycode = Ereignis.which;
	} else if (Ereignis.keyCode) {
		keycode = Ereignis.keyCode;
	}
	if (keycode === 13) {
		edit_wetag(document.getElementById("tagSelection").value);
	}
	//return false;
}

function openTagWizardPrompt(wrongTag) {
	var prompttext = WE().consts.g_l.weTagWizard.insert_tagname;
	if (wrongTag) {
		prompttext = WE().consts.g_l.weTagWizard.insert_tagname_not_exist.replace(/_wrongTag/, wrongTag) + prompttext;
	}

	var tagName = window.prompt(prompttext);
	var tagExists = false;

	if (typeof (tagName) === "string") {
		for (var i = 0; i < WE().consts.tagWizzard.groups.alltags.length && !tagExists; i++) {
			if (WE().consts.tagWizzard.groups.alltags[i] === tagName) {
				tagExists = true;
			}
		}

		if (tagExists) {
			edit_wetag(tagName, 1);
		} else {
			openTagWizardPrompt(tagName);

		}
	}
}

function insertAtStart(tagText) {
	if (window.editor && window.editor.frame) {
		window.editor.insertIntoLine(window.editor.firstLine(), 0, tagText + "\n");
	} else {
		document.we_form["we_" + srcDoc.docName + "_txt[data]"].value = tagText + "\n" + document.we_form["we_" + srcDoc.docName + "_txt[data]"].value;
	}
	_EditorFrame.setEditorIsHot(true);
}

function insertAtEnd(tagText) {
	if (window.editor && window.editor.frame) {
		window.editor.insertIntoLine(window.editor.lastLine(), "end", "\n" + tagText);
	} else {
		document.we_form["we_" + srcDoc.docName + "_txt[data]"].value += "\n" + tagText;
	}
	_EditorFrame.setEditorIsHot(true);
}

function addCursorPosition(tagText) {
	if (window.editor && window.editor.frame) {
		window.editor.replaceSelection(tagText);
		return;
	}
	var weForm = document.we_form["we_" + srcDoc.docName + "_txt[data]"],
					intStart, intEnd;
	if (document.selection) {
		weForm.focus();
		document.selection.createRange().text = tagText;
		document.selection.createRange().select();
	} else if (weForm.selectionStart || weForm.selectionStart == "0") {
		intStart = weForm.selectionStart;
		intEnd = weForm.selectionEnd;
		weForm.value = (weForm.value).substring(0, intStart) + tagText + (weForm.value).substring(intEnd, weForm.value.length);
		window.setTimeout(scrollToPosition, 50);
		weForm.focus();
		weForm.selectionStart = parseInt(intStart) + tagText.length;
		weForm.selectionEnd = parseInt(intStart) + tagText.length;
	} else {
		weForm.value += tagText;
	}
}

function refreshContentCompare() {
	window.orignalTemplateContent = document.getElementById("editarea").value.replace(/\r/g, "\n");
}

function editorChanged() {
	_EditorFrame.setEditorIsHot(window.orignalTemplateContent !== document.getElementById("editarea").value.replace(/\r/g, "\n"));
}