<?php

/**
 * webEdition CMS
 *
 * $Rev: 15468 $
 * $Author: mokraemer $
 * $Date: 2018-07-03 12:39:40 +0200 (Tue, 03 Jul 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class liveUpdateResponse{
	var $Type; //type of response
	var $Headline; //for template-type
	var $Content; //for template-type
	var $Header; //for template-type
	var $Code; //for eval-type
	var $EncodedCode;
	var $Encoding = false;

	protected function getErrorMsg(){
		$msg = '<div class="errorDiv">' .
			$this->Error['headline'] . '<br />' .
			$this->Error['message'] .
			($GLOBALS['liveUpdateError']['errorString'] ? $this->Error['lang']['errorMessage'] . ': <code class="errorText">' . $GLOBALS['liveUpdateError']['errorString'] . '</code><br />' .
			$this->Error['lang']['errorIn'] . ': <code class="errorText">' . $GLOBALS['liveUpdateError']['errorFile'] . '</code><br />' .
			$this->Error['lang']['errorLine'] . ': <code class="errorText">' . $GLOBALS['liveUpdateError']['errorLine'] . '</code>' : '') .
			'</div>';

		return '<script><!--
	top.frames.updatecontent.appendMessageLog("' . strtr($msg, ['"' => "'"]) . '");
	alert("' . strtr(strip_tags($msg), ['"' => "'"]) . '");
//-->
</script>';
	}

	protected function getProgress($msg, array $urls){
		return '<script>
			top.frames.updatecontent.decreaseSpeed = false;
			top.frames.updatecontent.nextUrl = "' . $urls['nextUrl'] . '"+(top.frames.updatecontent.param?top.frames.updatecontent.param:"");
			top.frames.updatecontent.setProgressBar("' . $urls['progress'] . '");
			var msg="' . str_replace(["\r", "\n"], '', $msg . $urls['message']) . '\n";
			top.frames.updatecontent.appendMessageLog(msg);' .
			($urls['nextStep'] ? 'top.frames.updatecontent.finishLiInstallerStep("' . $urls['nextStep'][0] . '");
			top.frames.updatecontent.activateLiInstallerStep("' . $urls['nextStep'][1] . '");' : '') . '
			window.setTimeout("top.frames.updatecontent.proceedUrl();", 20);
		</script>';
	}

	function initByArray($respArray){
		foreach($respArray as $key => $value){
			$this->$key = $value;
		}

		if($this->Encoding && $this->EncodedCode){
			$this->Code = base64_decode($this->EncodedCode);
		}
	}

	/**
	 * init the object with the response from the update-server
	 *
	 * @param string $response
	 * @return boolean
	 */
	function initByHttpResponse($response){
		if(($respArr = liveUpdateResponse::responseToArray($response))){
			$this->initByArray($respArr);
			return true;
		}
		return false;
	}

	function isError(){
		return ($this->Type === 'state' && $this->State === 'error');
	}

	function getField($fieldname){
		return (isset($this->$fieldname) ? $this->$fieldname : '');
	}

	protected function checkfreequota(&$AnzMB){
		$kB = str_repeat('0', 1024);
		$MB = str_repeat($kB, 1024);

		if(!is_dir(LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'tmp')){
			mkdir(LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'tmp');
		}
		$testfilename = LIVEUPDATE_CLIENT_DOCUMENT_DIR . 'tmp/testquota.txt';

		if(file_exists($testfilename)){
			unlink($testfilename);
		}
		$allesOK = true;
		for($i = 1; $i <= $AnzMB; $i++){
			if(!file_put_contents($testfilename, $MB, FILE_APPEND)){
				$allesOK = false;
				break;
			}
		}
		$AnzMB = $i - 1;
		unlink($testfilename);
		return $allesOK;
	}

	protected function changesResponse(){
		global $liveUpdateFnc;
		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp';
		$liveUpdateFnc->deleteDir($filesDir);

		$testdiskquota = 100;
		$diskquotawarning = $this->checkfreequota($testdiskquota);

		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp';
		$liveUpdateFnc->deleteDir($filesDir);
		if(!$diskquotawarning){
			return strtr($this->getErrorMsg(), ['__QUOTA__' => $testdiskquota]);
		}

		$liveUpdateFnc->insertUpdateLogEntry($this->LogMsg, $this->TargetVersion, 0);
		return $this->getProgress('', $this->Next);
	}

	protected function executePatches(){
		global $liveUpdateFnc;

		$redo = liveUpdateFunctionsServer::weUpdaterDoUpdate((empty($_REQUEST['progress']) ? '' : $_REQUEST['progress']['what']), (empty($_REQUEST['progress']) ? [] : $_REQUEST['progress']));
		$allFiles = [];
		$liveUpdateFnc->getFilesOfDir($allFiles, LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp/patches/');

		$success = true;
		$message = '';

		foreach($allFiles as $file){
			$message .= basename($file) . '<br />';
			$success = $liveUpdateFnc->executePatch($file);
			if(!$success){
				$errorFile .= ' ' . basename($file);
			}
			unlink($file);
		}

		if($success){
			$message = '';
			if($allFiles){
				$message = '<div>' . $this->Message . '</div>';
			}

			if(is_array($redo)){
				$message .= '<div>Update ' . $redo['text'] . '</div>';
				unset($redo['text']);
				return '<script>top.frames.updatecontent.param="&' . http_build_query(['progress' => $redo]) . '";</script>' .
					$this->getProgress($message, $this->Repeat);
			}

			$delDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/includes/';
			if(is_file($delDir . "del.files") && method_exists($liveUpdateFnc, 'removeObsoleteFiles')){
				if(is_file($delDir . 'deleted.files')){
					unlink($delDir . 'deleted.files');
				}
				$liveUpdateFnc->removeObsoleteFiles($delDir);
			}
			if(method_exists($liveUpdateFnc, "removeDirOnlineInstaller")){
				$liveUpdateFnc->removeDirOnlineInstaller();
			}

			return $this->getProgress($message, $this->Next);
		}

		return strtr($this->getErrorMsg(), [
			'__HEAD__' => $errorFile,
			'__MSG__' => $GLOBALS["errorDetail"]
			]);
	}

	function responseToArray($response){
		$respArray = @unserialize(base64_decode($response));
		return (is_array($respArray) ? $respArray : false);
	}

	protected function saveFiles(){
		global $liveUpdateFnc;
		$successFiles = []; // successfully saved files

		foreach($this->Files as $path => $content){

			$testPath = ltrim($path, '/');
			$testPath = strpos($testPath, 'tmp/files') === 0 ? '/' . ltrim(str_replace('tmp/files', '', $testPath), '/') : false;

			if($liveUpdateFnc->isPhpFile($path)){
				$content = $liveUpdateFnc->preparePhpCode($content);
			}

			if(!$liveUpdateFnc->filePutContent(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $path, $content)){
				str_replace('__PATH__', $path, $this->Error['path']);
				return $this->getErrorMsg();
			}
			if($testPath && method_exists($liveUpdateFnc, 'checkMakeFileWritable') && !$liveUpdateFnc->checkMakeFileWritable($testPath)){
				str_replace('__PATH__', $testPath, $this->Error['write']);
				return $this->getErrorMsg();
			}
			$successFiles[] = $path;
		}

		if($this->Parts){//multipart files to merge
			$content = '';
			for($i = 0; $i <= $this->Parts['Count']; $i++){
				$content .= $liveUpdateFnc->getFileContent(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $this->Parts['Name'] . 'part' . $i);
				$liveUpdateFnc->deleteFile(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $this->Parts['Name'] . 'part' . $i);
			}

			if($liveUpdateFnc->isPhpFile($this->Parts['Name'])){
				$content = $liveUpdateFnc->preparePhpCode($content);
			}

			if($liveUpdateFnc->filePutContent(LIVEUPDATE_CLIENT_DOCUMENT_DIR . $this->Parts['Name'], $content)){
				$successFiles[] = $path;
			}
		}


		$message = $this->SuccessText . '<br/>';
		return $this->getProgress($message, $this->Next);
	}

	protected function dbUpdate(){
		global $liveUpdateFnc;
		$queryDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp/queries/';

		$allFiles = [];
		$liveUpdateFnc->getFilesOfDir($allFiles, $queryDir);
		sort($allFiles);

		$message = '';

		for($i = $this->From; $i < count($allFiles) && $i < $this->To; $i++){
			// execute queries in each file
			$fileName = basename($allFiles[$i]);
			if($liveUpdateFnc->executeQueriesInFiles($allFiles[$i])){
				$text = substr($fileName, -40);
				$message .= "<div>&hellip;$text</div>";
			}
			$msg = array_filter($liveUpdateFnc->getQueryLog());
			unset($msg['success']);
			foreach($msg as $key => $vals){
				$err = ($key === 'error' ? 1 : 2);
				$message .= "<div class='" . ($err ? 'errorDiv' : 'messageDiv') . "'>" . $this->Lang['notice'] . '<br />' . $fileName . ': ' . $this->Lang[$key] . '</div>';
				$liveUpdateFnc->insertQueryLogEntries($key, $fileName . ': ' . $this->Lang[$key] . implode(',', $vals), $err, $this->ClientVersion);
			}
			$liveUpdateFnc->clearQueryLog();
		}
		if(count($allFiles) > $this->To){ // continue with DB steps
			return $this->getProgress($message, $this->Repeat);
		}// proceed to next step.
		return $this->getProgress($message, $this->Next);
	}

	protected function copyFiles(){
		global $liveUpdateFnc;
		$filesDir = LIVEUPDATE_CLIENT_DOCUMENT_DIR . '/tmp/files/';
		$preLength = strlen($filesDir);

		$success = true;

		$allFiles = [];
		$liveUpdateFnc->getFilesOfDir($allFiles, $filesDir);

		$donotcopy = [
			LIVEUPDATE_SOFTWARE_DIR . '/webEdition/we/include/we_hook/custom_hooks/weCustomHook_delete.inc.php',
			LIVEUPDATE_SOFTWARE_DIR . '/webEdition/we/include/we_hook/custom_hooks/weCustomHook_publish.inc.php',
			LIVEUPDATE_SOFTWARE_DIR . '/webEdition/we/include/we_hook/custom_hooks/weCustomHook_save.inc.php',
			LIVEUPDATE_SOFTWARE_DIR . '/webEdition/we/include/we_hook/custom_hooks/weCustomHook_unpublish.inc.php',
			LIVEUPDATE_SOFTWARE_DIR . '/webEdition/apps/toolfactory/hook/custom_hooks/weCustomHook_toolfactory_save.inc.php'
			];

		foreach($allFiles as $file){
			//$text = substr(basename($allFiles[$i]), -40);
			//$message .= "<div>&hellip;$text</div>";

			$success = (in_array(LIVEUPDATE_SOFTWARE_DIR . substr($file, $preLength), $donotcopy) ?
				$liveUpdateFnc->deleteFile($file) :
				$liveUpdateFnc->moveFile($file, LIVEUPDATE_SOFTWARE_DIR . substr($file, $preLength)));
			if(!$success){
				return $this->getErrorMsg();
			}
		}

		//$message = "<div>" . $message . "</div>";
		$message = '<div>' . $this->Message . '</div>';

		return $this->getProgress($message, $this->Next);
	}

	function getOutput(){
		if(!class_exists('liveUpdateFunctionsServer', false)){
			class_alias('liveUpdateFunctions', 'liveUpdateFunctionsServer');
		}

		if(empty($GLOBALS['liveUpdateFnc'])){
			$GLOBALS['liveUpdateFnc'] = new liveUpdateFunctionsServer();
			set_error_handler(['liveUpdateFunctionsServer', 'liveUpdateErrorHandler']);
		}

		switch($this->Type){
			case 'template':
				return liveUpdateTemplates::getHtml($this->Headline, $this->Content, $this->Header);
			case 'ChangesResponse':
				return $this->changesResponse();
			case 'SaveFiles':
				return $this->saveFiles();
			case 'DBUpdate':
				return $this->dbUpdate();
			case 'CopyFiles':
				return $this->copyFiles();
			case 'ExecutePatches':
				return $this->executePatches();
			case 'eval':
				t_e($this->Code);
				return eval('?>' . $this->Code);
			case 'state':
				return liveUpdateFrames::htmlStateMessage();
			default:
				t_e($this->Type . ' is not implemented yet', $this);
				return $this->Type . ' is not implemented yet';
		}
	}

}
