<?php

/**
 * webEdition CMS
 *
 * $Rev: 15889 $
 * $Author: mokraemer $
 * $Date: 2018-10-08 15:31:17 +0200 (Mon, 08 Oct 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_backup_wizard{
	const BACKUP = 'make_backup';
	const RECOVER = 'recover_backup';

	private $mode;
	private $frameset;
	private $fileUploader = null;
	private $json = [];

	function __construct($mode = self::BACKUP){
		if(we_base_request::_(we_base_request::INT, 'step', 0) < 2){
			//make sure we reset these vars on opening of this dialog
			unset($_SESSION['weS']['weBackupVars']);
		}
		$this->mode = $mode;
		$this->frameset = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . $mode;
		$this->json['mode'] = $mode;
		$this->json['modeCmd'] = $mode;
	}

	private function getJSDep(){
		$this->json['import_from'] = we_base_request::_(we_base_request::STRING, 'import_from');

		return we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard', 'data-backup' => setDynamicVar($this->json)]);
	}

	private function getOptions(){
		return array_filter([
			10 => 'handle_core',
			11 => we_modules_info::isActive(we_modules_info::OBJECT) ? 'handle_object' : '',
			12 => 'handle_versions',
			13 => 'handle_versions_binarys',
			14 => 'handle_binary',
			20 => 'handle_user',
			25 => we_modules_info::isActive(we_modules_info::CUSTOMER) ? 'handle_customer' : '',
			30 => we_modules_info::isActive(we_modules_info::SHOP) ? 'handle_shop' : '',
			35 => we_modules_info::isActive(we_modules_info::WORKFLOW) ? 'handle_workflow' : '',
			45 => we_modules_info::isActive(we_modules_info::NEWSLETTER) ? 'handle_newsletter' : '',
			50 => we_modules_info::isActive(we_modules_info::BANNER) ? 'handle_banner' : '',
			55 => we_modules_info::isActive(we_modules_info::SCHEDULER) ? 'handle_schedule' : '',
			60 => we_modules_info::isActive(we_modules_info::EXPORT) ? 'handle_export' : '',
			65 => we_modules_info::isActive(we_modules_info::VOTING) ? 'handle_voting' : '',
			75 => we_modules_info::isActive(we_modules_info::GLOSSARY) ? 'handle_glossary' : '',
			98 => 'handle_hooks',
			99 => 'handle_customtags',
			100 => 'handle_settings',
			101 => 'handle_temporary',
			102 => 'handle_history',
			300 => 'handle_extern',
			400 => 'handle_weSearch'
		]);
	}

	function getHTMLFrameset(){
		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_' . ($this->mode === self::BACKUP ? 'backup' : 'recover') . '_title]'), '', '', '', we_html_element::body([
					'id' => 'weMainBody']
					, we_html_element::iframe('edbody', $this->frameset . '&pnt=edbody', 'position:absolute;top:0;bottom:40px;left:0;right:0;', 'border:0;width:100%;height:100%;') .
					we_html_element::iframe('busy', $this->frameset, 'position:absolute;height:40px;bottom:0;left:0;right:0;overflow: hidden', '', '', false) .
					we_html_element::iframe('cmd', $this->frameset . '&pnt=cmd')
				)
		);
	}

	function getHTMLStep($step){
		switch($this->mode){
			default:
			case self::BACKUP:
				switch($step){
					default:
					case 1:
						return $this->getHTMLBackupStep1();
					case 2:
						return $this->getHTMLBackupStep2();
					case 3:
						return $this->getHTMLBackupStep3();
				}
			case self::RECOVER:
				switch($step){
					default:
					case 1:
						return $this->getHTMLRecoverStep1();
					case 2:
						return $this->getHTMLRecoverStep2();
					case 3:
						return $this->getHTMLRecoverStep3();
					case 4:
						return $this->getHTMLRecoverStep4();
				}
		}
	}

	function getHTMLRecoverStep1(){
		$parts = [['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[save_before]'), we_html_tools::TYPE_ALERT, 600), 'noline' => true],
			['headline' => '', 'html' => g_l('backup', '[save_question]'), 'noline' => true],
		];


		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
					'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(1)"], we_html_element::form([
						'name' => 'we_form', 'method' => 'post'], we_html_multiIconBox::getHTML('backup_options', $parts, true, '', -1, '', '', false, g_l('backup', '[step1]'))
					)
				)
		);
	}

	function getHTMLRecoverStep2(){
		$parts = [
			['headline' => '', 'html' => we_html_forms::radiobutton('import_server', true, 'import_from', g_l('backup', '[import][from_server]')), 'noline' => true],
			['headline' => '', 'html' => we_html_forms::radiobutton('import_upload', false, 'import_from', g_l('backup', '[import][from_local]')), 'noline' => true]
		];

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
					'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(2);"], we_html_element::form([
						'name' => 'we_form', 'method' => 'post'], we_html_element::hiddens(['pnt' => 'edbody', 'step' => 3]) .
						we_html_multiIconBox::getHTML('backup_options', $parts, true, '', -1, '', '', false, g_l('backup', '[step2]'))
					)
				)
		);
	}

	private function getHTMLRecoverStep3Import_upload(){
		$fileUploaderHead = $this->fileUploader->getCss() . $this->fileUploader->getJs();
		$inputTypeFile = $this->fileUploader->getHTML();

		return [['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[charset_warning]'), we_html_tools::TYPE_ALERT, 600, false), 'noline' => true],
			(DEFAULT_CHARSET ? null : ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[defaultcharset_warning]'), we_html_tools::TYPE_ALERT, 600, false),
			'noline' => true]),
			//['headline' => '', 'html' => $this->fileUploader->getHtmlAlertBoxes(), 'noline' => true],
			['headline' => '', 'html' => $inputTypeFile, 'noline' => true]
		];
	}

	private function getHTMLRecoverStep3Import_server(){
		$select = new we_html_select(['name' => 'backup_select', 'size' => 7, 'style' => "width: 600px;"]);
		$files = [];
		$extra_files = [];
		$dateformat = g_l('date', '[format][default]');
		for($i = 0; $i <= 1; $i++){
			$adddatadir = ($i == 0 ? '' : 'data/');
			$dstr = BACKUP_PATH . $adddatadir;
			$d = dir($dstr);
			while(($entry = $d->read())){
				switch($entry){
					case '.':
					case '..':
					case 'download':
					case 'tmp':
					case 'lastlog.php':
					case '.htaccess':
						continue 2;
					default:
						if(@is_dir($dstr . $entry)){
							continue 2;
						}
				}

				$filename = $dstr . $entry;
				$filesize = we_base_file::getHumanFileSize(filesize($filename));
				$filedate = date($dateformat, filemtime($filename));
				if(strpos($entry, 'weBackup_') !== 0){
					$extra_files[$adddatadir . $entry] = $entry . " $filedate $filesize";
					continue;
				}
				$ts = str_replace(['.php', '.xml', '.gz', '.bz', '.zip'], '', preg_replace('|^weBackup_|', '', $entry));

				if(is_numeric($ts) && !($ts < 1004569200)){//old Backup
					$comp = we_base_file::getCompression($entry);
					$files[$adddatadir . $entry] = /* g_l('backup', '[backup_form]') . ' ' . */ date($dateformat, $ts) . ($comp && $comp != "none" ? " ($comp)" : '') . ' ' . $filesize;
					continue;
				}

				if(substr_count($ts, '_') > 5){
					$matches = [];
					if(preg_match('|([^_]*)_(\d{4})_(\d{1,2})_(\d{1,2})__(\d{1,2})_(\d{1,2})_?([\d-]*)|', $ts, $matches)){
						list(, $url, $year, $month, $day, $hour, $min, $wever) = $matches;
						$filedate = date($dateformat, mktime($hour, $min, 0, $month, $day, $year));
					} else {
						$url = $wever = '';
					}
					$comp = we_base_file::getCompression($entry);
					$files[$adddatadir . $entry] = /* g_l('backup', '[backup_form]') . ' ' . */ $filedate . ($url ? ' - ' . $url : '') . ($wever ? ' (WE: ' . strtr($wever, ['-' => '.']) . ')' : '') . ($comp && $comp != 'none' ? ' (' . $comp . ')' : '') . " " . $filesize;
					continue;
				}

				$extra_files[$adddatadir . $entry] = $entry . " $filedate $filesize";
			}
		}
		$d->close();

		krsort($files);
		asort($extra_files);
		$i = 0;

		/* foreach($files as $fk=>$fv)	$select->addOption($fk,$fv); */

		$default = we_html_select::getNewOptionGroup(['class' => 'bold', 'style' => 'font-style: normal; color: darkblue;', 'label' => g_l('backup', '[we_backups]')]);
		$other = we_html_select::getNewOptionGroup(['class' => 'bold', 'style' => 'font-style: normal; color: darkblue;', 'label' => g_l('backup', '[other_files]')]);

		foreach($files as $fk => $fv){
			if(strlen($fv) > 75){
				$fv = addslashes(substr($fv, 0, 65) . '...' . substr($fv, -10));
			}
			$default->addChild(we_html_select::getNewOption($fk, $fv));
		}
		foreach($extra_files as $fk => $fv){
			if(strlen($fv) > 75){
				$fv = addslashes(substr($fv, 0, 65) . '...' . substr($fv, -10));
			}
			$other->addChild(we_html_select::getNewOption($fk, $fv));
		}

		$select->addChild($default);
		$select->addChild($other);

		$this->json['extra_files'] = [];
		$this->json['extra_files_desc'] = [];
		foreach($extra_files as $fk => $fv){
			$this->json['extra_files'][] = $fk;
			$this->json['extra_files_desc'][] = $fv;
		}

		return [
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[charset_warning]'), we_html_tools::TYPE_ALERT, 600, false), 'noline' => true],
			(DEFAULT_CHARSET ? null : ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[defaultcharset_warning]'), we_html_tools::TYPE_ALERT, 600, false),
			'noline' => true] ),
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[old_backups_warning]'), we_html_tools::TYPE_ALERT, 600, false), 'noline' => true],
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[select_server_file]'), we_html_tools::TYPE_INFO, 600, false), 'noline' => true],
			['headline' => '', 'html' => $select->getHtml(), 'noline' => true],
			//array("headline"=>"","html"=>we_html_forms::checkbox(1, false, "show_all", g_l('backup',"[show_all]"), false, "defaultfont", "showAll()"),"space"=>0,"noline"=>1);
			['headline' => '', 'html' => we_html_button::create_button('delete_backup', "javascript:delSelected();", '', '', false, false),]
		];
	}

	private function getHTMLRecoverStep3(){
		if(isset($_SESSION['weS']['weBackupVars'])){
			// workaround for php bug #18071
			// bug: session has been restarted
			$_SESSION['weS']['weBackupVars'] = [];
			// workaround end
			unset($_SESSION['weS']['weBackupVars']);
		}

		$this->fileUploader = new we_fileupload_ui_base('we_upload_file', [], ['.xml', '.tgz', '.gz']);
		$this->fileUploader->setGenericFileName(TEMP_DIR . we_fileupload::REPLACE_BY_FILENAME);
		$this->fileUploader->setGenericFileName(BACKUP_DIR . 'data/' . we_fileupload::REPLACE_BY_FILENAME);
		$this->fileUploader->setShowFilenamesFull(true);

		$maxsize = we_fileupload::getMaxUploadSizeB();
		$importUpload = (we_base_request::_(we_base_request::STRING, 'import_from') === 'import_upload');

		$parts = ($importUpload ?
			$this->getHTMLRecoverStep3Import_upload() :
			$this->getHTMLRecoverStep3Import_server());

		$parts[] = ['headline' => '', 'html' => we_html_forms::checkbox(1, true, 'rebuild', g_l('backup', '[rebuild]'), false),];

		$parts[] = ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[delold_notice]', false, true), we_html_tools::TYPE_QUESTION, 600, false) .
			we_html_button::create_button('delold', "javascript:delOldFiles();", '', '', false, false), 'noline' => true];


		$parts[] = ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[import][options]'), we_html_tools::TYPE_INFO, 600, false), 'space' => we_html_multiIconBox::SPACE_SMALL,
			'noline' => true];

		$form_properties = [310 => 'convert_charset',] + $this->getOptions();
		ksort($form_properties, SORT_NUMERIC);
		$this->json['form_properties'] = $form_properties;

		foreach($form_properties as $k => $v){
			switch($v){
				case 'convert_charset':
				case 'extern_exp':
				case 'backup_log':
					continue 2;
				case 'handle_extern':
					continue 2;
				//$checked = false;
				default:
					$checked = true;
			}
			//$help = g_l('backup', '[' . str_replace('handle_', '', $v) . '_info]', true);
			$parts[] = ['headline' => '', 'html' => we_html_forms::checkbox(1, $checked, $v, g_l('backup', '[' . str_replace('handle_', 'import][', $v) . '_data]'), false, '', "doClick($k);"),
				'space' => we_html_multiIconBox::SPACE_SMALL, 'noline' => true];
		}

		$parts[] = ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[extern_info]'), we_html_tools::TYPE_ALERT, 600, false), 'space' => we_html_multiIconBox::SPACE_SMALL,
			'noline' => true];
		$parts[] = ['headline' => '', 'html' => we_html_forms::checkbox(1, false, 'handle_extern', g_l('backup', '[import][extern_data]'), false, '', "doClick(300);"),
			'space' => we_html_multiIconBox::SPACE_SMALL, 'noline' => true];

		$parts[] = ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[convert_charset]'), we_html_tools::TYPE_ALERT, 600, false), 'space' => we_html_multiIconBox::SPACE_SMALL,
			'noline' => true];
		$parts[] = ['headline' => '', 'html' => we_html_forms::checkbox(1, false, 'convert_charset', g_l('backup', '[convert_charset_data]'), false, '', "doClick(310);doUnCheck(101);doUnCheck(100);doUnCheck(70)"),
			'space' => we_html_multiIconBox::SPACE_SMALL, 'noline' => true];

		$parts[] = ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[backup_log_exp]'), we_html_tools::TYPE_INFO, 600, false), 'space' => we_html_multiIconBox::SPACE_SMALL,
			'noline' => true];
		$parts[] = ['headline' => '', 'html' => we_html_forms::checkbox(1, true, "backup_log", g_l('backup', '[export][backup_log]'), false, '', "doClick(320);"),
			'space' => we_html_multiIconBox::SPACE_SMALL, 'noline' => true];


		$form_attribs = (we_base_request::_(we_base_request::STRING, "import_from") === "import_upload" ?
			['name' => 'we_form', 'method' => 'post', 'action' => $this->frameset, 'target' => 'cmd', 'enctype' => 'multipart/form-data'] :
			['name' => 'we_form', 'method' => 'post', 'action' => $this->frameset, 'target' => 'cmd']
			);

		$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(3);self.focus();"], we_html_element::form($form_attribs, we_html_element::hiddens([
						'pnt' => 'cmd',
						'cmd' => 'import',
						'step' => 3,
						'MAX_FILE_SIZE' => $maxsize]) .
					we_html_element::hidden('operation_mode', 'import') .
					we_html_element::jsScript(JS_DIR . 'multiIconBox.js') .
					we_html_multiIconBox::getHTML('backup_options', $parts, true, '', 7, g_l('backup', '[recover_option]'), "<b>" . g_l('backup', '[recover_option]') . "</b>", false, g_l('backup', '[step3]'))
				)
		);

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
					'data-backup' => setDynamicVar($this->json)]) . ($importUpload ? $this->fileUploader->getCss() . $this->fileUploader->getJs() : '') .
				$this->getJSDep(), $body);
	}

	function getHTMLRecoverStep4(){

		if(isset($_SESSION['weS']['weBackupVars'])){
			// workaround for php bug #18071
			// bug: session has been restarted
			$_SESSION['weS']['weBackupVars'] = [];
			// workaround end
			unset($_SESSION['weS']['weBackupVars']);
		}

		$parts = [
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[finished_success]'), we_html_tools::TYPE_INFO, 600), 'noline' => true],
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[old_backups_warning]'), we_html_tools::TYPE_ALERT, 600, false), 'noline' => true]
		];

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
					'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => "stopBusy()"], we_html_element::form([
						'name' => 'we_form', 'method' => 'post', 'enctype' => 'multipart/form-data'], we_html_multiIconBox::getHTML('backup_options', $parts, true, '', -1, '', '', false, g_l('backup', '[step3]'))
					)
				)
		);
	}

	function getHTMLBackupStep1(){
		if(isset($_SESSION['weS']['weBackupVars'])){
			// workaround for php bug #18071
			// bug: session has been restarted
			$_SESSION['weS']['weBackupVars'] = [];
			// workaround end
			unset($_SESSION['weS']['weBackupVars']);
		}

		$form_properties = [
			1 => 'export_server',
			2 => 'export_send',
			320 => 'backup_log',
			] + $this->getOptions();

		ksort($form_properties, SORT_NUMERIC);

		$compression = we_base_file::hasCompression('gzip');

		$parts = [
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', ($compression ? '[filename_compression]' : '[filename_info]'), false, true), we_html_tools::TYPE_INFO, 600, false),
				'noline' => true],
			['headline' => g_l('backup', '[filename]') . ': ', 'html' => we_html_tools::textInput('filename', 'weBackup_' . str_replace('.', '-', $_SERVER['SERVER_NAME']) . '_' . date('Y_m_d__H_i', time()) . '_' . str_replace('.', '-', WE_VERSION) . '.xml', '', [
					], 'text', '30em'),
				'rightHeadline' => true, 'noline' => true],
			($compression ?
			['headline' => '', 'html' => we_html_forms::checkbox(we_backup_util::COMPRESSION, true, "compress", g_l('backup', '[compress]'), false, '', '', false, g_l('backup', '[ftp_hint]'), we_html_tools::TYPE_ALERT, 600),
			] : []),
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[protect_txt]'), we_html_tools::TYPE_INFO, 600, false), 'noline' => true],
			['headline' => '', 'html' => we_html_forms::checkbox(1, false, "protect", g_l('backup', '[protect]'), false, '', '')],
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[export][location]'), we_html_tools::TYPE_INFO, 600, false), 'noline' => true],
			['headline' => '', 'html' => we_html_forms::checkbox(1, true, "export_server", g_l('backup', '[export][location_server]'), false, '', "doClick(1)"),
				'noline' => true],
			['headline' => '', 'html' => we_html_forms::checkbox(1, false, "export_send", g_l('backup', '[export][location_send]'), false, '', "doClick(2)", (!we_base_permission::hasPerm(we_base_perm::EXIM_EXPORT))),
			],
			['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('backup', '[export][options]'), we_html_tools::TYPE_INFO, 600, false), 'noline' => true]
		];

		$this->json['form_properties'] = $form_properties;
		foreach($form_properties as $k => $v){
			switch($v){
				case 'export_server':
				case 'export_send':
				case 'backup_log':
					continue 2;
				case 'handle_versions_binarys':
				case 'handle_extern':
					$checked = false;
					break;
				default:
					$checked = true;
					break;
			}
			$help = g_l('backup', '[' . str_replace('handle_', '', $v) . '_info]', true);
			$parts[] = [
				'headline' => '',
				'html' => we_html_forms::checkbox(1, $checked, $v, g_l('backup', '[' . str_replace('handle_', 'export][', $v) . '_data]'), false, '', "doClick(" . $k . ");") . ($help ? we_html_tools::alertAttentionBox($help, we_html_tools::TYPE_HELP, 0, false) : ''),
				'noline' => true
			];
		}

		$parts[] = ['headline' => '',
			'html' => we_html_forms::checkbox(1, true, 'backup_log', g_l('backup', '[export][backup_log]'), false, '', "doClick(320);") . we_html_tools::alertAttentionBox(g_l('backup', '[backup_log_exp]'), we_html_tools::TYPE_HELP, 600, false),
			'noline' => true];


		$js = $this->getJSDep();

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title_export]'), '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
					'data-backup' => setDynamicVar($this->json)]) . $js, we_html_element::body(['class' => 'weDialogBody', 'onload' => "startStep(1)"], we_html_element::form([
						'name' => 'we_form', 'method' => 'post', 'onsubmit' => 'return false;'], we_html_element::hiddens(['pnt' => 'cmd',
							'cmd' => 'export',
							'operation_mode' => 'backup',
							'do_import_after_backup' => we_base_request::_(we_base_request::BOOL, 'do_import_after_backup')]) .
						we_html_element::jsScript(JS_DIR . 'multiIconBox.js') .
						we_html_multiIconBox::getHTML("backup_options1", $parts, true, '', ($compression ? 9 : 7), g_l('backup', '[option]'), "<b>" . g_l('backup', '[option]') . "</b>", false, g_l('backup', '[export][step1]'))
					)
				)
		);
	}

	function getHTMLBackupStep2(){
		if($_SESSION['weS']['weBackupVars']['options']['export2send']){
			$content = (is_file($_SESSION['weS']['weBackupVars']['backup_file']) ?
//Note: we show a link for external download - do we need this?
				we_html_element::div([], self::getDownloadLinkText() . '<br/><br/>' .
					we_html_element::a([
						'href' => WEBEDITION_DIR . 'showTempFile.php?' . http_build_query([
							'file' => str_replace(WEBEDITION_PATH, '', $_SESSION['weS']['weBackupVars']['backup_file']),
							'binary' => 1
						]),
						'download' => $_SESSION['weS']['weBackupVars']['filename']], g_l('backup', '[download_file]'))
				) :
				we_html_element::div([], g_l('backup', '[download_failed]'))
				);
		} else {
			$content = '';
		}

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title_export]'), '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js', '', ['id' => 'loadVarBackup_wizard',
					'data-backup' => setDynamicVar($this->json)]), we_html_element::body(['class' => 'weDialogBody', 'onload' => 'startStep(3,' . (!empty($_SESSION['weS']['weBackupVars']['options']['do_import_after_backup']) ? 1 : 0) . ');'], we_html_element::form([
						'name' => 'we_form', 'method' => 'post'], we_html_tools::dialogLayout(we_html_element::div(['style' => 'padding-bottom:20px;'], g_l('backup', '[finish]')) . $content, g_l('backup', '[export][step2]'))
					)
				)
		);
	}

	function getHTMLBackupStep3(){
		$ret = '';
		update_time_limit(0);
		if(we_base_request::_(we_base_request::BOOL, 'backupfile')){
			$filename = urldecode(we_base_request::_(we_base_request::RAW, 'backupfile'));

			if(file_exists($filename) && stripos($filename, BACKUP_PATH) !== false){ // Does file exist and does it saved in backup dir?
				$size = filesize($filename);

				header('Pragma: public');
				header('Expires: 0');
				header('Cache-control: private, max-age=0, must-revalidate');
				header('Content-Type: application/octet-stream');
				header('Content-Disposition: attachment; filename="' . trim(htmlentities(basename($filename))) . '"');
				header('Content-Description: ' . trim(htmlentities(basename($filename))));
				header('Content-Length: ' . $size);

				if(($filehandler = fopen($filename, 'rb'))){
					while(!feof($filehandler)){
						echo fread($filehandler, 8192);
						flush();
					}
					fclose($filehandler);
				} else {
					$ret = $this->build_error_message();
				}
			} else {
				$ret = $this->build_error_message();
			}
		} else {
			$ret = $this->build_error_message();
		}

		if(!empty($_SESSION['weS']['weBackupVars']['backup_file']) && empty($_SESSION['weS']['weBackupVars']['options']['export2server']) &&
			is_file($_SESSION['weS']['weBackupVars']['backup_file'])){
			we_base_file::insertIntoCleanUp($_SESSION['weS']['weBackupVars']['backup_file'], 0);
		}

		if(isset($_SESSION['weS']['weBackupVars'])){
			// workaround for php bug #18071
			// bug: session has been restarted
			$_SESSION['weS']['weBackupVars'] = [];
			// workaround end
			unset($_SESSION['weS']['weBackupVars']);
		}
		return $ret;
	}

	function build_error_message(){
		$error_message = new we_html_table(['class' => 'default'], 1, 1, [
			[g_l('backup', '[download_failed]')]]);

		return we_html_tools::getHtmlTop('', '', '', '', '<body class="weDialogBody">' . we_html_tools::dialogLayout($error_message->getHtml(), g_l('backup', '[export][step2]')));
	}

	function getHTMLExtern(){
		$txt = g_l('backup', '[extern_backup_question_' . we_base_request::_(we_base_request::STRING, 'w', 'exp') . ']');

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), '', '', '', we_html_element::body(['class' => 'weEditorBody', 'onblur' => "self.focus()",
					'onload' => "self.focus();"], we_html_element::form(['name' => 'we_form'], we_html_tools::yesNoCancelDialog($txt, we_html_element::span(['class' => "fa-stack fa-lg",
								'style' => "color:#F2F200;"], '<i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i>'), "ja", "nein", '', "self.close();", "top.opener.top.edbody.clearExtern();self.close();"))
				)
		);
	}

	function getHTMLBusy(){
		$head = $body = '';

		$table = new we_html_table(['class' => 'default', 'style' => 'width:100%;text-align:right'], 1, 3);

		if(we_base_request::_(we_base_request::STRING, 'operation_mode') === 'busy'){
			$text = we_base_request::_(we_base_request::STRING, 'current_description', g_l('backup', '[working]'));
			$progress = new we_gui_progressBar(we_base_request::_(we_base_request::INT, "percent", 0), 200);
			$progress->addText($text, we_gui_progressBar::TOP, 'current_description');
			$head .= we_gui_progressBar::getJSCode();
			$table->setCol(0, 0, ['style' => 'text-align:left;'], $progress->getHtml('', 'margin-left:15px'));
		}

		$step = we_base_request::_(we_base_request::INT, 'step', 0);

		switch($this->mode){
			case self::BACKUP:
				switch($step){
					case 1:
						$table->setColContent(0, 1, we_html_button::position_yes_no_cancel(we_html_button::create_button('make_backup', "javascript:doExport();"), null, we_html_button::create_button(we_html_button::CANCEL, 'javascript:top.close();')));
						break;
					case 2:
						$table->setColContent(0, 1, we_html_button::create_button(we_html_button::CANCEL, "javascript:top.close();"));
						break;
					case 3://recover_backup
						if(we_base_request::_(we_base_request::BOOL, 'do_import_after_backup')){
							$body = we_html_button::create_button(we_html_button::NEXT, "javascript:top.busy.recoverBackupAfterMaking();");
						} else if(!empty($_SESSION['weS']['inbackup'])){
							$body = we_html_button::create_button(we_html_button::NEXT, "javascript:top.opener.weiter();top.close();");
							unset($_SESSION['weS']['inbackup']);
						} else {
							$body = we_html_button::create_button(we_html_button::CLOSE, "javascript:top.close();");
						}
						$table->setColContent(0, 1, $body);
						break;
					default:
				}
				break;
			case self::RECOVER:
				switch($step){
					case 1:
						$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::YES, "javascript:pressYesStep1();"), we_html_button::create_button(we_html_button::NO, "javascript:top.edbody.location='" . $this->frameset . "&pnt=edbody&step=2';"), we_html_button::create_button(we_html_button::CANCEL, "javascript:top.close();"));
						$table->setColContent(0, 1, $buttons);
						break;
					case 2:
						$nextbuts = we_html_button::create_button(we_html_button::BACK, "javascript:top.edbody.location='" . $this->frameset . "&pnt=edbody&step=1'") .
							we_html_button::create_button(we_html_button::NEXT, "javascript:top.edbody.we_submitForm('edbody','" . $this->frameset . "');");

						$buttons = we_html_button::position_yes_no_cancel($nextbuts, null, we_html_button::create_button(we_html_button::CANCEL, "javascript:top.close();"));

						$table->setColContent(0, 1, $buttons);
						break;
					case 3:
						$startImportCall = 'top.edbody.startImport()';
						$cancelCall = 'top.close();';

						if(we_modules_info::isActive(we_modules_info::WORKFLOW)){
							$db = new DB_WE();
							$nextbut = (we_workflow_utility::getAllWorkflowDocs(FILE_TABLE) || (we_modules_info::isActive(we_modules_info::OBJECT) && we_workflow_utility::getAllWorkflowDocs(OBJECT_FILES_TABLE)) ?
								we_html_button::create_button('restore_backup', "javascript:if(window.confirm('" . g_l('modules_workflow', '[ask_before_recover]') . "')) " . $startImportCall . ";") :
								we_html_button::create_button('restore_backup', "javascript:" . $startImportCall));
						} else {
							$nextbut = we_html_button::create_button('restore_backup', "javascript:" . $startImportCall);
						}

						$nextprevbuts = we_html_button::create_button(we_html_button::BACK, "javascript:top.edbody.location='" . $this->frameset . "&pnt=edbody&step=2';") . $nextbut;
						$buttons = we_html_button::position_yes_no_cancel($nextprevbuts, null, we_html_button::create_button(we_html_button::CANCEL, "javascript:" . $cancelCall));

						$table->setColContent(0, 1, $buttons);
						break;
					case 4:
						$table->setColContent(0, 1, we_html_button::create_button(we_html_button::CANCEL, "javascript:top.close();"));
						break;
					case 5:
						$table->setColContent(0, 1, we_html_button::create_button(we_html_button::CLOSE, "javascript:top.close();"));
						break;
					default:
				}
				break;
		}

		return we_html_tools::getHtmlTop(g_l('backup', '[wizard_title]'), '', '', we_html_element::jsScript(JS_DIR . 'we_backup_busy.js', '', ['id' => 'loadVarBackup_wizard',
					'data-backup' => setDynamicVar($this->json)]) .
				$head, we_html_element::body(['class' => "editfooter"], $table->getHtml())
		);
	}

	function getHTMLCmd(we_base_jsCmd $jsCmd){
		switch(we_base_request::_(we_base_request::STRING, 'operation_mode', '-1')){
			case '-1':
				return;
			case 'rebuild':
				$jsCmd->addCmd('rebuild');
				return;
			case 'deleteall':
				$_SESSION['weS']['backup_delete'] = 1;
				$_SESSION['weS']['delete_files_nok'] = [];
				$_SESSION['weS']['delete_files_info'] = g_l('backup', '[files_not_deleted]');

				$jsCmd->addCmd('deleteall');
				return;
			case 'deletebackup':
				$bfile = we_base_request::_(we_base_request::FILE, "bfile");
				if(strpos($bfile, '..') === 0){
					$jsCmd->addMsg(g_l('backup', '[name_notok]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if(!is_writable(BACKUP_PATH . $bfile)){
					$jsCmd->addMsg(g_l('backup', '[error_delete]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				if(we_base_file::deleteLocalFile(BACKUP_PATH . $bfile)){
					$jsCmd->addCmd('deletebackup');
				} else {
					$jsCmd->addMsg(g_l('backup', '[error_delete]'), we_base_util::WE_MESSAGE_ERROR);
				}
				return;
			default:
				$jsCmd->addMsg(g_l('backup', '[error]'), we_base_util::WE_MESSAGE_ERROR);
				return;
		}
	}

	static function getDownloadLinkText(){
		switch(we_base_browserDetect::inst()->getBrowser()){
			case we_base_browserDetect::SAFARI:
			case we_base_browserDetect::APPLE:
				$out = g_l('browser', '[save_link_as_SAFARI]', false, true);
				break;
			case we_base_browserDetect::IE:
				$out = g_l('browser', '[save_link_as_IE]', false, true);
				break;
			case we_base_browserDetect::FF:
				$out = g_l('browser', '[save_link_as_FF]', false, true);
				break;
			case we_base_browserDetect::OPERA:
			default:
				$out = g_l('browser', '[save_link_as_DEFAULT]', false, true);
		}

		return nl2br(oldHtmlspecialchars(preg_replace('#<br\s*/?\s*>#i', "\n", $out)));
	}

	public static function showLog(){
		if(we_base_permission::hasPerm(we_base_perm::EXIM_BACKUPLOG)){
			$parts = [
				['headline' => g_l('backup', '[view_log]'),
					'rightHeadline' => true,
					'html' => (file_exists(BACKUP_PATH . we_backup_util::logFile) ?
					'<pre>' . file_get_contents(BACKUP_PATH . we_backup_util::logFile) . '</pre>' :
					'<p>' . g_l('backup', '[view_log_not_found]') . '</p>'),
					'space' => we_html_multiIconBox::SPACE_SMALL
				]
			];
		} else {
			$parts = [
				['headline' => g_l('backup', '[view_log]'),
					'rightHeadline' => true,
					'html' => '<p>' . g_l('backup', '[view_log_no_perm]') . '</p>',
					'space' => we_html_multiIconBox::SPACE_SMALL
				]
			];
		}
		$buttons = we_html_button::formatButtons(we_html_button::create_button(we_html_button::CLOSE, "javascript:self.close()"));
		echo we_html_tools::getHtmlTop(g_l('backup', '[view_log]'), '', '', we_html_element::jsScript(JS_DIR . 'closeEscape.js')
			, we_html_element::body(['class' => 'weDialogBody', 'onload' => "self.focus();"], we_html_element::div(['id' => 'info'], we_html_element::jsScript(JS_DIR . 'multiIconBox.js') .
					we_html_multiIconBox::getHTML('', $parts, true, $buttons)))
		);
	}

	public static function showBackupFrameset(){
		$weBackupWizard = new self(we_backup_wizard::BACKUP);

		switch($what = we_base_request::_(we_base_request::STRING, 'pnt', 'frameset')){
			case 'frameset':
				echo $weBackupWizard->getHTMLFrameset();
				break;
			case 'edbody':
				echo $weBackupWizard->getHTMLStep(we_base_request::_(we_base_request::INT, 'step', 1));
				break;
			case 'cmd':
				$jsCmd = new we_base_jsCmd();
				$weBackupWizard->getHTMLCmd($jsCmd);
				echo we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js') . $jsCmd->getCmds(), we_html_element::body());
				break;
			case 'busy':
				echo $weBackupWizard->getHTMLBusy();
				break;
			case 'extern':
				echo $weBackupWizard->getHTMLExtern();
				break;
			default:
				t_e(__FILE__ . ' unknown reference: ' . $what);
		}
	}

	public static function showRecoverFrameset(){
		$what = we_base_request::_(we_base_request::STRING, 'pnt', 'frameset');
		$step = we_base_request::_(we_base_request::INT, 'step', 1);
		$weBackupWizard = new self(we_backup_wizard::RECOVER);

		switch($what){
			case 'frameset':
				echo $weBackupWizard->getHTMLFrameset();
				break;
			case 'edbody':
				echo $weBackupWizard->getHTMLStep($step);
				break;
			case 'cmd':
				$jsCmd = new we_base_jsCmd();
				$weBackupWizard->getHTMLCmd($jsCmd);
				if(!$jsCmd->isEmpty()){
					echo we_html_tools::getHtmlTop('webEdition', '', '', we_html_element::jsScript(JS_DIR . 'backup_wizard.js') . $jsCmd->getCmds(), we_html_element::body());
					break;
				}
			//FIXME: no break??
			case 'busy':
				echo $weBackupWizard->getHTMLBusy();
				break;
			case 'extern':
				echo $weBackupWizard->getHTMLExtern();
				break;
			default:
				t_e(__FILE__ . ' unknown reference: ' . $what);
		}
	}

	public static function getJSLangConsts(){
		//'make_backup' : 'recover_backup'
		return we_base_util::setLangString([
				'make_backup' => [
					'banner_dep' => (g_l('backup', '[export][banner_dep]')),
					'binary_data' => g_l('backup', '[export][binary_data]'),
					'binary_dep' => (g_l('backup', '[export][binary_dep]')),
					'core_data' => g_l('backup', '[export][core_data]'),
					'customer_data' => g_l('backup', '[export][customer_data]'),
					'history_data' => g_l('backup', '[export][history_data]'),
					'newsletter_data' => g_l('backup', '[export][newsletter_data]'),
					'newsletter_dep' => (g_l('backup', '[export][newsletter_dep]')),
					'object_data' => g_l('backup', '[export][object_data]'),
					'schedule_data' => g_l('backup', '[export][schedule_data]'),
					'schedule_dep' => (g_l('backup', '[export][schedule_dep]')),
					'shop_data' => g_l('backup', '[export][shop_data]'),
					'shop_dep' => (g_l('backup', '[export][shop_dep]')),
					'temporary_data' => g_l('backup', '[export][temporary_data]'),
					'temporary_dep' => (g_l('backup', '[export][temporary_dep]')),
					'user_data' => g_l('backup', '[export][user_data]'),
					'versions_binarys_data' => g_l('backup', '[export][versions_binarys_data]'),
					'versions_binarys_dep' => (g_l('backup', '[export][versions_binarys_dep]')),
					'versions_data' => g_l('backup', '[export][versions_data]'),
					'versions_dep' => (g_l('backup', '[export][versions_dep]')),
					'workflow_data' => g_l('backup', '[export][workflow_data]'),
					'workflow_dep' => (g_l('backup', '[export][workflow_dep]')),
				],
				'recover_backup' => [
					'banner_dep' => (g_l('backup', '[import][banner_dep]')),
					'binary_data' => g_l('backup', '[import][binary_data]'),
					'binary_dep' => (g_l('backup', '[import][binary_dep]')),
					'core_data' => g_l('backup', '[import][core_data]'),
					'customer_data' => g_l('backup', '[import][customer_data]'),
					'history_data' => g_l('backup', '[import][history_data]'),
					'newsletter_data' => g_l('backup', '[import][newsletter_data]'),
					'newsletter_dep' => (g_l('backup', '[import][newsletter_dep]')),
					'object_data' => g_l('backup', '[import][object_data]'),
					'schedule_data' => g_l('backup', '[import][schedule_data]'),
					'schedule_dep' => (g_l('backup', '[import][schedule_dep]')),
					'shop_data' => g_l('backup', '[import][shop_data]'),
					'shop_dep' => (g_l('backup', '[import][shop_dep]')),
					'temporary_data' => g_l('backup', '[import][temporary_data]'),
					'temporary_dep' => (g_l('backup', '[import][temporary_dep]')),
					'user_data' => g_l('backup', '[import][user_data]'),
					'versions_binarys_data' => g_l('backup', '[import][versions_binarys_data]'),
					'versions_binarys_dep' => (g_l('backup', '[import][versions_binarys_dep]')),
					'versions_data' => g_l('backup', '[import][versions_data]'),
					'versions_dep' => (g_l('backup', '[import][versions_dep]')),
					'workflow_data' => g_l('backup', '[import][workflow_data]'),
					'workflow_dep' => (g_l('backup', '[import][workflow_dep]')),
				],
				'del_backup_confirm' => g_l('backup', '[del_backup_confirm]'),
				'delold_confirm' => g_l('backup', '[delold_confirm]'),
				'error_timeout' => g_l('backup', '[error_timeout]'),
				'finished' => g_l('backup', '[finished]'),
				'finished_success' => g_l('backup', '[finished_success]'),
				'import_file_found' => str_replace('"', '\'', g_l('backup', '[import][file_found]') . ' \n\n' . g_l('backup', '[import][file_found_question]')),
				'nothing_selected' => g_l('backup', '[nothing_selected]'),
				'nothing_selected_fromlist' => g_l('backup', '[nothing_selected_fromlist]'),
				'recover_backup_unsaved_changes' => g_l('backup', '[recover_backup_unsaved_changes]'),
				'save_not_checked' => g_l('backup', '[save_not_checked]'),
				'unselect_dep2' => g_l('backup', '[unselect_dep2]'),
				'unselect_dep3' => g_l('backup', '[unselect_dep3]'),
		]);
	}

}
