<?php

/**
 * webEdition CMS
 *
 * $Rev: 15672 $
 * $Author: mokraemer $
 * $Date: 2018-08-30 01:14:31 +0200 (Thu, 30 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_banner_frames extends we_modules_frame{

	public function __construct($frameset){
		parent::__construct($frameset);
		$this->module = we_modules_info::BANNER;

		$this->View = new we_banner_view();
		$this->Tree = new we_banner_tree($this->jsCmd);
	}

	function getHTML($what = '', $mode = '', $step = 0){
		switch($what){
			case 'edfooter':
				return $this->getHTMLEditorFooter([
						we_html_button::FOOTER_SAVE => [[we_base_perm::BANNER_EDIT], 'save_banner'],
						we_html_button::FOOTER_TRASH => [[we_base_perm::BANNER_DELETE], 'delete_banner'],
						], we_html_element::jsScript(WE_JS_MODULES_DIR . 'banner/banner_footer.js'));
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	protected function getHTMLEditorHeader($mode = 0){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return parent::getHTMLEditorHeader(0);
		}

		$isFolder = we_base_request::_(we_base_request::BOOL, 'isFolder');
		$page = we_base_request::_(we_base_request::INT, 'page', 0);
		$headline1 = g_l('modules_banner', $isFolder ? '[group]' : '[banner]');
		$text = we_base_request::_(we_base_request::STRING, "txt", g_l('modules_banner', ($isFolder ? '[newbannergroup]' : '[newbanner]')));

		$we_tabs = new we_gui_tabs();

		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, ($isFolder || $page == we_banner_banner::PAGE_PROPERTY), we_banner_banner::PAGE_PROPERTY, [
			'title' => g_l('tabs', '[module][properties]')]);
		if(!$isFolder){
			$we_tabs->addTab(g_l('tabs', '[module][placement]'), '', ($page == we_banner_banner::PAGE_PLACEMENT), we_banner_banner::PAGE_PLACEMENT);
			$we_tabs->addTab('', 'fa-pie-chart', ($page == we_banner_banner::PAGE_STATISTICS), we_banner_banner::PAGE_STATISTICS, [
				'title' => g_l('tabs', '[module][statistics]')]);
		}

		$extraHead = we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'initTabs.js') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'banner/banner_top.js');

		//TODO: we have the following body in several modules!
		$body = we_html_element::body(['onresize' => 'weTabs.setFrameSize()', 'onload' => 'weTabs.setFrameSize()', 'id' => 'eHeaderBody'], we_html_element::div([
					'id' => 'main'], we_html_element::div(['id' => 'headrow'], we_html_element::b(str_replace(' ', "&nbsp;", $headline1) . ':&nbsp;') .
						we_html_element::span(['id' => 'h_path', 'class' => 'header_small'], '<b id="titlePath">' . str_replace(' ', "&nbsp;", $text) . '</b>'
						)
					) .
					$we_tabs->getHTML()
				)
		);

		return $this->getHTMLDocument($body, $extraHead);
	}

	protected function getHTMLEditorBody(){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return $this->View->getHomeScreen();
		}
		return $this->getHTMLDocument($this->View->getProperties($this->jsCmd));
	}

	protected function getHTMLCmd(){
		if(($pid = we_base_request::_(we_base_request::RAW, "pid")) === false){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$offset = we_base_request::_(we_base_request::INT, "offset", 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_banner_tree::getItems($pid, $offset, $this->Tree->default_segment)]);

		return $this->getHTMLDocument(
				we_html_element::body([], we_html_element::form(['name' => 'we_form'], we_html_element::hiddens(['pnt' => "cmd",
							'cmd' => "no_cmd"])
					)
		));
	}

	function getHTMLDCheck(){
		return $this->getHTMLDocument(we_html_element::body(['onload' => 'self.focus();'], $this->View->getHTMLDCheck()));
	}

}
