<?php

/**
 * webEdition CMS
 *
 * $Rev: 15855 $
 * $Author: lukasimhof $
 * $Date: 2018-10-04 00:04:34 +0200 (Thu, 04 Oct 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* a class for handling templates */
class we_object extends we_contents_objectBase{
	const ELEMENT_LENGHT = 'length';
	const ELEMENT_TYPE = 'dtype';
	const ELEMENT_WIDTH = 'width';
	const ELEMENT_HEIGHT = 'height';
	const ELEMENT_CLASS = 'class';
	const ELEMENT_MAX = 'max';
	const ELEMENT_DEFAULT = 'default';
	const QUERY_PREFIX = 'object_';
	const VARIANT_TYPES = [
		self::TYPE_INPUT,
		self::TYPE_LINK,
		self::TYPE_TEXT,
		self::TYPE_IMG,
		self::TYPE_INT,
		self::TYPE_FLOAT,
		self::TYPE_META,
		self::TYPE_DATE,
		self::TYPE_HREF
	];

	var $Users = ''; // Default Owners
	var $UsersReadOnly = ''; // For DefaultOwners
	var $RestrictUsers = '';
	var $Workspaces = '';
	var $DefaultWorkspaces = '';
	var $WorkspaceFlag = 1;
	var $Templates = '';
	var $SerializedArray = []; // #3931
	var $DefaultValues = '';

	const URL_FIELDS = [
		'urlfield1' => 64,
		'urlfield2' => 64,
		'urlfield3' => 64,
	];
	const URL_UNIQUE = ['urlunique' => 16,];

	public function __construct(){
		parent::__construct();
		array_push($this->persistent_slots, 'WorkspaceFlag', 'RestrictUsers', 'UsersReadOnly', 'Text', 'SerializedArray', 'Templates', 'Workspaces', 'DefaultWorkspaces', 'ID', 'Users', 'Category', 'DefaultCategory', 'DefaultText', 'DefaultValues', 'DefaultTitle', 'DefaultKeywords', 'DefaultUrl', 'DefaultUrlfield0', 'DefaultUrlfield1', 'DefaultUrlfield2', 'DefaultUrlfield3', 'DefaultTriggerID', 'DefaultDesc', 'CSS');
		if(isWE()){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_WORKSPACE, we_base_constants::WE_EDITPAGE_INFO,
				we_base_constants::WE_EDITPAGE_CONTENT]); // ,we_base_constants::WE_EDITPAGE_PREVIEW
		}
		$this->setElement('Charset', DEFAULT_CHARSET, 'attrib');
		$this->Table = OBJECT_TABLE;
		$this->Published = PHP_INT_MAX;
		$this->CSS = '';
	}

	function save($resave = 0, $skipHook = 0){
		if(!$this->checkIfPathOk()){
			return false;
		}

		$this->ModDate = time();
		$this->ModifierID = !isset($GLOBALS['we']['Scheduler_active']) && isset($_SESSION['user']['ID']) ? $_SESSION['user']['ID'] : 0;

		$this->saveToDB();

		$GLOBALS['we_responseText'] = g_l('weClass', '[response_save_ok]');
		$GLOBALS['we_responseTextType'] = we_base_util::WE_MESSAGE_NOTICE;

		if(!$this->OldPath || ($this->OldPath != $this->Path)){
			$fID = f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . $this->DB_WE->escape($this->OldPath) . '"');
			$pID = intval(f('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE Path="' . str_replace('\\', '/', dirname($this->Path)) . '"'));
			$cf = new we_class_folder();
			$cf->initByID($fID, OBJECT_FILES_TABLE);
			$cf->Text = $this->Text;
			$cf->Filename = $this->Text;
			$cf->setParentID($pID);
			$cf->Path = $cf->getPath();
			$cf->TableID = $this->ID;
			$cf->we_save(true);
			$cf->modifyChildrenPath();
		}

		$this->OldPath = $this->Path; // reset oldPath
		$GLOBALS['we_JavaScript'][] = ['reload_editpage'];
		$GLOBALS['we_JavaScript'][] = ['setEditorDocumentId', $this->ID];
		$GLOBALS['we_JavaScript'][] = $this->getUpdateTreeScript(true, null, true);
		$GLOBALS['we_JavaScript'][] = we_main_headermenu::getMenuReloadCode('', true);
		return true;
	}

	function saveToDB(){
		$arrt = ['WorkspaceFlag' => $this->WorkspaceFlag,
			//	Save charsets in defaultvalues
//	charset must be in other namespace -> for header !!!
			'elements' => ['Charset' => ['dat' => $this->getElement('Charset', we_contents_base::ETYPE_TEXT)]],
		];

		$this->wasUpdate = $this->ID > 0;

		if(($def = $this->getElement('Defaultanzahl'))){
			$this->DefaultText = '';

			for($i = 0; $i <= $def; $i++){
				$was = 'DefaultText_' . $i;
				if(($dat = $this->getElement($was))){
					foreach(self::URL_UNIQUE as $key => $len){
						if(stristr($dat, $key)){
							$unique = $this->getElement($key . '_' . $i);
							$dat = '%' . str_replace('%', '', $dat) . (($unique > 0) ? $unique : $len) . '%';
							$this->setElement($was, $dat, 'defaultText');
						}
						$this->DefaultText .= $dat;
					}
				}
			}
		}
		if(($def = $this->getElement('DefaultanzahlUrl'))){
//FIXME: make this json - this format is totally nonsense
			$this->DefaultUrl = '';
			$allFields = array_merge(self::URL_UNIQUE, self::URL_FIELDS);
			for($i = 0; $i <= $def; $i++){
				$was = 'DefaultUrl_' . $i;

				if(($dat = $this->getElement($was))){
					foreach($allFields as $key => $len){
						if(stristr($dat, $key)){
							$unique = $this->getElement($key . '_' . $i);
							$dat = '%' . str_replace('%', '', $dat) . (($unique > 0) ? $unique : $len) . '%';
						}
					}
					$this->setElement($was, $dat);
					$this->DefaultUrl .= $dat;
				}
			}
		}


		if(!$this->wasUpdate){//create table, add element to tblObject
			$q = ['OF_ID' => 'INT unsigned NOT NULL',];

			$indexe = ['PRIMARY KEY (OF_ID)'];

			$arrt['WE_CSS_FOR_CLASS'] = $this->CSS;
			$this->DefaultValues = we_serialize($arrt, SERIALIZE_JSON);
			$this->DefaultTitle = ($tmp = $this->getElement('title')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
			$this->DefaultDesc = ($tmp = $this->getElement('desc')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
			$this->DefaultKeywords = ($tmp = $this->getElement('keywords')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
			$this->DefaultUrlfield0 = ($tmp = $this->getElement('urlfield0')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
			$this->DefaultUrlfield1 = ($tmp = $this->getElement('urlfield1')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($this->getElement('urlfield1')) : '_';
			$this->DefaultUrlfield2 = ($tmp = $this->getElement('urlfield2')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
			$this->DefaultUrlfield3 = ($tmp = $this->getElement('urlfield3')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
			$this->DefaultTriggerID = ($tmp = $this->getElement('triggerid')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '0';


			$this->DefaultCategory = $this->Category;
			$this->savePersistentSlotsToDB();

			$ctable = OBJECT_X_TABLE . intval($this->ID);

			$this->DB_WE->delTable($ctable);
			$this->DB_WE->addTable($ctable, $q, $indexe);

			$q = $indexe = [];
			$this->wasUpdate = true;
		}

		$we_sort = $this->getElement('we_sort');
		asort($we_sort, SORT_NUMERIC);
		$this->setElement('we_sort', $we_sort);

		$ctable = OBJECT_X_TABLE . intval($this->ID);
		$tableInfo = $this->DB_WE->metadata($ctable, we_database_base::META_NAME);
		$q = $regs = [];
		$fieldsToDelete = array_intersect($this->getElement('felderloeschen', we_contents_base::ETYPE_TEXT, []), $tableInfo);
		foreach($fieldsToDelete as $cur){
			$q[] = ' DROP `' . $cur . '` ';
		}

		$neu = $this->getElement('neuefelder', we_contents_base::ETYPE_TEXT, []);
		$lastElement = 'OF_ID';
		foreach(array_keys($we_sort) as $id){
			$cur = $this->getElement('wholename' . $id, we_contents_base::ETYPE_TEXT);
			$newType = $this->getElement($cur . self::ELEMENT_TYPE);
			$nam = $newType . '_' . $this->getElement($cur, we_contents_base::ETYPE_TEXT);

			if(in_array($cur, $neu)){
				$q[] = ' ADD `' . $nam . '` ' . $this->getSQLType($cur) . ' AFTER `' . $lastElement . '`';
			} else {
				//change from object is indexed to unindexed
				if((strpos($cur, self::QUERY_PREFIX) === 0) && ($newType . '_' != self::QUERY_PREFIX)){
					$q[] = ' DROP KEY `' . $cur . '` ';
				}

				$q[] = ' CHANGE `' . $cur . '` `' . $nam . '` ' . $this->getSQLType($cur) . ' AFTER `' . $lastElement . '`';
			}
			//add index for complex queries
			if((strpos($cur, self::QUERY_PREFIX) !== 0) && ($newType . '_' == self::QUERY_PREFIX)){
				$q[] = 'ADD INDEX (`' . $nam . '`) ';
			}
			$lastElement = $nam;

			$arrt[$nam] = array_filter([
				'default' => $this->getElement($cur . 'default'),
				'defaultThumb' => $this->getElement($cur . 'defaultThumb'),
				'autobr' => $this->getElement($cur . 'autobr'),
				'defaultdir' => $this->getElement($cur . 'defaultdir'),
				'rootdir' => $this->getElement($cur . 'rootdir'),
				'dhtmledit' => $this->getElement($cur . 'dhtmledit'),
				'showmenus' => $this->getElement($cur . 'showmenus'),
				'commands' => $this->getElement($cur . 'commands'),
				'contextmenu' => $this->getElement($cur . 'contextmenu'),
				self::ELEMENT_HEIGHT => $this->getElement($cur . 'height', we_contents_base::ETYPE_TEXT, 180),
				self::ELEMENT_WIDTH => $this->getElement($cur . self::ELEMENT_WIDTH, we_contents_base::ETYPE_TEXT, 388),
				'bgcolor' => $this->getElement($cur . 'bgcolor'),
				self::ELEMENT_CLASS => $this->getElement($cur . 'class'),
				'max' => $this->getElement($cur . 'max'),
				'cssClasses' => $this->getElement($cur . 'cssClasses'),
				'fontnames' => $this->getElement($cur . 'fontnames'),
				'fontsizes' => $this->getElement($cur . 'fontsizes'),
				'formats' => $this->getElement($cur . 'formats'),
				'tinyparams' => $this->getElement($cur . 'tinyparams'),
				'templates' => $this->getElement($cur . 'templates'),
				'gallerytemplates' => $this->getElement($cur . 'gallerytemplates'),
				'xml' => $this->getElement($cur . 'xml'),
				'removefirstparagraph' => $this->getElement($cur . 'removefirstparagraph'),
				'forbidhtml' => $this->getElement($cur . 'forbidhtml'),
				'inlineedit' => $this->getElement($cur . 'inlineedit'),
				'users' => $this->getElement($cur . 'users'),
				'required' => $this->getElement($cur . 'required'),
				'editdescription' => $this->getElement($cur . 'editdescription'),
				'int' => $this->getElement($cur . 'int'),
				'intID' => $this->getElement($cur . 'intID'),
				'hreftype' => $this->getElement($cur . 'hreftype'),
				'hrefdirectory' => $this->getElement($cur . 'hrefdirectory', we_contents_base::ETYPE_TEXT, 'false'),
				'hreffile' => $this->getElement($cur . 'hreffile', we_contents_base::ETYPE_TEXT, 'true'),
				'shopcatField' => $this->getElement($cur . 'shopcatField'),
				'shopcatShowPath' => $this->getElement($cur . 'shopcatShowPath'),
				'shopcatRootdir' => $this->getElement($cur . 'shopcatRootdir'),
				'shopcatLimitChoice' => $this->getElement($cur . 'shopcatLimitChoice'),
				'uniqueID' => $this->SerializedArray[$cur]['uniqueID'] ?: md5(uniqid(__FILE__, true)),
			]);
			if($this->isVariantField($cur) && $this->getElement($cur . 'variant') == 1){
				$arrt[$nam]['variant'] = 1;
			} else if($this->issetElement($cur . 'variant')){
				$this->delElement($cur . 'variant');
			}

			for($f = 0; $f <= $this->getElement($cur . 'count', we_contents_base::ETYPE_TEXT, 0); ++$f){
				if($this->issetElement($cur . 'defaultkey' . $f)){
					if((!isset($arrt[$nam]['meta'])) || (!is_array($arrt[$nam]['meta']))){
						$arrt[$nam]['meta'] = [];
					}

					$val = $this->getElement($cur . 'defaultvalue' . $f);
					$val = ($val != $cur . 'defaultvalue' . $f ? $val : '');
					if($newType == self::TYPE_MULTIOBJECT){
						$arrt[$nam]['meta'][] = $val;
					} else {
						$arrt[$nam]['meta'][$this->getElement($cur . 'defaultkey' . $f)] = $val;
					}
				}
			}
		}

		$this->DefaultCategory = $this->Category;

		$this->DefaultTitle = $this->getElement(($tmp = $this->getElement('title')) . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp);
		$this->DefaultDesc = $this->getElement(($tmp = $this->getElement('desc')) . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp);
		$this->DefaultKeywords = $this->getElement(($tmp = $this->getElement('keywords')) . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp);


		$this->DefaultUrlfield0 = ($tmp = $this->getElement('urlfield0')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
		$this->DefaultUrlfield1 = ($tmp = $this->getElement('urlfield1')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
		$this->DefaultUrlfield2 = ($tmp = $this->getElement('urlfield2')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
		$this->DefaultUrlfield3 = ($tmp = $this->getElement('urlfield3')) ? $this->getElement($tmp . self::ELEMENT_TYPE) . '_' . $this->getElement($tmp) : '_';
//$this->DefaultTriggerID = ($tmp = $this->getElement('triggerid')) ? $this->getElement($tmp.'dtype').'_'.$this->getElement($tmp) : '0';

		$arrt['WE_CSS_FOR_CLASS'] = $this->CSS;

		$this->DefaultValues = we_serialize($arrt, SERIALIZE_JSON);

		$variant_field = 'variant_' . we_base_variants::ELEMENT_NAME;

		$exists = $this->DB_WE->isColExist($ctable, $variant_field);

		if($this->hasVariantFields()){
			if(!$exists){
				$this->DB_WE->addCol($ctable, $variant_field, 'TEXT NOT NULL');
			}
		} elseif($exists){
			$this->DB_WE->delCol($ctable, $variant_field);
		}

		$q = array_filter($q);
		if($q){
			$this->DB_WE->query('ALTER TABLE ' . $ctable . ' ' . implode(',', $q));
		}

		$this->savePersistentSlotsToDB();

		unset($this->elements);
		$this->getContentData();
//$this->initByID($this->ID,$this->Table);
	}

	private function getSQLType($name){
		$len = $this->getElement($name . 'length', we_contents_base::ETYPE_TEXT);
		switch($this->getElement($name . self::ELEMENT_TYPE, we_contents_base::ETYPE_TEXT)){
			case self::TYPE_META://FIXME set to enum!!
				return 'VARCHAR(' . (($len > 0 && ($len < 255)) ? $len : 255) . ') NOT NULL';
			case self::TYPE_INPUT:
				return 'VARCHAR(' . (($len > 0 && ($len < 4096)) ? $len : 255) . ') NOT NULL';
			case self::TYPE_DATE:
				return 'INT unsigned NOT NULL';
			case self::TYPE_COUNTRY:
			case self::TYPE_LANGUAGE:
				return 'CHAR(2) NOT NULL';
			case self::TYPE_LINK:
			case self::TYPE_HREF:
			case self::TYPE_MULTIOBJECT:
			case self::TYPE_SHOPCATEGORY:
				return 'TEXT NOT NULL';
			case self::TYPE_TEXT:
				return 'LONGTEXT NOT NULL';
			case self::TYPE_IMG:
			case self::TYPE_FLASHMOVIE:
			case self::TYPE_AUDIO:
			case self::TYPE_VIDEO:
			case self::TYPE_BINARY:
			case self::TYPE_COLLECTION:
			case self::TYPE_OBJECT:
				return 'INT unsigned DEFAULT "0" NOT NULL';
			case self::TYPE_CHECKBOX:
				return 'TINYINT unsigned DEFAULT "' . ($this->getElement($name . 'default', we_contents_base::ETYPE_TEXT) == 1 ? '1' : '0') . '" NOT NULL';
			case self::TYPE_INT:
				$typeLen = $this->getElement($name . 'typeLen', we_contents_base::ETYPE_TEXT);
				$vals = explode('_', $typeLen);
				return ' ' . ($vals[0] ?: 'INT') . (!empty($vals[1]) && $vals[1] === 'U' ? ' UNSIGNED' : '') . ' DEFAULT NULL ';
			case self::TYPE_FLOAT:
				return 'DOUBLE DEFAULT NULL';
			case self::TYPE_SHOPVAT:
				return 'decimal(4,2) default "0" NOT NULL';
			default:
				return '';
		}
	}

	function getPath(){
		return rtrim($this->getParentPath(), '/') . '/' . $this->Text;
	}

	function ModifyPathInformation($parentID){
		$this->setParentID($parentID);
		$this->Path = $this->getPath();
		$this->wasUpdate = true;
		$this->savePersistentSlotsToDB(['Text', 'Path', 'ParentID']);
	}

	function setSort(){
		if($this->issetElement('we_sort')){
			return;
		}
		$ctable = OBJECT_X_TABLE . intval($this->ID);
		$tableInfo = $this->ID ? $this->DB_WE->metadata($ctable, we_database_base::META_NAME) : ['OF_ID'];
		$sort = [];
		$i = 0;
		foreach($tableInfo as $name){
			list($type, $name) = explode('_', $name, 2);
			switch($type){
				case 'OF':
				case 'variant':
					break;
				default:
					$sort[str_replace('.', '', uniqid($name))] = $i++;
			}
		}
		$this->setElement('we_sort', $sort);
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null){
		if(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0) === 'save_document'){
			$this->save();
			$GLOBALS['we_JavaScript'] = [];
			$GLOBALS['we_responseText'] = sprintf(g_l('weClass', '[response_save_ok]'), $this->Path);
			$GLOBALS['we_responseTextType'] = we_base_util::WE_MESSAGE_NOTICE;
			return new we_editor_save($this, $jsCmd);
		}
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
			case we_base_constants::WE_EDITPAGE_WORKSPACE:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info_object($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_CONTENT:
				return new we_editor_contentobject($this, $jsCmd);
		}
	}

	private function moveEntryAtClass($identifier, $move){
		$sort = $this->getElement('we_sort');
		$pos = $sort[$identifier];

		$newPos = array_search($pos + $move, $sort);
		if($newPos !== false){
			$sort[$identifier] = $sort[$newPos];
			$sort[$newPos] = $pos;
			asort($sort, SORT_NUMERIC);
			$this->setElement('we_sort', $sort);
		}
	}

	function downEntryAtClass($identifier){
		$this->moveEntryAtClass($identifier, 1);
	}

	function upEntryAtClass($identifier){
		$this->moveEntryAtClass($identifier, -1);
	}

	function addEntryToClass($identifier, $after = ''){
		$sort = $this->getElement('we_sort');
		$uid = uniqid();

		$this->setElement($uid, '');
		$this->setElement($uid . self::ELEMENT_LENGHT, '');
		$this->setElement($uid . self::ELEMENT_TYPE, '');
		$this->setElement($uid . self::ELEMENT_WIDTH, 388);
		$this->setElement($uid . self::ELEMENT_HEIGHT, 180);
		$this->setElement($uid . self::ELEMENT_CLASS, '');
		$this->setElement($uid . self::ELEMENT_MAX, '');
		$this->setElement('wholename' . $identifier, $uid);
		$nf = $this->getElement('neuefelder', we_contents_base::ETYPE_TEXT, []);
		$nf[] = $uid;
		$this->setElement('neuefelder', $nf);

		if($after && isset($sort[$after])){
			$pos = $sort[$after];

			foreach($sort as &$identpos){
				if($identpos > $pos){
					$identpos = $identpos + 1;
				}
			}
			$sort[$identifier] = $pos + 1;
		} else {
			$sort[$identifier] = count($sort);
		}
		asort($sort, SORT_NUMERIC);
		$this->setElement('we_sort', $sort);
	}

	function removeEntryFromClass($identifier){
		$sort = $this->getElement('we_sort', we_contents_base::ETYPE_TEXT);

		$uid = $this->getElement('wholename' . $identifier);

		if(($pos = array_search($uid, ($nf = $this->getElement('neuefelder', we_contents_base::ETYPE_TEXT, [])))) !== false){
			unset($nf[$pos]);
			$this->setElement('neuefelder', $nf);
		} else {
			$er = $this->getElement('felderloeschen', we_contents_base::ETYPE_TEXT, []);
			$er[] = $uid;
			$this->setElement('felderloeschen', $er);
		}

		$this->delElement('wholename' . $identifier);
		$this->delElement($uid);
		$this->delElement($uid . self::ELEMENT_LENGHT);
		$this->delElement($uid . self::ELEMENT_TYPE);
		$this->delElement($uid . self::ELEMENT_HEIGHT);
		$this->delElement($uid . self::ELEMENT_WIDTH);
		$this->delElement($uid . self::ELEMENT_DEFAULT);
		$this->delElement($uid . self::ELEMENT_CLASS);
		$this->delElement($uid . self::ELEMENT_MAX);


//move elements
		$pos = $sort[$identifier];
		unset($sort[$identifier]);
		foreach($sort as &$identpos){
			if($identpos > $pos){
				$identpos = $identpos - 1;
			}
		}

		asort($sort, SORT_NUMERIC);
		$this->setElement('we_sort', ($sort ?: []));
	}

	function downMetaAtClass($name, $i){
		$temp = $this->getElement($name . 'defaultkey' . ($i + 1), we_contents_base::ETYPE_TEXT);
		$this->setElement($name . 'defaultkey' . ($i + 1), $this->getElement($name . 'defaultkey' . ($i), we_contents_base::ETYPE_TEXT));
		$this->setElement($name . 'defaultkey' . ($i), $temp);

		$temp = $this->getElement($name . 'defaultvalue' . ($i + 1), we_contents_base::ETYPE_TEXT);
		$this->setElement($name . 'defaultvalue' . ($i + 1), $this->getElement($name . 'defaultvalue' . ($i), we_contents_base::ETYPE_TEXT));
		$this->setElement($name . 'defaultvalue' . ($i), $temp);
	}

	function upMetaAtClass($name, $i){
		$temp = $this->getElement($name . 'defaultkey' . ($i - 1), we_contents_base::ETYPE_TEXT);
		$this->setElement($name . 'defaultkey' . ($i - 1), $this->getElement($name . 'defaultkey' . ($i), we_contents_base::ETYPE_TEXT));
		$this->setElement($name . 'defaultkey' . ($i), $temp);

		$temp = $this->getElement($name . 'defaultvalue' . ($i - 1), we_contents_base::ETYPE_TEXT);
		$this->setElement($name . 'defaultvalue' . ($i - 1), $this->getElement($name . 'defaultvalue' . ($i), we_contents_base::ETYPE_TEXT));
		$this->setElement($name . 'defaultvalue' . ($i), $temp);
	}

	function addMetaToClass($name, $pos){
// get from request
		$amount = we_base_request::_(we_base_request::INT, "amount_insert_meta_at_class_" . $name . $pos, 1);

// set new amount
		$cnt = $this->getElement($name . 'count') + $amount;
		$this->setElement($name . 'count', $cnt);

// move elements - add new elements
		for($i = $cnt; 0 <= $i; $i--){

			if(($pos + $amount) < $i){// move existing fields
				$this->setElement($name . 'defaultkey' . $i, ($this->getElement($name . 'defaultkey' . ($i - $amount))));
				$this->setElement($name . 'defaultvalue' . $i, ($this->getElement($name . 'defaultvalue' . ($i - $amount))));
			} else if($pos < $i && $i <= ($pos + $amount)){ // add new fields
				$this->setElement($name . 'defaultkey' . $i, '');
				$this->setElement($name . 'defaultvalue' . $i, '');
			}
		}
	}

	function removeMetaFromClass($name, $nr){
//move elements
		$cnt = $this->getElement($name . 'count');
		for($i = 0; $i < $cnt; $i++){
			if($i >= $nr){
				$this->setElement($name . 'defaultkey' . $i, ($this->getElement($name . 'defaultkey' . ($i + 1))));
				$this->setElement($name . 'defaultvalue' . $i, ($this->getElement($name . 'defaultvalue' . ($i + 1))));
			}
		}
		$this->setElement($name . 'defaultkey' . $i, '');
		$this->setElement($name . 'defaultvalue' . $i, '');
		$this->setElement($name . 'count', max($cnt - 1, 0));
	}

	function getEmptyDefaultFields(){
		$ret = '';
		foreach([
		'title', 'desc', 'keywords',
		'urlfield0', 'urlfield1', 'urlfield2', 'urlfield3'
		] as $field){
			$ret .= '<input type="radio" value="0" name="we_' . $this->Name . '_input[' . $field . ']" id="empty_' . $this->Name . '_input[' . $field . ']"/>';
		}
		return we_html_element::div(['style' => "display:none"], $ret);
	}

	private function getFieldHTMLName($type, $name, $identifier, &$fieldName){
		switch($type){
			case self::TYPE_OBJECT:
				$val = $this->getElement($name, we_contents_base::ETYPE_TEXT);
				$vals = $this->DB_WE->getAllFirstq('SELECT ID,Path FROM ' . OBJECT_TABLE . ' ORDER BY Path', false);
				$fieldName = empty($vals[$val]) ? '' : $vals[$val];
				return we_html_tools::select('we_' . $this->Name . '_input[' . $name . ']', $vals, 1, $val, '', ['onchange' => "if(this.form.elements['we_" . $this->Name . '_input[' . $name . "default]']){this.form.elements['we_" . $this->Name . '_input[' . $name . "default]'].value='' };_EditorFrame.setEditorIsHot(true);we_cmd('object_change_entry_at_class','" . $GLOBALS['we_transaction'] . "','" . $identifier . "')"], true, 388);
			case self::TYPE_SHOPVAT:
			case self::TYPE_SHOPCATEGORY:
				$fieldName = $type === self::TYPE_SHOPCATEGORY ? we_shop_shop::CATEGORY_FIELD_NAME : we_shop_shop::VAT_FIELD_NAME;
				return we_html_element::hidden('we_' . $this->Name . '_input[' . $name . ']', $fieldName) .
					we_html_tools::textInput('tmp' . $fieldName, $fieldName, 52, ['readonly' => 'readonly', 'disabled' => 'disabled'], 'text');
			default:
				$fieldName = $this->getElement($name, we_contents_base::ETYPE_TEXT) ?: g_l('modules_object', '[field][new]');
				return we_html_tools::textInput('we_' . $this->Name . '_input[' . $name . ']', $fieldName, 52, ['oldValue' => $fieldName, 'onblur' => "we_checkObjFieldname(this);",
						'onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text');
		}
	}

	private function getFieldHTMLLength($type, $name){
		switch($type){
			case self::TYPE_AUDIO:
			case self::TYPE_BINARY:
			case self::TYPE_CHECKBOX:
			case self::TYPE_COLLECTION:
			case self::TYPE_COUNTRY:
			case self::TYPE_DATE:
			case self::TYPE_FLASHMOVIE:
			case self::TYPE_FLOAT:
			case self::TYPE_HREF:
			case self::TYPE_IMG:
			case self::TYPE_LANGUAGE:
			case self::TYPE_LINK:
			case self::TYPE_META:
			case self::TYPE_MULTIOBJECT:
			case self::TYPE_OBJECT:
			case self::TYPE_SHOPCATEGORY:
			case self::TYPE_SHOPVAT:
			case self::TYPE_TEXT:
			case self::TYPE_VIDEO:
				return '';
			case self::TYPE_INT:
				$values = [
					g_l('modules_object', '[int][signed]') => we_html_tools::OPTGROUP,
					'TINYINT' => '[-128 .. 127] (tiny)',
					'SMALLINT' => '[-32.768 .. 32.767] (small)',
					'MEDIUMINT' => '[-8.388.608 .. 8.388.607] (medium)',
					'INT' => '[-2.147.483.648 .. 2.147.483.647] (int)',
					'BIGINT' => '[-9.223.372.036.854.775.808 .. 9.223.372.036.854.775.807] (big)',
					g_l('modules_object', '[int][unsigned]') => we_html_tools::OPTGROUP,
					'TINYINT_U' => '[0 .. 255] (tiny)',
					'SMALLINT_U' => '[0 .. 65.535] (small)',
					'MEDIUMINT_U' => '[0 .. 16.777.215] (medium)',
					'INT_U' => '[0 .. 4.294.967.295] (int)',
					'BIGINT_U' => '[0 .. 18.446.744.073.709.551.615] (big)',
				];
				$sel = $this->getElement($name . 'typeLen', we_contents_base::ETYPE_TEXT);
				return '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[range][int]') . '</br>' .
					we_html_tools::select('we_' . $this->Name . '_input[' . $name . 'typeLen]', $values, 1, $sel ? $sel : 'INT', false, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
					]) .
					'</td></tr>';
			default:
// Length
				$maxLengthVal = 4095;
				$len = $this->getElement($name . 'length', we_contents_base::ETYPE_TEXT);
				return '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[length]') . '<br/>' .
					we_html_tools::textInput('we_' . $this->Name . '_input[' . $name . self::ELEMENT_LENGHT . ']', max(0, min($len, $maxLengthVal)), 4, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
						'weType' => 'weObject_' . $type . '_length'], 'text') .
					'</td></tr>';
		}
	}

	private function getDateFieldHTML($name){
		$d = abs($this->getElement($name . 'default'));
		$dd = $this->getElement($name . 'defaultThumb', self::ETYPE_UNKNOWN, false);
		return we_html_forms::checkboxWithHidden($dd, 'we_' . $this->Name . '_xdate[' . $name . 'defaultThumb]', g_l('modules_object', '[creationDate]'), false, '', "_EditorFrame.setEditorIsHot(true);document.getElementById('we_" . $this->Name . '_date[' . $name . "default_dateTable]').style.display=this.checked?'none':'';") .
			we_html_tools::getDateInput('we_' . $this->Name . '_date[' . $name . 'default]', ($d ?: time()), true, '', '', 'weSelect', false, 0, 0, '', '', $dd ? 'display="none";' : '');
	}

	private function getCommonBinaryFieldHTML($name, $identifier){
		return $this->formDirChooser(0, FILE_TABLE, 'ParentPath', 'input[' . $name . 'rootdir]', '', $this->getElement($name . 'rootdir', self::ETYPE_TEXT), $identifier) .
			'</td></tr>' .
			'<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[defaultdir]') . '<br/>' .
			$this->formDirChooser(0, FILE_TABLE, 'StartPath', 'input[' . $name . 'defaultdir]', '', $this->getElement($name . 'defaultdir', self::ETYPE_TEXT), $identifier) .
			'</td></tr>' .
			'<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>';
	}

	private function getMetaFieldHTML($name, $identifier){
		$content = '<table><tr><td>Key</td><td>Value</td><td></td></tr>';
		if(!$this->issetElement($name . 'count')){
			$this->setElement($name . 'count', 0);
		}

		$addArray = [1 => 1, 2 => 2, 3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10];

		for($f = 0; $f <= $this->getElement($name . 'count', we_contents_base::ETYPE_TEXT, 0); $f++){
			$content .= '<tr><td>' . we_html_tools::textInput('we_' . $this->Name . '_input[' . $name . 'defaultkey' . $f . ']', $this->getElement($name . 'defaultkey' . $f), 255, [
					'onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text', 105) .
				'</td><td>' . we_html_tools::textInput('we_' . $this->Name . '_input[' . $name . 'defaultvalue' . $f . ']', $this->getElement($name . "defaultvalue" . $f), 255, [
					'onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text', 105);

			$content .= '</td><td>' .
				we_html_tools::select('amount_insert_meta_at_class_' . $name . $f, $addArray) .
				we_html_button::create_button(we_html_button::LIST_ADD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_insert_meta_at_class','" . $GLOBALS['we_transaction'] . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") .
				(($f > 0) ?
				we_html_button::create_button(we_html_button::DIRUP, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_up_meta_at_class','" . $GLOBALS['we_transaction'] . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") :
				we_html_button::create_button(we_html_button::DIRUP, "#", '', '', true)
				) .
				(($f < ($this->getElement($name . 'count', we_contents_base::ETYPE_TEXT, 0))) ?
				we_html_button::create_button(we_html_button::DIRDOWN, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_down_meta_at_class','" . $GLOBALS['we_transaction'] . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") :
				we_html_button::create_button(we_html_button::DIRDOWN, "#", '', '', true)
				) .
				we_html_button::create_button(we_html_button::TRASH, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_delete_meta_class','" . $GLOBALS['we_transaction'] . "','" . ($identifier) . "','" . $name . "','" . ($f) . "')") .
				'</td></tr>';
		}
		$content .= '</table>';

		return $content;
	}

	private function getHrefFieldHTML($name, $identifier){
		$typeVal = $this->getElement($name . 'hreftype', we_contents_base::ETYPE_TEXT);
		// FIXME: use we_html_select
		$typeSelect = '<select style="margin-right:4em;" class="weSelect" id="we_' . $this->Name . '_input[' . $name . 'hreftype]" name="we_' . $this->Name . '_input[' . $name . 'hreftype]" onchange="' . "we_cmd('object_reload_entry_at_class','" . $GLOBALS['we_transaction'] . "','" . $identifier . "');" . '">
			<option' . (($typeVal === we_base_link::TYPE_ALL || !$typeVal) ? ' selected' : '') . ' value="' . we_base_link::TYPE_ALL . '">all</option>
			<option' . (($typeVal === we_base_link::TYPE_INT) ? ' selected' : '') . ' value="' . we_base_link::TYPE_INT . '">int</option>
			<option' . (($typeVal === we_base_link::TYPE_EXT) ? ' selected' : '') . ' value="' . we_base_link::TYPE_EXT . '">ext</option>
			</select>';
		$fileVal = $this->getElement($name . 'hreffile') ?: 'true';
		$fileSelect = '<select style="margin-right:4em;" class="weSelect" id="we_' . $this->Name . '_input[' . $name . 'hreffile]" name="we_' . $this->Name . '_input[' . $name . 'hreffile]">
			<option' . (($fileVal === 'true') ? ' selected="selected"' : '') . ' value="true">true</option>
			<option' . (($fileVal === 'false') ? ' selected="selected"' : '') . ' value="false">false</option>
			</select>';
		$dirVal = $this->getElement($name . 'hrefdirectory') ?: 'false'; // options anzeige umgedreht wegen 4363
		$dirSelect = '<select class="weSelect" id="we_' . $this->Name . '_input[' . $name . 'hrefdirectory]" name="we_' . $this->Name . '_input[' . $name . 'hrefdirectory]">
			<option' . (($dirVal === 'true') ? ' selected="selected"' : '') . ' value="true">false</option>
			<option' . (($dirVal === 'false') ? ' selected="selected"' : '') . ' value="false">true</option>
			</select>';

		return $typeSelect . 'file ' .
			$fileSelect . 'directory ' .
			$dirSelect .
			'</td></tr>
					<tr style="vertical-align:top"><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
			$this->htmlHref($name);
	}

	private function getFullMultiobjectFieldHTML($name, $identifier){
		$vals = $this->DB_WE->getAllFirstq('SELECT ID,Path FROM ' . OBJECT_TABLE . ' ORDER BY Path', false);
		if($this->getElement($name . 'class') === ''){
			$this->setElement($name . 'class', array_shift(array_flip($vals)));
		}

		$content = we_html_tools::select('we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . 'class]', $vals, 1, $this->getElement($name . 'class', we_contents_base::ETYPE_TEXT), '', [
				'onchange' => "if(this.form.elements['we_" . $this->Name . '_input[' . $name . 'default]' . "']){this.form.elements['we_" . $this->Name . '_input[' . $name . 'default]' . "'].value='';};_EditorFrame.setEditorIsHot(true);we_cmd('object_change_multiobject_at_class','" . $GLOBALS['we_transaction'] . "','" . $identifier . "','" . $name . "')"]) .
			'</td></tr>
<tr style="vertical-align:top"><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[max_objects]') . '<br/>' . we_html_tools::textInput('we_' . $this->Name . '_' . self::TYPE_MULTIOBJECT . '[' . $name . 'max]', $this->getElement($name . 'max', we_contents_base::ETYPE_TEXT), 3, [
				'onchange' => "we_cmd('object_reload_entry_at_class','" . $GLOBALS['we_transaction'] . "','" . ($identifier) . "');"], 'text', 50) . ' (' . g_l('modules_object', '[no_maximum]') . ')</td>
</tr>
<tr style="vertical-align:top"><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/><table>';

		if(!$this->issetElement($name . 'count')){
			$this->setElement($name . 'count', 0);
		}
		for($f = 0; $f <= $this->getElement($name . 'count', we_contents_base::ETYPE_TEXT, 0); $f++){
			$content .= $this->getMultiObjectFieldHTML($name, $identifier, $f);
		}

		return $content . '</tr></table>';
	}

	function getFieldHTML(we_base_jsCmd $jsCmd, $name, $identifier, $we_transaction){
		$type = $this->getElement($name . self::ELEMENT_TYPE, we_contents_base::ETYPE_TEXT) ?: self::TYPE_INPUT;
		$val = [
			g_l('modules_object', '[fieldgroups][input]') => we_html_tools::OPTGROUP,
			self::TYPE_INPUT => g_l('modules_object', '[field][input]'),
			self::TYPE_TEXT => g_l('modules_object', '[field][textarea]'),
			self::TYPE_INT => g_l('modules_object', '[field][int]'),
			self::TYPE_FLOAT => g_l('modules_object', '[field][float]'),
			g_l('modules_object', '[fieldgroups][select]') => we_html_tools::OPTGROUP,
			self::TYPE_DATE => g_l('modules_object', '[field][date]'),
			self::TYPE_CHECKBOX => g_l('modules_object', '[field][checkbox]'),
			self::TYPE_META => g_l('modules_object', '[field][meta]'),
			g_l('modules_object', '[fieldgroups][media]') => we_html_tools::OPTGROUP,
			self::TYPE_IMG => g_l('modules_object', '[field][img]'),
			self::TYPE_AUDIO => g_l('modules_object', '[field][audio]'),
			self::TYPE_VIDEO => g_l('modules_object', '[field][video]'),
			self::TYPE_COLLECTION => g_l('modules_object', '[field][collection]'),
			self::TYPE_FLASHMOVIE => g_l('modules_object', '[field][flashmovie]'),
			g_l('modules_object', '[fieldgroups][language]') => we_html_tools::OPTGROUP,
			self::TYPE_COUNTRY => g_l('modules_object', '[field][country]'),
			self::TYPE_LANGUAGE => g_l('modules_object', '[field][language]'),
			g_l('modules_object', '[fieldgroups][link]') => we_html_tools::OPTGROUP,
			self::TYPE_LINK => g_l('modules_object', '[field][link]'),
			self::TYPE_HREF => g_l('modules_object', '[field][href]'),
			self::TYPE_BINARY => g_l('modules_object', '[field][binary]'),
			self::TYPE_OBJECT => g_l('modules_object', '[field][objectFile]'),
			self::TYPE_MULTIOBJECT => g_l('modules_object', '[field][multiObjectFile]'),
		];
		if(we_modules_info::isActive(we_modules_info::SHOP)){
			$val[g_l('modules_object', '[fieldgroups][shop]')] = we_html_tools::OPTGROUP;
			$val[self::TYPE_SHOPVAT] = g_l('modules_object', '[field][shopVat]');
			$val[self::TYPE_SHOPCATEGORY] = g_l('modules_object', '[field][shopCat]');
		}
		$fieldName = '';
		$nameField = $this->getFieldHTMLName($type, $name, $identifier, $fieldName);
		$content = '<tr>
			<td class="weMultiIconBoxHeadline leftSpace-med" rowspan="100">' . we_html_element::div(['class' => 'weObjectHeadline'], ($fieldName ?: g_l('weClass', '[name]')) .
				we_html_element::div(['class' => 'addTrashUpDown'], we_html_element::div([], we_html_button::create_button(we_html_button::ADD_FIELD, "javascript:we_cmd('object_insert_entry_at_class','" . $we_transaction . "','" . $identifier . "');") .
						we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_delete_entry_at_class','" . $we_transaction . "','" . $identifier . "');")) .
					we_html_element::div(['class' => 'UpDown'], we_html_button::create_button(we_html_button::DIRUP, "javascript:we_cmd('object_up_entry_at_class','" . $we_transaction . "','" . $identifier . "');", '', '', false, false, "_" . $identifier) .
						we_html_button::create_button(we_html_button::DIRDOWN, "javascript:we_cmd('object_down_entry_at_class','" . $we_transaction . "','" . $identifier . "');", '', '', false, false, "_" . $identifier))
				)
			) .
			'</td>
			<td class="weMultiIconBoxHeadlineThin">' . g_l('weClass', '[name]') . '<br/>' . $nameField . '</td></tr>' .
			'<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('global', '[description]') . '<br/>' .
			$this->htmlTextArea('we_' . $this->Name . '_input[' . $name . 'editdescription]', 1, 40, $this->getElement($name . 'editdescription'), ['onchange' => '_EditorFrame.setEditorIsHot(true)',
				'class' => 'wetextarea']) .
			'</td></tr>' .
//type
			'<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[type]') . '<br/>' .
			we_html_tools::select('we_' . $this->Name . "_input[" . $name . self::ELEMENT_TYPE . ']', $val, 1, $type, '', ['onchange' => "if(this.form.elements['we_" . $this->Name . '_input[' . $name . 'default]' . "']){this.form.elements['we_" . $this->Name . '_input[' . $name . 'default]' . "'].value='' };_EditorFrame.setEditorIsHot(true);we_cmd('object_change_entry_at_class','" . $GLOBALS['we_transaction'] . "','" . $identifier . "');"]) .
			'</td></tr>' .
			$this->getFieldHTMLLength($type, $name);

		switch($type){
			case self::TYPE_MULTIOBJECT:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('contentTypes', '[object]') . '<br/>' .
					$this->getFullMultiobjectFieldHTML($name, $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_HREF:
				$content .= '<tr><td>type ' . $this->getHrefFieldHTML($name, $identifier) . '</td></tr>';
				break;
			case self::TYPE_CHECKBOX:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					we_html_forms::checkbox(1, $this->getElement($name . 'default', we_contents_base::ETYPE_TEXT), 'we_' . $this->Name . '_input[' . $name . "default1]", g_l('modules_object', '[checked]'), true, '', "if(this.checked){document.we_form.elements['we_" . $this->Name . "_input[" . $name . "default]" . "'].value=1;}else{ document.we_form.elements['we_" . $this->Name . '_input[' . $name . 'default]' . "'].value=0;}") .
					'<input type=hidden name="we_' . $this->Name . "_input[" . $name . "default]" . '" value="' . $this->getElement($name . "default", self::ETYPE_TEXT) . '" />' .
					'</td></tr>';
				break;
			case self::TYPE_IMG:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
					$this->getCommonBinaryFieldHTML($name, $identifier) .
					$this->getImageHTML($name . 'default', $this->getElement($name . 'default', self::ETYPE_TEXT), $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_FLASHMOVIE:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
					$this->getCommonBinaryFieldHTML($name, $identifier) .
					$this->getFlashmovieHTML($name . 'default', $this->getElement($name . 'default', self::ETYPE_TEXT), $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_AUDIO:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
					$this->getCommonBinaryFieldHTML($name, $identifier) .
					$this->getAudioHTML($name . 'default', $this->getElement($name . 'default', self::ETYPE_TEXT), $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_VIDEO:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
					$this->getCommonBinaryFieldHTML($name, $identifier) .
					$this->getVideoHTML($name . 'default', $this->getElement($name . 'default', self::ETYPE_TEXT), $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_BINARY:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[rootdir]') . '<br/>' .
					$this->getCommonBinaryFieldHTML($name, $identifier) .
					$this->getBinaryHTML($name . 'default', $this->getElement($name . 'default', self::ETYPE_TEXT), $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_DATE:
				$content .= '<tr><td>Default<br/>' .
					$this->getDateFieldHTML($name) .
					'</td></tr>';
				break;
			case self::TYPE_TEXT:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					$this->dhtmledit($name, $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_OBJECT:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					$this->getObjectFieldHTML($name, '') .
					'</td></tr>';
				break;
			case self::TYPE_COLLECTION:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					$this->getCollectionFieldHTML($name . 'default', $this->getElement($name . 'default', self::ETYPE_TEXT)) .
					'</td></tr>';
				break;
			case self::TYPE_META:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					$this->getMetaFieldHTML($name, $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_COUNTRY:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					we_html_tools::textInput('we_' . $this->Name . '_country[' . $name . 'default]', $this->getElement($name . 'default', self::ETYPE_TEXT), 10, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
						'weType' => $type], 'text', 388) .
					'</td></tr>';
				break;
			case self::TYPE_LANGUAGE:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					we_html_tools::textInput('we_' . $this->Name . '_language[' . $name . 'default]', $this->getElement($name . 'default', self::ETYPE_TEXT), 15, ['onchange' => "_EditorFrame.setEditorIsHot(true);",
						'weType' => $type], 'text', 388) .
					'</td></tr>';
				break;
			case self::TYPE_LINK:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					$this->htmlLinkInput($name, $identifier) .
					'</td></tr>';
				break;
			case self::TYPE_SHOPVAT:
				$values = [];
				if(we_modules_info::isActive(we_modules_info::SHOP)){
					$allVats = we_shop_vats::getAllShopVATs();
					foreach($allVats as $id => $shopVat){
						$values[$id] = $shopVat->vat . ' - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
					}

					$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
						we_html_tools::select('we_' . $this->Name . '_shopVat[' . $name . 'default]', $values, 1, $this->getElement($name . 'default', self::ETYPE_TEXT)) .
						'</td></tr>';
				}
				break;
			case self::TYPE_SHOPCATEGORY:
				if(we_modules_info::isActive(we_modules_info::SHOP)){
					$selectField = self::htmlSelect('we_' . $this->Name . '_input[' . $name . 'shopcatField]', ['0' => '', 'ID' => 'ID', 'Category' => 'Category', 'Path' => 'Path',
							'Title' => 'Title', 'Description' => 'Description', 'destPrinciple' => 'destPrinciple'], 1, $this->getElement($name . 'shopcatField', we_contents_base::ETYPE_TEXT));

					$selectShopPath = self::htmlSelect('we_' . $this->Name . '_input[' . $name . 'shopcatShowPath]', ['true' => 'true', 'false' => 'false'], 1, $this->getElement($name . 'shopcatShowPath', we_contents_base::ETYPE_TEXT));
					$textRootdir = self::htmlTextInput('we_' . $this->Name . '_input[' . $name . 'shopcatRootdir]', $this->getElement($name . 'shopcatRootdir', we_contents_base::ETYPE_TEXT));

					$values = ['0' => ' '] + we_shop_category::getShopCatFieldsFromDir('Path', true); //Fix #9355 don't use array_merge() because numeric keys will be renumbered!
					$selectCategories = we_html_tools::select('we_' . $this->Name . '_shopCategory[' . $name . 'default]', $values, 1, $this->getElement($name . 'default', we_contents_base::ETYPE_TEXT), false, [
							], true, 388);
					$selectLimitChoice = we_html_forms::checkboxWithHidden((abs($this->getElement($name . 'shopcatLimitChoice', we_contents_base::ETYPE_TEXT)) == '1' ? true : false), 'we_' . $this->Name . '_input[' . $name . 'shopcatLimitChoice]', 'use default only', false, '', '_EditorFrame.setEditorIsHot(true);');

					$content .= '<tr><td>' .
						'field ' . $selectField . ' showpath ' . $selectShopPath . '</td>
						</tr>
						<tr><td>' .
						'rootdir ' . $textRootdir . '</td>
						</tr>
						<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '</td><td>' .
						$selectCategories . '<br/>' . $selectLimitChoice . '</td>
						</tr>';
				}
				break;
			default: // default for input, int and float
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('modules_object', '[default]') . '<br/>' .
					we_html_tools::textInput('we_' . $this->Name . '_input[' . $name . 'default]', $this->getElement($name . 'default', we_contents_base::ETYPE_TEXT), ($type === self::TYPE_INT ? 9 : ($type === self::TYPE_FLOAT ? 19 : 255)), [
						'onchange' => "_EditorFrame.setEditorIsHot(true);", 'weType' => $type], 'text', 388) .
					'</td></tr>';
				break;
		}

		switch($type){
			case self::TYPE_DATE:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin"><br/>';
				break;
			case self::TYPE_TEXT:
			case self::TYPE_INPUT:
			case self::TYPE_META:
			case self::TYPE_LINK:
			case self::TYPE_HREF:
				$content .= '<tr><td class="weMultiIconBoxHeadlineThin"><br/><div style="float: left;width:50%;">';
				foreach(['title' => g_l('global', '[title]'), 'desc' => g_l('global', '[description]'), 'keywords' => g_l('weClass', '[Keywords]')] as $fname => $gl){
					$content .= we_html_forms::radiobutton($name, ($this->getElement($fname, we_contents_base::ETYPE_TEXT) == $name), 'we_' . $this->Name . '_input[' . $fname . ']', $gl, true, '', "if(this.waschecked){document.getElementById('empty_" . $this->Name . '_input[' . $fname . "]').checked=true;this.waschecked=false;}_EditorFrame.setEditorIsHot(true);", false, '', 0, 0, "if(this.checked){this.waschecked=true}");
				}
				$content .= '</div>';
				break;
			default:
		}

		switch($type){
			case self::TYPE_TEXT:
			case self::TYPE_INPUT:
			case self::TYPE_DATE:
				$content .= '<div>';
				$fields = ($type === self::TYPE_DATE ? ['urlfield0'] : array_keys(self::URL_FIELDS));
				foreach($fields as $key){
					$content .= we_html_forms::radiobutton($name, ($this->getElement($key) == $name), 'we_' . $this->Name . '_input[' . $key . ']', g_l('weClass', '[' . $key . ']'), true, '', "if(this.waschecked){document.getElementById('empty_" . $this->Name . '_input[' . $key . "]').checked=true;this.waschecked=false;}_EditorFrame.setEditorIsHot(true);", false, '', 0, 0, "if(this.checked){this.waschecked=true}");
				}
				$content .= '</div>';
			//nobreak;
			case self::TYPE_META:
			case self::TYPE_LINK:
			case self::TYPE_HREF:
				$content .= '</td></tr>';
				break;
			default:
		}


		if($type != self::TYPE_CHECKBOX){
//Pflichtfeld
			$content .= '<tr style="vertical-align:top"><td>' .
				we_html_forms::checkbox(1, $this->getElement($name . 'required', we_contents_base::ETYPE_TEXT), 'we_' . $this->Name . '_input[' . $name . 'required1]', g_l('global', '[required_field]'), true, '', "if(this.checked){document.we_form.elements['we_" . $this->Name . '_input[' . $name . "required]'].value=1;}else{ document.we_form.elements['we_" . $this->Name . '_input[' . $name . "required]'].value=0;}") .
				($this->canHaveVariants() && $this->isVariantField($name) ?
				we_html_forms::checkboxWithHidden($this->getElement($name . 'variant', self::ETYPE_TEXT), 'we_' . $this->Name . '_variant[' . $name . 'variant]', g_l('global', '[variant_field]'), false, '', '_EditorFrame.setEditorIsHot(true);') :
				'') .
				'<input type=hidden name="we_' . $this->Name . '_input[' . $name . 'required]" value="' . $this->getElement($name . 'required', self::ETYPE_TEXT) . '" />' .
				'</td></tr>';
// description for editmode.
		} else {
			$content .= (we_modules_info::isActive(we_modules_info::SHOP) ?
//Pflichtfeld
				'<tr style="vertical-align:top"><td>' .
				($this->canHaveVariants() && $this->isVariantField($name) ?
				we_html_forms::checkboxWithHidden($this->getElement($name . 'variant', self::ETYPE_TEXT), 'we_' . $this->Name . '_variant[' . $name . 'variant]', g_l('global', '[variant_field]'), false, '', '_EditorFrame.setEditorIsHot(true);') :
				'') .
				'<input type=hidden name="we_' . $this->Name . '_input[' . $name . 'required]" value="0" />' .
				'</td></tr>' :
				'<input type=hidden name="we_' . $this->Name . '_input[' . $name . 'required]" value="0" />' );
		}


		$content .= '<tr><td class="weMultiIconBoxHeadlineThin">' . g_l('weClass', '[fieldusers]') . '<br/>' .
			$this->formUsers1($jsCmd, $name, $identifier) .
			'</td></tr>';

		return $content;
	}

	function htmlHref($name){
		$type = $this->getElement($name . 'hreftype');
		$name .= 'default';
		$attr = ' size="20" ';
		$hrefArr = we_unserialize($this->getElement($name));

		$int = isset($hrefArr['int']) ? $hrefArr['int'] : false;
		$intID = isset($hrefArr['intID']) ? $hrefArr['intID'] : 0;
		$intPath = $intID ? id_to_path($intID) : '';
		$int_elem_Name = 'we_' . $this->Name . '_href[' . $name . we_base_link::MAGIC_INT_LINK . ']';
		$intPath_elem_Name = 'we_' . $this->Name . '_vars[' . $name . we_base_link::MAGIC_INT_LINK_PATH . ']';
		$intID_elem_Name = 'we_' . $this->Name . '_href[' . $name . we_base_link::MAGIC_INT_LINK_ID . ']'; //TODO: should we use #bdid?
		$ext_elem_Name = 'we_' . $this->Name . '_href[' . $name . we_base_link::MAGIC_INT_LINK_EXTPATH . ']';

		switch($type){
			case we_base_link::TYPE_INT:
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name);
				break;
			case we_base_link::TYPE_EXT:
				$extPath = isset($hrefArr['extPath']) ? $hrefArr['extPath'] : '';
				$out = self::hrefRow('', '', $ext_elem_Name, $extPath, $attr, $int_elem_Name);
				break;
			default:
				$extPath = isset($hrefArr['extPath']) ? $hrefArr['extPath'] : '';
				$out = self::hrefRow($intID_elem_Name, $intID, $intPath_elem_Name, $intPath, $attr, $int_elem_Name, true, $int) .
					self::hrefRow('', '', $ext_elem_Name, $extPath, $attr, $int_elem_Name, true, $int);
		}
		return '<table class="default">' . $out . '</table>';
	}

	private function htmlLinkInput($name, $i){
		$name .= 'default';

		$attribs = ['name' => $name];
		$elem = $this->getElement($name);
		$link = ($elem ? (is_array($elem) ? $elem : we_unserialize($elem)) : []) ?:
			['ctype' => 'text', 'type' => we_base_link::TYPE_EXT, 'href' => '#', 'text' => g_l('global', '[new_link]')];

		$img = new we_imageDocument();
		$content = self::getLinkContent($link, $this->ParentID, $this->Path, $img);

		$startTag = self::getLinkStartTag($link, $attribs, $this->ParentID, $this->Path, $img);
		$editbut = we_html_button::create_button(we_html_button::LINK_EDIT, "javascript:we_cmd('edit_link_at_class','" . $name . "','','" . $i . "');");
		$delbut = we_html_button::create_button(we_html_button::TRASH, "javascript:setScrollTo();we_cmd('object_delete_link_at_class','" . $GLOBALS['we_transaction'] . "','" . $i . "','" . $name . "')");
		if(!$content){
			$content = g_l('global', '[new_link]');
		}
		return '<table class="default defaultfont">
	<tr><td>' . ($startTag ?: '') . $content . '</a></td>
		<td width="5"></td><td>' . $editbut . $delbut . '</td>
	</tr>
</table>';
	}

	private function getObjectFieldHTML($ObjectID, $attribs, $editable = true){
		$pid = intval($this->getElement($ObjectID));
		if(!$editable){
			return '';
		}
		$textname = 'we_' . $this->Name . '_txt[' . $pid . '_path]';
		$idname = 'we_' . $this->Name . '_input[' . $ObjectID . 'default]';
		$myid = $this->getElement($ObjectID . 'default', self::ETYPE_TEXT);
		$DoubleNames = $this->includedObjectHasDoubbleFieldNames($pid);
		$path = $this->getElement(we_listview_base::OBJECT_FIELD_PREFIX . $pid . '_path') ?: ($myid ? f('SELECT Path FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $myid) : '');
		$rootDir = f('SELECT of.ID FROM ' . OBJECT_FILES_TABLE . ' of JOIN ' . OBJECT_TABLE . ' o ON (o.ID=of.TableID AND of.Path=o.Path) WHERE o.ID=' . $pid);
		$table = OBJECT_FILES_TABLE;
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,'" . $table . "','" . $idname . "','" . $textname . "','setHot','','" . $rootDir . "',WE().consts.contentTypes.OBJECT_FILE," . (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1) . ")");
		$delbutton = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value=''");
		/*
		  DAMD: der Autocompleter funktioniert hier nicht. Der HTML-Cokde wird dynamisch erzeugt das
		  $weSuggest =& weSuggest::getInstance();
		  $weSuggest->setAcId("TypeObject");
		  $weSuggest->setContentType("folder,objectFile");
		  $weSuggest->setInput($textname,$path);
		  $weSuggest->setMaxResults(20);
		  $weSuggest->setRequired(true);
		  $weSuggest->setResult($idname,$myid);
		  $weSuggest->setSelector(weSuggest::DocSelector);
		  $weSuggest->setTable($table);
		  $weSuggest->setWidth(246);
		  $weSuggest->setSelectButton($button);
		  $weSuggest->setTrashButton($delbutton);

		  return $weSuggest->getHTML();
		 */
		return we_html_tools::formElementTable('', we_html_tools::textInput($textname, $path, '', ['readonly' => 'readonly'], "text", 246, 0), we_html_element::hidden($idname, $myid), $button, $delbutton) . ($DoubleNames ? '<span style="color:red" >' . sprintf(g_l('modules_object', '[incObject_sameFieldname]'), implode(', ', $DoubleNames)) . '</span>' : '');
	}

	private function getCollectionFieldHTML($name, $defaultID, $editable = true){
		if(!$editable){
			return '';
		}

		$textname = 'we_' . $this->Name . '_txt[' . $name . '_path]';
		$idname = 'we_' . $this->Name . '_input[' . $name . ']';
		$path = id_to_path($defaultID, VFILE_TABLE);
		$rootDir = 0;

		$btnSelect = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,WE().consts.tables.VFILE_TABLE,'" . $idname . "','" . $textname . "','setHot','','" . $rootDir . "',WE().consts.contentTypes.COLLECTION," . (we_base_permission::hasPerm(we_base_perm::WORK_COLLECTIONS_CAN_SEE) ? 0 : 1) . ")");

		$btnDelete = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value=''");

		$btnNewCollection = we_html_button::create_button(we_html_button::ADD_COLLECTION, "javascript:top.we_cmd('edit_new_collection','write_back_to_opener," . $idname . ',' . $textname . "','',-1,'" . stripTblPrefix(VFILE_TABLE) . "');", '', '', false, false);

		/*
		  DAMD: der Autocompleter funktioniert hier nicht...
		  $weSuggest =& weSuggest::getInstance();
		  $weSuggest->setAcId("TypeObject");
		  $weSuggest->setContentType("folder,objectFile");
		  $weSuggest->setInput($textname,$path);
		  $weSuggest->setMaxResults(20);
		  $weSuggest->setRequired(true);
		  $weSuggest->setResult($idname,$myid);
		  $weSuggest->setSelector(weSuggest::DocSelector);
		  $weSuggest->setTable($table);
		  $weSuggest->setWidth(246);
		  $weSuggest->setSelectButton($btnSelect);
		  $weSuggest->setTrashButton($btnDelete);

		  return $weSuggest->getHTML();
		 */
		$buttons = $btnSelect . (we_base_permission::hasPerm(we_base_perm::WORK_COLLECTION_NEW) ? $btnNewCollection : '') . $btnDelete;

		return we_html_tools::formElementTable('', we_html_tools::textInput($textname, $path, '', ['readonly' => 'readonly'], 'text', 216, 0), $buttons);
	}

	private function getMultiObjectFieldHTML($name, $i, $f){
		$pid = intval($this->getElement($name . 'class'));

		$textname = 'we_' . $this->Name . '_txt[' . $name . '_path' . $f . ']';
		$idname = 'we_' . $this->Name . "_input[" . $name . 'defaultvalue' . $f . ']';
		$myid = $this->getElement($name . 'defaultvalue' . $f, we_contents_base::ETYPE_TEXT);

		$path = (
			$this->getElement(we_listview_base::OBJECT_FIELD_PREFIX . $name . '_path') ?:
			($myid ?
			f('SELECT Path FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $myid) :
			''
			) );
		$rootDir = f('SELECT of.ID FROM ' . OBJECT_FILES_TABLE . ' of JOIN ' . OBJECT_TABLE . ' o ON (o.ID=of.TableID AND of.Path=o.Path) WHERE o.ID=' . $pid);

		$selectObject = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,WE().consts.tables.OBJECT_FILES_TABLE,'" . $idname . "','" . $textname . "','setHot','','" . $rootDir . "',WE().consts.contentTypes.OBJECT_FILE," . (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1) . ")");
		$delbutton = we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value=''");

		$count = $this->getElement($name . 'count', we_contents_base::ETYPE_TEXT, 0);

		return '<tr>' .
			'<td>' . we_html_tools::textInput($textname, $path, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);", 'readonly' => 'readonly'], 'text', 146) . '</td>' .
			'<td>' .
			$selectObject .
			we_html_element::hidden($idname, $myid) .
			(($count + 1 < $this->getElement($name . 'max') || $this->getElement($name . 'max') == '') ?
			we_html_button::create_button(we_html_button::LIST_ADD, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_insert_meta_at_class','" . $GLOBALS['we_transaction'] . "','" . ($i) . "','" . $name . "','" . ($f) . "')") :
			we_html_button::create_button(we_html_button::LIST_ADD, "#", '', '', true)
			) .
			(($f > 0) ?
			we_html_button::create_button(we_html_button::DIRUP, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_up_meta_at_class','" . $GLOBALS['we_transaction'] . "','" . ($i) . "','" . $name . "','" . ($f) . "')") :
			we_html_button::create_button(we_html_button::DIRUP, "#", '', '', true)
			) .
			(($f < ($count)) ?
			we_html_button::create_button(we_html_button::DIRDOWN, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_down_meta_at_class','" . $GLOBALS['we_transaction'] . "','" . ($i) . "','" . $name . "','" . ($f) . "')") :
			we_html_button::create_button(we_html_button::DIRDOWN, "#", '', '', true)
			) .
			($count >= 1 ?
			we_html_button::create_button(we_html_button::TRASH, "javascript:_EditorFrame.setEditorIsHot(true);we_cmd('object_delete_meta_class','" . $GLOBALS['we_transaction'] . "','" . ($i) . "','" . $name . "','" . ($f) . "')") :
			$delbutton
			) .
			'</td></tr>';
	}

	function dhtmledit($name, $i = 0){
		return we_html_element::div([], we_html_button::create_button('attributes', "javascript:setScrollTo();we_cmd('object_editObjectTextArea','" . $i . "','" . $name . "','" . $GLOBALS['we_transaction'] . "');")) .
			$this->getWysiwygArea($name);
	}

	function getWysiwygArea($name){
		$rmfp = $this->getElement($name . 'removefirstparagraph');
		$attribs = [
			'bgcolor' => $this->getElement($name . 'bgcolor', we_contents_base::ETYPE_TEXT, ''),
			'class' => $this->getElement($name . 'class'),
			'classes' => $this->getElement($name . 'cssClasses'),
			'cols' => 30,
			'commands' => preg_replace('/ *, */', ',', $this->getElement($name . 'commands') ?: COMMANDS_DEFAULT),
			'contextmenu' => $this->getElement($name . 'contextmenu'),
			'fontnames' => $this->getElement($name . 'fontnames'),
			'fontsizes' => $this->getElement($name . 'fontsizes'),
			'formats' => $this->getElement($name . 'formats'),
			'gallerytemplates' => $this->getElement($name . 'gallerytemplates'),
			'height' => $this->getElement($name . 'height'),
			'inlineedit' => $this->getElement($name . 'inlineedit'),
			'removefirstparagraph' => $rmfp ? $rmfp === 'on' : REMOVEFIRSTPARAGRAPH_DEFAULT,
			'rows' => 3,
			'showmenus' => $this->getElement($name . 'showmenus', we_contents_base::ETYPE_TEXT, 'off'),
			'stylesheets' => $this->CSS,
			'templates' => $this->getElement($name . 'templates'),
			'tinyparams' => $this->getElement($name . 'tinyparams'),
			'width' => $this->getElement($name . 'width'),
			'wysiwyg' => $this->getElement($name . 'dhtmledit'),
			'xml' => $this->getElement($name . 'xml'),
		];

		return we_html_forms::weTextarea('we_' . $this->Name . '_input[' . $name . 'default]', $this->getElement($name . 'default', we_contents_base::ETYPE_TEXT), $attribs, $this->getElement($name . 'autobr'), '', (($this->CSS || $attribs['classes']) ? false : true), false, false, ($rmfp ? $rmfp === 'on' : REMOVEFIRSTPARAGRAPH_DEFAULT), "");
	}

	function add_user_to_field(array $ids, $name){
		$users = array_filter(explode(',', $this->getElement($name . 'users', we_contents_base::ETYPE_TEXT)));
		foreach($ids as $id){
			if($id && (!in_array($id, $users))){
				$users[] = $id;
			}
		}
		$this->setElement($name . 'users', implode(',', $users));
	}

	function del_user_from_field($id, $name){
		$users = array_filter(explode(',', $this->getElement($name . 'users', we_contents_base::ETYPE_TEXT)));
		$pos = array_search($id, $users);
		if($pos !== false){
			unset($users[$pos]);
		}
		$this->setElement($name . 'users', implode(',', $users));
	}

	private function formUsers1(we_base_jsCmd $jsCmd, $name, $nr = 0){
		$users = $this->getElement($name . 'users') ? array_filter(explode(',', $this->getElement($name . 'users'))) : [];
		$content = '';
		if($users){
			$this->DB_WE->query('SELECT ID,Path,(IF(IsFolder,"' . we_base_ContentTypes::USERGROUP . '",(IF(Alias>0,"' . we_base_ContentTypes::USERALIAS . '","' . we_base_ContentTypes::USER . '")))) AS ContentType FROM ' . USER_TABLE . ' WHERE ID IN (' . implode(',', $users) . ')');
			$allUsers = array_flip($users);
			while($this->DB_WE->next_record(MYSQLI_ASSOC)){
				$content .= '<tr><td style="width:20px;" class="userIcon" data-contenttype="' . $this->DB_WE->f('ContentType') . '"></td><td style="width:324px;">' . $this->DB_WE->f('Path') . '</td><td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_del_user_from_field','" . $GLOBALS['we_transaction'] . "','" . $nr . "'," . $this->DB_WE->f('ID') . ",'" . $name . "');") . '</td></tr>';
				unset($allUsers[$this->DB_WE->f('ID')]);
			}
//all non-existing users
			foreach(array_keys($allUsers) as $user){
				$content .= '<tr><td style="width:20px;"></td><td>Unknown</td><td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_del_user_from_field','" . $GLOBALS['we_transaction'] . "','" . $nr . "'," . $user . ",'" . $name . "');") . '</td></tr>';
			}
		} else {
			$content .= '<tr><td style="width:20px;" class="userIcon" data-contenttype="' . we_base_ContentTypes::USERGROUP . '"></td><td style="width:324px;">' . g_l('weClass', '[everybody]') . '</td><td></td></tr>';
		}

		$textname = 'we_' . $this->Name . '_input[' . $name . 'usertext]';
		$idname = 'we_' . $this->Name . '_input[' . $name . 'userid]';

		$cmd = "object_add_user_to_field," . $GLOBALS['we_transaction'] . "," . $nr . ",," . $name;

		$jsCmd->addCmd('setIconOfDocClass', 'userIcon');
		return '<table class="default"><tr><td>
			<div class="multichooser">
				<table class="default" style="width:388px;margin:5px;">' .
			$content . '
				</table></div>
</td></tr><tr><td style="text-align:right">' .
			we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('object_del_all_users','" . $GLOBALS['we_transaction'] . "','" . $nr . "','" . $name . "')", '', '', ($users ? false : true)) .
			we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_users_selector',document.we_form.elements['" . $idname . "'].value,'" . $textname . "','','" . $idname . "','" . $cmd . "','','',1)") .
			we_html_element::hiddens([$idname => 0, $textname => '']) .
			'</td></tr></table>';
	}

	private function formUsers(we_base_jsCmd $jsCmd, $canChange = true){
		$users = array_filter(explode(',', $this->Users));
		$usersReadOnly = array_filter(explode(',', $this->UsersReadOnly));
		$content = '<table class="default defaultfont" style="width:388px;margin:5px;">';

		if($users){
			$this->DB_WE->query('SELECT ID,Path,(IF(IsFolder,"' . we_base_ContentTypes::USERGROUP . '",(IF(Alias>0,"' . we_base_ContentTypes::USERALIAS . '","' . we_base_ContentTypes::USER . '")))) AS ContentType FROM ' . USER_TABLE . ' WHERE ID IN(' . implode(',', $users) . ')');
			$allUsers = $this->DB_WE->getAllFirst(true, MYSQLI_ASSOC);
			foreach($allUsers as $user => $data){
				$content .= '<tr><td style="width:20px;" class="userIcon" data-contenttype="' . $data['ContentType'] . '"></td><td  style="width:333px;">' . $data['Path'] . '</td><td style="width:20px;">' .
					($canChange ?
					we_html_element::hidden('we_users_read_only[' . $user . ']', (!empty($usersReadOnly[$user])) ? $usersReadOnly[$user] : "") .
					'<input type="checkbox" value="1" name="wetmp_users_read_only[' . $user . ']"' . ( (!empty($usersReadOnly[$user])) ? ' checked' : '') . ' onclick="this.form.elements[\'we_users_read_only[' . $user . ']\'].value=(this.checked ? 1 : 0);_EditorFrame.setEditorIsHot(true);" />' :
					'<i class="fa fa-' . ($usersReadOnly[$user] ? 'check-' : '') . 'square-o wecheckIcon disabled"></i>'
					) . '</td><td style="width:80px;">' . g_l('weClass', '[readOnly]') . '</td><td>' .
					($canChange ?
					we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('users_del_user','" . $user . "');_EditorFrame.setEditorIsHot(true);") :
					''
					) . '</td></tr>';
			}
		} else {
			$content .= '<tr><td style="width:20px;" class="userIcon" data-contenttype="we/user"></td><td class="defaultfont">' . g_l('weClass', '[onlyOwner]') . '</td><td></td></tr>';
		}
		$content .= '</table>';

		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('object_del_all_users','')", '', '', $this->Users ? false : true);
//FIXME: bad encoded field!

		$addbut = !$canChange ? '' :
			we_html_element::hiddens(['userIDTmp' => '', 'userNameTmp' => '']) . we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_users_selector',document.we_form.elements['userIDTmp'].value,'userNameTmp','',document.we_form.elements['userIDTmp'].value,'users_add_user','','',1)");

		$content = '<table class="default">
<tr><td><div style="width:506px;" class="multichooser">' . $content . '</div></td></tr>' .
			($canChange ? '<tr><td style="text-align:right;padding-top:1em;">' . $delallbut . $addbut . '</td></tr>' : "") . '</table>';
		$jsCmd->addCmd('setIconOfDocClass', 'userIcon');
		return we_html_tools::formElementTable(g_l('weClass', '[otherowners]'), $content);
	}

	function del_all_users($name){
		if($name === ''){
			$this->Users = '';
			return;
		}
		$this->setElement($name . 'users', '');
	}

	function add_user($id){
		$ids = is_array($id) ? $id : explode(',', $id);
		$this->Users = implode(',', array_unique(array_filter(explode(',', $this->Users)) + $ids, SORT_NUMERIC));
	}

	function del_user($id){
		$users = array_filter(explode(',', $this->Users));
		if(($pos = array_search($id, $users, false)) == false){
			return;
		}
		unset($users[$pos]);
		$this->Users = implode(',', $users);
	}

	function add_css(array $id){
		$this->CSS = implode(',', array_filter(
				array_unique(
					array_merge(explode(',', $this->CSS), $id)
			))
		);
	}

	function del_css($id){
		$css = explode(',', $this->CSS);
		if(($pos = array_search($id, $css, false)) === false){
			return;
		}
		unset($css[$pos]);
		$this->CSS = implode(',', $css);
	}

	private function getImageHTML($name, $defaultname, $i = 0){
		$img = new we_imageDocument();
		$id = $defaultname;
		if($id){
			$img->initByID($id, FILE_TABLE, false);
		} else {
			$img->we_new();
		}

		$fname = 'we_' . $this->Name . '_input[' . $name . ']';
		$cmd = 'object_reload_entry_at_class,' . $GLOBALS['we_transaction'] . ',' . $i;
		$content = '<input type=hidden name="' . $fname . '" value="' . $defaultname . '" />' .
			we_html_button::create_button(we_html_button::IMAGE_SELECT, "javascript:we_cmd('we_selector_image','" . $id . "',WE().consts.tables.FILE_TABLE,'" . $fname . "','','" . $cmd . "','',0,WE().consts.contentTypes.IMAGE)") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_class','" . $GLOBALS['we_transaction'] . "','" . $i . "','" . $name . "')") .
			'<br/>' . $img->getHtml();

// gets thumbnails and shows a select field, if there are any:
		$thumbList = $GLOBALS['DB_WE']->getAllFirstq('SELECT ID,Text FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text', false);
		if($thumbList){
			$thumbList = ['-'] + $thumbList;
			$content .= '<br/>' . g_l('modules_object', '[use_thumbnail_preview]') . ":<br/>";
			$tmp = $this->getElement($name . 'Thumb');
			$currentSelection = ($tmp && isset($thumbList[$tmp]) ? $tmp : '');

			$content .= we_html_tools::select('we_' . $this->Name . '_input[' . $name . 'Thumb]', $thumbList, 1, $currentSelection, '', ['onchange' => "_EditorFrame.setEditorIsHot(true);"]);
		}
		return $content;
	}

	private function getFlashmovieHTML($name, $defaultname, $i = 0){
		$img = new we_flashDocument();
		$id = $defaultname;
		$img->initByID($id, FILE_TABLE, false);
		$fname = 'we_' . $this->Name . '_input[' . $name . ']';
		$cmd = 'object_reload_entry_at_class,' . $GLOBALS['we_transaction'] . ',' . $i;

		return '<input type=hidden name="' . $fname . '" value="' . $defaultname . '" />' .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document','" . $id . "',WE().consts.tables.FILE_TABLE,'" . $fname . "','','" . $cmd . "','',0,WE().consts.contentTypes.FLASH)") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_class','" . $GLOBALS['we_transaction'] . "','" . $i . "','" . $name . "')") .
			'<br/>' . $img->getHtml();
	}

		private function getVideoHTML($name, $defaultname, $i = 0){
		$video = new we_document_video();
		$id = $defaultname;
		$video->initByID($id, FILE_TABLE, false);
		$fname = 'we_' . $this->Name . '_input[' . $name . ']';
		$cmd = 'object_reload_entry_at_class,' . $GLOBALS['we_transaction'] . ',' . $i;

		return '<input type=hidden name="' . $fname . '" value="' . $defaultname . '" />' .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document','" . $id . "',WE().consts.tables.FILE_TABLE,'" . $fname . "','','" . $cmd . "','',0,WE().consts.contentTypes.VIDEO)") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_class','" . $GLOBALS['we_transaction'] . "','" . $i . "','" . $name . "')") .
			'<br/>' . $video->getHtml();
	}

		private function getAudioHTML($name, $defaultname, $i = 0){
		$audio = new we_document_audio();
		$id = $defaultname;
		$audio->initByID($id, FILE_TABLE, false);
		$fname = 'we_' . $this->Name . '_input[' . $name . ']';
		$cmd = 'object_reload_entry_at_class,' . $GLOBALS['we_transaction'] . ',' . $i;

		return '<input type=hidden name="' . $fname . '" value="' . $defaultname . '" />' .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document','" . $id . "',WE().consts.tables.FILE_TABLE,'" . $fname . "','','" . $cmd . "','',0,WE().consts.contentTypes.AUDIO)") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_class','" . $GLOBALS['we_transaction'] . "','" . $i . "','" . $name . "')") .
			'<br/>' . $audio->getHtml();
	}


	private function getBinaryHTML($name, $defaultname, $i = 0){
		$other = new we_otherDocument();
		$id = $defaultname; //$this->getElement($defaultname);
		$other->initByID($id, FILE_TABLE, false);
		$fname = 'we_' . $this->Name . '_input[' . $name . ']';
		$cmd = 'object_reload_entry_at_class,' . $GLOBALS['we_transaction'] . ',' . $i;

		return '<input type=hidden name="' . $fname . '" value="' . $defaultname . '" />' .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document','" . $id . "',WE().consts.tables.FILE_TABLE,'" . $fname . "','','" . $cmd . "','',0,WE().consts.contentTypes.APPLICATION)") .
			we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('object_remove_image_at_class','" . $GLOBALS['we_transaction'] . "','" . $i . "','" . $name . "');") .
			'<br/>' . $other->getHtml();
	}

	private function formDefaultName(we_base_jsCmd $jsCmd){
		$select = '';

		if(($anz = $this->getElement('Defaultanzahl', we_contents_base::ETYPE_TEXT, -1)) >= 0){
			$this->DefaultText = '';
			for($i = 0; $i <= $anz; $i++){
				$was = 'DefaultText_' . $i;
				if(($dat = $this->getElement($was, we_contents_base::ETYPE_TEXT, '')) !== ''){
					if(stristr($dat, 'unique')){
						$unique = $this->getElement('unique_' . $i, we_contents_base::ETYPE_TEXT);
						$dat = '%' . str_replace('%', '', $dat) . ( $unique > 0 ? $unique : 16) . "%";
						$this->setElement($was, $dat, 'defaultText');
					}
					$this->DefaultText .= $dat;
				}
			}
		}

		$all = $this->DefaultText;
		$zahl = 0;
		$regs = [];
		$vals = g_l('modules_object', '[value]');

		while($all){
			if(preg_match('/^%([^%]+)%/', $all, $regs)){
				$all = substr($all, strlen($regs[1]) + 2);
				$key = $regs[1];
				if(preg_match('/unique([^%]*)/', $key, $regs)){
					$anz = (!$regs[1] ? 16 : abs($regs[1]));
					$unique = substr(md5(uniqid('', true)), 0, min($anz, 32));
					$text = preg_replace('/%unique[^%]*%/', $unique, (isset($text) ? $text : ''));
					$select .= we_html_tools::select('we_' . $this->Name . '_input[DefaultText_' . $zahl . ']', $vals, 1, '%unique%', false, ['onchange' => '_EditorFrame.setEditorIsHot(true);we_cmd(\'reload_editpage\');']) .
						we_html_tools::textInput('we_' . $this->Name . '_input[unique_' . $zahl . ']', $anz, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text', 140);
				} else {
					$select .= we_html_tools::select('we_' . $this->Name . '_input[DefaultText_' . $zahl . ']', $vals, 1, '%' . $key . '%', false, ['onchange' => '_EditorFrame.setEditorIsHot(true);we_cmd(\'reload_editpage\');']);
				}
			} else if(preg_match('/^([^%]+)/', $all, $regs)){
				$all = substr($all, strlen($regs[1]));
				$key = $regs[1];
				$select .= we_html_tools::select('textwert_' . $zahl, $vals, 1, 'Text', false, ['onchange' => "_EditorFrame.setEditorIsHot(true); document.we_form.elements['we_" . $this->Name . '_input[DefaultText_' . $zahl . "]'].value = this.options[this.selectedIndex].value; we_cmd('reload_editpage');"]) .
					we_html_tools::textInput('we_' . $this->Name . '_input[DefaultText_' . $zahl . ']', $key, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"], 'text', 140);
			}

			$select .= we_html_element::br();
			$zahl++;
		}

		return $select . we_html_tools::select('we_' . $this->Name . '_input[DefaultText_' . $zahl . ']', $vals, 1, '', false, ['onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"]) .
			we_html_element::hidden('we_' . $this->Name . '_input[Defaultanzahl]', $zahl);
	}

	private function formDefaultUrl(we_base_jsCmd $jsCmd){
		$select2 = '';
		if(($anz = $this->getElement('DefaultanzahlUrl', we_contents_base::ETYPE_TEXT, -1)) >= 0){ //Fix #9964 0 is an corret value!
			$allFields = array_merge(self::URL_UNIQUE, self::URL_FIELDS);
			$this->DefaultUrl = '';

			for($i = 0; $i <= $anz; $i++){
				$was = 'DefaultUrl_' . $i;
				if(($curDat = $this->getElement($was, we_contents_base::ETYPE_TEXT, ''))){
					foreach($allFields as $key => $len){
						if(stristr($curDat, $key)){
							$curDat = '%' . str_replace('%', '', $curDat) . ($this->getElement($key . '_' . $i, we_contents_base::ETYPE_TEXT) ?: $len) . '%';
						}
					}
					$this->setElement($was, $curDat);
					$this->DefaultUrl .= $curDat;
				}
			}
		}

		$all = $this->DefaultUrl;
		$zahl = 0;

		while($all){
			if(preg_match('/^%([^%]+)%/', $all, $regs)){
				$all = substr($all, strlen($regs[1]) + 2);
				$data = $regs[1];
				$found = false;
				foreach(self::URL_UNIQUE as $key => $len){
					if(preg_match('/' . $key . '([^%]*)/', $data, $regs)){
						$anz = (!$regs[1] ? $len : abs($regs[1]));
						$unique = substr(md5(uniqid('', true)), 0, min($anz, 32));
						$text = preg_replace('/%' . $key . '[^%]*%/', $unique, (isset($text) ? $text : ""));
						$select2 .= we_html_tools::select('we_' . $this->Name . '_input[DefaultUrl_' . $zahl . ']', g_l('modules_object', '[url]'), 1, '%' . $key . '%', '', [
								'onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"]) .
							we_html_tools::textInput('we_' . $this->Name . '_input[' . $key . '_' . $zahl . ']', $anz, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"], "text", 140);
						$found = true;
						break;
					}
				}
				if(!$found){
					foreach(self::URL_FIELDS as $key => $len){
						if(preg_match('/' . $key . '([^%]*)/', $data, $regs)){
							$anz = (!$regs[1] ? $len : abs($regs[1]));
							$select2 .= we_html_tools::select('we_' . $this->Name . '_input[DefaultUrl_' . $zahl . ']', g_l('modules_object', '[url]'), 1, '%' . $key . '%', '', [
									'onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"], true, 140) . '&nbsp;' .
								we_html_tools::textInput('we_' . $this->Name . '_input[' . $key . '_' . $zahl . ']', $anz, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"], "text", 140);
							$found = true;
							break;
						}
					}
					if(!$found){
						$select2 .= we_html_tools::select('we_' . $this->Name . '_input[DefaultUrl_' . $zahl . ']', g_l('modules_object', '[url]'), 1, '%' . $data . '%', '', [
								'onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"], true, 140) . "&nbsp;";
					}
				}
			} else if(preg_match('/^([^%]+)/', $all, $regs)){
				$all = substr($all, strlen($regs[1]));
				$data = $regs[1];
				$select2 .= we_html_tools::select('textwert_' . $zahl, g_l('modules_object', '[url]'), 1, 'Text', '', ['onchange' => "_EditorFrame.setEditorIsHot(true); document.we_form.elements['we_" . $this->Name . '_input[DefaultUrl_' . $zahl . "]'].value = this.options[this.selectedIndex].value; we_cmd('reload_editpage');"], true, 140) . "&nbsp;" .
					we_html_tools::textInput('we_' . $this->Name . '_input[DefaultUrl_' . $zahl . ']', $data, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"], "text", 140);
			}

			$select2 .= we_html_element::br();
			$zahl++;
		}

		return $select2 . we_html_tools::select('we_' . $this->Name . '_input[DefaultUrl_' . $zahl . ']', g_l('modules_object', '[url]'), 1, '', '', ['onchange' => "_EditorFrame.setEditorIsHot(true);we_cmd('reload_editpage');"]) .
			we_html_element::hidden('we_' . $this->Name . '_input[DefaultanzahlUrl]', $zahl);
	}

	private function formDefault(we_base_jsCmd $jsCmd){
		return $this->formTriggerDocument(true);
	}

	function formRestrictUsers($canChange = true){
		return ($canChange ?
			we_html_element::hidden('we_' . $this->Name . '_RestrictUsers', abs($this->RestrictUsers)) .
			we_html_forms::checkbox(1, $this->RestrictUsers ? true : false, 'tmpwe_' . $this->Name . '_RestrictUsers', g_l('weClass', '[limitedAccess]'), true, "defaultfont", "_EditorFrame.setEditorIsHot(true);this.form.elements['we_" . $this->Name . "_RestrictUsers'].value=(this.checked ? '1' : '0');we_cmd('reload_editpage');") :
			'<table class="default"><tr><td><i class="fa fa-' . ($this->RestrictUsers ? 'check-' : '') . 'square-o wecheckIcon disabled"></i></td><td class="defaultfont">&nbsp;' . g_l('weClass', '[limitedAccess]') . '</td></tr></table>'
			);
	}

	public function formPath($disablePath = false, $notSetHot = false, $extra = ''){
		return '<table class="default">
	<tr><td>' . $this->formInputField('', 'Text', g_l('modules_object', '[classname]'), 0, 255, ['onchange' => "pathOfDocumentChanged(true);"]) . '</td><td></td><td></td></tr>
</table>';
	}

	private function formWorkspaces(we_base_jsCmd $jsCmd){
//remove not existing workspaces - deal with templates as well
		$arr = makeArrayFromCSV($this->Workspaces);
		$defaultArr = makeArrayFromCSV($this->DefaultWorkspaces);
		$tmplArr = array_filter(explode(',', $this->Templates));
		$newTmplArr = $newDefaultArr = $newArr = [];

//    check if workspace exists - correct templates if neccessary !!
		for($i = 0; $i < count($arr); $i++){
			if(we_base_file::isWeFile($arr[$i], FILE_TABLE, $this->DB_WE)){
				$newArr[] = $arr[$i];
				if(in_array($arr[$i], $defaultArr)){
					$newDefaultArr[] = $arr[$i];
				}
				$newTmplArr[] = (isset($tmplArr[$i]) ? $tmplArr[$i] : '');
			}
		}

		$this->Workspaces = implode(',', $newArr);
		$this->Templates = implode(',', $newTmplArr);
		$this->DefaultWorkspaces = implode(',', $newDefaultArr);

		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_directory','',WE().consts.tables.FILE_TABLE,'','','object_add_workspace','','','',1)");

		$obj = new we_chooser_multiDirTemplateAndDefault(600, $this->Workspaces, 'object_del_workspace', $addbut, get_ws(FILE_TABLE), $this->Templates, 'we_' . $this->Name . '_Templates', '', get_ws(TEMPLATES_TABLE), 'we_' . $this->Name . '_DefaultWorkspaces', $this->DefaultWorkspaces);
		$obj->CanDelete = true;
		$obj->create = 1;
		$content = $obj->get($jsCmd);

		if(!empty($GLOBALS['we']['DEL_WORKSPACE_ERROR'])){
			unset($GLOBALS['we']['DEL_WORKSPACE_ERROR']);
			$jsCmd->addMsg(g_l('weClass', '[we_del_workspace_error]'), we_base_util::WE_MESSAGE_ERROR);
		}
		return $content;
	}

	private function formWorkspacesFlag(){
		return '<div style="margin-bottom:8px;">' . we_html_forms::radiobutton(1, $this->WorkspaceFlag == 1, 'we_' . $this->Name . '_WorkspaceFlag', g_l('modules_object', '[behaviour_all]')) . '</div><div>' .
			we_html_forms::radiobutton(0, $this->WorkspaceFlag != 1, 'we_' . $this->Name . '_WorkspaceFlag', g_l('modules_object', '[behaviour_no]')) . '</div>';
	}

	private function formCSS(we_base_jsCmd $jsCmd){
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document', 0, WE().consts.tables.FILE_TABLE,'','','object_add_css','','',WE().consts.contentTypes.CSS, 1,1)");
		$css = new we_chooser_multiDir(510, $this->CSS, 'object_del_css', $addbut, '', 'ContentType', FILE_TABLE);
		return $css->get($jsCmd);
	}

	protected function formCopyDocument(){
		return we_html_element::hidden('we_' . $this->Name . '_CopyID', $this->CopyID) .
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['we_" . $this->Name . "_CopyID'].value,'" . $this->Table . "','we_" . $this->Name . "_CopyID','','copyDocumentSelect','',0,'" . $this->ContentType . "');");
	}

	function copyDoc($id){
		if(!$id){
			return false;
		}
		$doc = new we_object();
		$doc->InitByID($id, $this->Table, self::LOAD_TEMP_DB);
		if($this->ID == 0){
			foreach($this->persistent_slots as $name){
				$this->{$name} = isset($doc->{$name}) ? $doc->{$name} : '';
			}
			$this->CreationDate = time();
			$this->CreatorID = $_SESSION['user']['ID'];
			$this->ID = 0;
			$this->OldPath = '';
			$this->Published = 1;
			$this->Text .= '_copy';
			$this->Path = $this->ParentPath . $this->Text;
			$this->OldPath = $this->Path;
		}
		$this->elements = $doc->elements;
		$newEls = [];
		foreach($this->elements as $n => $e){
			if(strtolower(substr($n, 0, 9)) === 'wholename'){
				$newEls[] = $e['dat'];
			}
		}
		if($newEls){
			$this->setElement('neuefelder', array_merge($this->getElement('neuefelder', we_contents_base::ETYPE_TEXT, []), $newEls));
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
		$this->Category = $doc->Category;
		return true;
	}

	function changeTempl_ob($nr, $id){
		$arr = explode(',', $this->Templates);
		$arr[$nr] = $id;

		$this->Templates = implode(',', $arr);
	}

	function add_workspace($id){
		$wsp = trim($this->Workspaces, ',');
		$this->Workspaces = implode(',', ($wsp !== '' ? array_unique(array_merge(explode(',', $wsp), [$id]), SORT_NUMERIC) : [$id]));
	}

	function del_workspace($id){
		if(f('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=0 AND TableID=' . intval($this->ID) . ' AND FIND_IN_SET(' . intval($id) . ',Workspaces) LIMIT 1', '', $this->DB_WE)){
			$GLOBALS['we']['DEL_WORKSPACE_ERROR'] = true;
			return;
		}

		$workspaces = $this->Workspaces === '' ? [] : explode(',', $this->Workspaces);
		$defaultWorkspaces = $this->DefaultWorkspaces === '' ? [] : explode(',', $this->DefaultWorkspaces);
		$Templates = $this->Templates === '' ? [] : explode(',', $this->Templates);
		if(($pos = array_search($id, $workspaces)) !== false){
			unset($workspaces[$pos]);
			if(in_array($id, $defaultWorkspaces)){
				unset($defaultWorkspaces[array_search($id, $defaultWorkspaces)]);
			}
			unset($Templates[$pos]);
		}

		$this->Workspaces = implode(',', $workspaces);
		$this->DefaultWorkspaces = implode(',', $defaultWorkspaces);
		$this->Templates = implode(',', $Templates);
	}

	public function we_initSessDat(array $sessDat){
//	charset must be in other namespace -> for header !!!
		$this->setElement('Charset', (isset($sessDat[0]['SerializedArray']['elements']['Charset']) ? $sessDat[0]['SerializedArray']['elements']['Charset']['dat'] : ""), 'attrib');
		parent::we_initSessDat($sessDat);
		$this->setSort();
	}

	protected function getContentData(){
		if(!$this->ID){
			return;
		}
		$f = 0;
		$rec = getHash('SELECT DefaultCategory,DefaultText,DefaultDesc,DefaultTitle,DefaultUrl,DefaultUrlfield0,DefaultUrlfield1,DefaultUrlfield2,DefaultUrlfield3,DefaultTriggerID,DefaultKeywords,DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->ID, $this->DB_WE);

		$this->setSort();

		$this->DefaultValues = $rec['DefaultValues'];

		$vals = we_unserialize($this->DefaultValues);
		$names = array_keys($vals);

		foreach($names as $name){
			if($name === 'WE_CSS_FOR_CLASS'){
				$this->CSS = $vals[$name];
			}
			if(isset($vals[$name]) && is_array($vals[$name])){
				$this->setElement($name . 'count', ((!empty($vals[$name]['meta'])) ? (count($vals[$name]['meta']) - 1) : 0));
				if(isset($vals[$name]['meta']) && is_array($vals[$name]['meta'])){
					$keynames = array_keys($vals[$name]['meta']);
					for($ll = 0; $ll <= count($vals[$name]['meta']); $ll++){
						$this->setElement($name . 'defaultkey' . $ll, (isset($keynames[$ll]) ? $keynames[$ll] : ''));
						$this->setElement($name . 'defaultvalue' . $ll, (isset($keynames[$ll]) ? $vals[$name]['meta'][$keynames[$ll]] : ''));
					}
				}
			}
		}

		$this->DefaultCategory = $rec['DefaultCategory'];
		$this->Category = $this->DefaultCategory;
		$this->SerializedArray = we_unserialize($rec['DefaultValues']);

//	charset must be in other namespace -> for header !!!
		$this->setElement('Charset', (empty($this->SerializedArray['elements']['Charset']['dat']) ? '' : $this->SerializedArray['elements']['Charset']['dat']));

		$this->WorkspaceFlag = empty($this->SerializedArray['WorkspaceFlag']) ? '' : $this->SerializedArray['WorkspaceFlag'];
		$this->setElement('title', $rec['DefaultTitle']);
		$this->setElement('desc', $rec['DefaultDesc']);
		$this->setElement('keywords', $rec['DefaultKeywords']);

		$this->DefaultText = $rec['DefaultText'];
		$this->DefaultUrl = $rec['DefaultUrl'];

		$this->setElement('urlfield0', $rec['DefaultUrlfield0']);
		$this->setElement('urlfield1', $rec['DefaultUrlfield1']);
		$this->setElement('urlfield2', $rec['DefaultUrlfield2']);
		$this->setElement('urlfield3', $rec['DefaultUrlfield3']);
		$this->setElement('triggerid', $rec['DefaultTriggerID']);
		$this->DefaultTriggerID = $rec['DefaultTriggerID'];

		$ctable = OBJECT_X_TABLE . intval($this->ID);
		$tableInfo = $this->DB_WE->metadata($ctable);
		$fields = ['max' => '',
			'default' => '',
			'defaultThumb' => '',
			//'autobr' => '',
			'rootdir' => '',
			'defaultdir' => '',
			'dhtmledit' => 'off',
			'showmenus' => 'off',
			'commands' => '',
			'contextmenu' => '',
			'height' => 50,
			self::ELEMENT_WIDTH => 200,
			'bgcolor' => '',
			'class' => '',
			'cssClasses' => '',
			'fontnames' => '',
			'fontsizes' => '',
			'formats' => '',
			'tinyparams' => '',
			'templates' => '',
			'gallerytemplates' => '',
			'xml' => '',
			'removefirstparagraph' => '',
			'forbidhtml' => 'off',
			'inlineedit' => '',
			'users' => '',
			'required' => '',
			'editdescription' => '',
			'int' => '',
			'intID' => '',
			'hreftype' => '',
			'hreffile' => '',
			'hrefdirectory' => 'false',
			'shopcatField' => '',
			'shopcatShowPath' => 'true',
			'shopcatRootdir' => '',
			'shopcatLimitChoice' => 0,
			'intPath' => '',
		];
		$sort = $this->getElement('we_sort');

		foreach($tableInfo as $info){
			list($type, $name) = explode('_', $info['name'], 2);
			if($name && $type != 'OF' && $type != 'variant'){

				$this->setElement($info['name'], $name, we_contents_base::ETYPE_TEXT);
				$this->setElement('wholename' . array_search($f, $sort), $info['name'], we_contents_base::ETYPE_TEXT);
				$this->setElement($info['name'] . self::ELEMENT_LENGHT, $info['len'], we_contents_base::ETYPE_TEXT);
				$this->setElement($info['name'] . self::ELEMENT_TYPE, $type, we_contents_base::ETYPE_TEXT);
				$typeLen = strtoupper($info['type']) . (strpos($info['flags'], 'unsigned') !== false || (defined('MYSQLI_UNSIGNED_FLAG') && (($info['flags'] & MYSQLI_UNSIGNED_FLAG) > 0)) ? '_U' : '');
				$this->setElement($info['name'] . 'typeLen', $typeLen, we_contents_base::ETYPE_TEXT);

				if(isset($vals[$info['name']]['variant'])){
					$this->setElement($info['name'] . 'variant', $vals[$info['name']]['variant']);
				}
				foreach($fields as $field => $def){
					$this->setElement($info['name'] . $field, isset($vals[$info['name']][$field]) ? $vals[$info['name']][$field] : $def);
				}

				$f++;
			}
		}
	}

	protected function setPersistentSlot($name, $value){
		if(in_array($name, $this->persistent_slots)){
			$this->$name = $value;
			return;
		}
		if($name === 'Templates_0'){
			$tmp = $dfWs = [];
			$wsp = makeArrayFromCSV($this->Workspaces);
			foreach($wsp as $i => $cur){
				$tmp[] = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_Templates_' . $i);
				if(we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_DefaultWorkspaces_' . $i) !== false){
					$dfWs[] = $cur;
				}
			}
			$this->Templates = implode(',', $tmp);
			$this->DefaultWorkspaces = implode(',', $dfWs);
		}
	}

	protected function setText(){
// do nothing here!
	}

	protected function isFilenameEmpty(){
		return ($this->Text === '');
	}

	protected function isFilenameNotValid(){
		static $allowedReplace = [
			'%ID%',
			'%d%',
			'%j%',
			'%m%',
			'%y%',
			'%Y%',
			'%n%',
			'%h%',
			'%H%',
			'%g%',
			'%G%',
			'%unique%'
		];

		foreach($this->nextElement('defaultText') as $k){
			if(substr($k, 0, 12) === 'DefaultText_'){
				$end = substr($k, 12, strlen($k));
				if(isset($_REQUEST['textwert_' . $end]) && ($v = $this->getElement($k)) && !in_array($v, $allowedReplace) && preg_match('/[^\w\-.]/', $v)){
					return true;
				}
			}
		}
		return (preg_match('/[^\w\-.]/', $this->Text));
	}

	protected function isFilenameNotAllowed(){
		return false;
	}

	protected function isFilenameDouble(){
		return f('SELECT 1 FROM ' . $this->Table . ' WHERE ParentID=' . intval($this->ParentID) . ' AND Text="' . $this->DB_WE->escape($this->Text) . '" AND ID!=' . intval($this->ID), '', $this->DB_WE);
	}

	function checkPathDiffAndCreate(){
		return true;
	}

	protected function hasDoubbleFieldNames(){
		$sort = $this->getElement('we_sort');
		if(empty($sort)){
			return false;
		}
		$usedNames = [];
		foreach(array_keys($sort) as $elem){
			$name = $this->getElement('wholename' . $elem, we_contents_base::ETYPE_TEXT);
			$foo = $this->getElement($name, we_contents_base::ETYPE_TEXT);
			if(empty($foo)){ //we don't allow field names evaluating to false, report them as double ;-)
				return false;
			}
			if(!in_array($foo, $usedNames)){
				$usedNames[] = $foo;
				continue;
			}
			switch($this->getElement($name . 'dtype', we_contents_base::ETYPE_TEXT)){
				case self::TYPE_OBJECT:
					return f('SELECT Path FROM ' . OBJECT_TABLE . ' WHERE ID=' . $foo, '', $this->DB_WE);
				default:
					return $foo;
			}
		}
		return false;
	}

	protected function fieldNameNotValid(){
		$sort = $this->getElement('we_sort');
		if(empty($sort)){
			return false;
		}
		foreach(array_keys($sort) as $elem){
			$name = $this->getElement('wholename' . $elem, we_contents_base::ETYPE_TEXT);
			if(we_database_base::isReservedField($name)){
				return $name;
			}
		}
		return false;
	}

	public function checkFieldsOnSave(){
		if(($n = $this->hasDoubbleFieldNames())){
			return sprintf(g_l('weEditor', '[doubble_field_alert]'), $n);
		}

		if(($n = $this->fieldNameNotValid())){
			return sprintf(g_l('weEditor', '[reserved_field_alert]'), $n);
		}

		if($this->triggerIdNotValdid()){
			return g_l('weEditor', '[triggerIdNotValid]');
		}
		if($this->triggerDocNotDynamic()){
			return g_l('weEditor', '[triggerDocNotDynamic]');
		}

		return parent::checkFieldsOnSave();
	}

	private function triggerIdNotValdid(){
		return $this->DefaultUrl && (!$this->DefaultTriggerID || !f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($this->DefaultTriggerID) . ' AND IsFolder=0', '', $this->DB_WE));
	}

	private function triggerDocNotDynamic(){
		return $this->DefaultUrl && (!f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($this->DefaultTriggerID) . ' AND IsFolder=0 AND IsDynamic=1', '', $this->DB_WE));
	}

	private function includedObjectHasDoubbleFieldNames($incClass){
		$sort = $this->getElement('we_sort');
		$usedNames = $doubleNames = [];
		if(is_array($sort)){
			foreach(array_keys($sort) as $elem){
				$foo = $this->getElement($this->getElement('wholename' . $elem), we_contents_base::ETYPE_TEXT);
				$usedNames[] = $foo;
			}
		}
		$incclassobj = new we_object();
		$incclassobj->initByID($incClass, $this->Table);
		$isort = $incclassobj->getElement('we_sort');
		if(is_array($isort) && $isort){
			foreach(array_keys($isort) as $elem){
				$foo = $incclassobj->getElement($incclassobj->getElement('wholename' . $elem), we_contents_base::ETYPE_TEXT);
				if(in_array($foo, $usedNames)){
					$doubleNames[] = $foo;
				}
			}
		}
		return ($doubleNames ?: false);
	}

	protected function writeDocument(){
		return true; // we don't have to write!
	}

	protected function setElementsFromHTTP(){
		parent::setElementsFromHTTP();
		if(!$_REQUEST){
			return;
		}
		$regs = [];
		$hrefFields = false;

		foreach(array_keys($_REQUEST) as $n){
			if(preg_match('/^we_' . $this->Name . '_(' . self::TYPE_HREF . ')$/', $n, $regs)){
				${$regs[1] . 'Fields'} |= true;
			}
		}

		if($hrefFields){
			$empty = ['int' => 1, 'intID' => '', 'intPath' => '', 'extPath' => ''];
			$hrefs = $match = [];
			foreach($_REQUEST['we_' . $this->Name . '_' . self::TYPE_HREF] as $k => $val){
				if(preg_match('|^(.+)' . we_base_link::MAGIC_INFIX . '(.+)$|', $k, $match)){
					$hrefs[$match[1]][$match[2]] = $val;
				}
			}
			foreach($hrefs as $k => $v){
				$href = array_merge($empty, $v);
				$this->setElement($k, we_serialize($href, SERIALIZE_JSON), self::TYPE_HREF);
			}
		}
	}

	public function we_save($resave = false, $skipHook = false){
		$this->save();
		if(!$resave){
			we_base_history::insertIntoHistory($this);
		}
		/* hook */
		$this->unregisterMediaLinks();
		$ret = $this->registerMediaLinks(true, false);
		if($skipHook){
			return true;
		}
		$hook = new we_hook_base('save', '', [$this, 'resave' => $resave]);
//check if doc should be saved
		if($hook->executeHook() === false){
			$this->errMsg = $hook->getErrorString();
			return false;
		}
		return true;
	}

	function registerMediaLinks($temp = false, $linksReady = false){// FIXME: publish is obsolete for classes
		$serializedArray = is_array($this->SerializedArray) ? $this->SerializedArray : [];
		foreach($serializedArray as $k => $v){
			if(count(($typeName = explode('_', $k, 2))) > 1){
				switch($typeName[0]){
					case 'binary':
					case 'flashmovie':
					case 'img':
						if(!empty($v['default'])){
							$this->MediaLinks[$typeName[0] . '[name=' . $typeName[1] . ']'] = $v['default'];
						}
						break;
					case 'link':
						$default = we_unserialize(isset($v['default']) ? $v['default'] : '', ['type' => 0, 'ctype' => 0]);
						if($default['type'] === 'int' && $default['id']){
							$this->MediaLinks[$typeName[0] . '[name=' . $typeName[1] . ']'] = $default['id'];
						}
						if($default['ctype'] === 'int' && $default['img_id']){
							$this->MediaLinks[$typeName[0] . '[name=' . $typeName[1] . ']'] = $default['img_id'];
						}
						break;
					case 'text':
						if($v['dhtmledit'] == 'on' || $v['dhtmledit'] === true){//FIXME: make bool!
							$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::reparseInternalLinks($v['default'], false, $typeName[1]));
						}
						break;
					default:
					//
				}
			}
		}

		return parent::registerMediaLinks(false, true);
	}

	/**
	 * @return	boolean if the field can have variants the function returns true otherwise false
	 * @param	$field - the name of the field
	 */
	function isVariantField($field){
		$type = ($this->getElement($field . self::ELEMENT_TYPE, we_contents_base::ETYPE_TEXT) ) ?: '';
		return $type && in_array($type, self::VARIANT_TYPES);
	}

	/**
	 * @return	boolean the function returns the number of variant fields
	 */
	function hasVariantFields(){
		return !empty($this->getVariantFields());
	}

	/**
	 * if document can have variants the function returns true, otherwise false
	 *
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants($checkFields = false){
		return true;
	}

	private function loadDefaultsValues(){
		$this->DefaultValues = $this->DefaultValues ?: f('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . $this->ID, $this->DB_WE);
	}

	/**
	 * @desc 	the function returns the array with all object field names
	 * @return	array with the filed names and attributes
	 */
	function getAllVariantFields(){
		$return = [];
		$this->loadDefaultsValues();
		$fields = we_unserialize($this->DefaultValues);
		foreach($fields as $name => $field){
			if($this->isVariantField($name)){
				$return[$name] = $field;
			}
		}
		return $return;
	}

	/**
	 * @return	array with the filed names and attributes
	 * @param	none
	 */
	function getVariantFields(){
		$return = [];
		$fields = $this->getAllVariantFields();
		foreach($fields as $name => $field){
			if(isset($field['variant']) && $field['variant'] == 1){
				$return[$name] = $field;
			}
		}
		return $return;
	}

	/* creates the DirectoryChoooser field with the 'browse'-Button. Clicking on the Button opens the fileselector */

	protected function formDirChooser($rootDirID = 0, $table = '', $Pathname = 'ParentPath', $IDName = 'ParentID', $cmd = '', $pathID = 0, $identifier = ''){
		$path = id_to_path($pathID);

		if(!$table){
			$table = $this->Table;
		}
		$textname = 'we_' . $this->Name . '_' . $Pathname . ($identifier ? '_' . $identifier : '');
		$idname = 'we_' . $this->Name . '_' . $IDName;

		// Important: custom callback commands must call we_cmd('pathOfDocumentChanged') at the beginning
		$cmd = $cmd ?: 'pathOfDocumentChanged';
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',document.we_form.elements['" . $idname . "'].value,'" . $table . "','" . $idname . "','" . $textname . "','" . $cmd . "','','" . $rootDirID . "')");

		return we_html_tools::formElementTable('', we_html_tools::textInput($textname, $path, '', ['readonly' => 'readonly'], 'text', 0, 0), we_html_element::hidden($idname, $pathID), $button);
	}

	function userCanSave($ctConditionOk = false){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return true;
		}
		$ownersReadOnly = array_filter(explode(',', $this->UsersReadOnly));
		return parent::userCanSave() && !we_users_util::isUserInUsers($_SESSION['user']['ID'], $ownersReadOnly);
	}

	static function isUsedByObjectFile($id){
		return $id && f('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' WHERE IsClassFolder=0 AND TableID=' . intval($id) . ' LIMIT 1');
	}

	public function getDocumentCss(){
		return id_to_path($this->CSS, FILE_TABLE, null, true);
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd){
		if($this->EditPageNr != we_base_constants::WE_EDITPAGE_WORKSPACE){
			$parts = [
				['headline' => g_l('weClass', '[path]'),
					'html' => $this->formPath(),
					'icon' => we_html_multiIconBox::PROP_PATH
				],
				['headline' => g_l('modules_object', '[default]'),
					'html' => $this->formDefault($jsCmd),
					'icon' => we_html_multiIconBox::PROP_ATTRIB
				],
				['headline' => g_l('modules_object', '[name]'),
					'html' => $this->formDefaultName($jsCmd),
					'icon' => we_html_multiIconBox::PROP_AUTONAME,
					'fold' => true
				],
				['headline' => g_l('modules_object', '[seourl]'),
					'html' => $this->formDefaultUrl($jsCmd),
					'icon' => we_html_multiIconBox::PROP_SEO,
					'fold' => true
				],
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? ['icon' => we_html_multiIconBox::PROP_CATEGORIES, 'headline' => g_l('global', '[categorys]'), 'html' => $this->formCategory($jsCmd),
				'fold' => true] : null),
				['icon' => we_html_multiIconBox::PROP_USER, 'headline' => g_l('weClass', '[owners]'), 'html' => $this->formRestrictUsers() . '<br/>' .
					($this->RestrictUsers ? $this->formUsers($jsCmd) : ''), 'fold' => true],
				['headline' => g_l('weClass', '[Charset]'),
					'html' => $this->formCharset(),
					'icon' => we_html_multiIconBox::PROP_CHARSET
				],
				['headline' => g_l('weClass', '[CSS]'),
					'html' => $this->formCSS($jsCmd),
					'icon' => we_html_multiIconBox::PROP_OTHER,
					'fold' => true
				],
				['headline' => g_l('modules_object', '[copyClass]'),
					'html' => $this->formCopyDocument(),
					'icon' => we_html_multiIconBox::PROP_COPY
				]
			];
		} else {
			$parts = [
				['headline' => g_l('weClass', '[workspaces]'),
					'html' => $this->formWorkspaces($jsCmd),
					'icon' => we_html_multiIconBox::PROP_WORKSPACE
				],
				['headline' => g_l('modules_object', '[behaviour]'),
					'html' => $this->formWorkspacesFlag(),
					'icon' => we_html_multiIconBox::PROP_DISPLAY
				]
			];
		}
		return we_html_multiIconBox::getHTML('PropertyPage', $parts, true);
	}

}
