<?php

/**
 * webEdition CMS
 *
 * $Rev: 15093 $
 * $Author: mokraemer $
 * $Date: 2018-03-21 12:07:05 +0100 (Wed, 21 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_glossary_tree extends we_tree_base{

	protected function customJSFile(){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'glossary/glossary_tree.js', 'initTree();');
	}

	public static function getItems($ParentId, $Offset = 0, $Segment = 500, $sort = false){
		$Types = [we_glossary_glossary::TYPE_ABBREVATION,
			we_glossary_glossary::TYPE_ACRONYM,
			we_glossary_glossary::TYPE_FOREIGNWORD,
			we_glossary_glossary::TYPE_LINK,
			we_glossary_glossary::TYPE_TEXTREPLACE,
		];

		$Temp = explode('_', $ParentId);
		if(in_array($Temp[(count($Temp) - 1)], $Types)){
			$Type = array_pop($Temp);
			$Language = implode('_', $Temp);
			return self::getItemsFromDB($Language, $Type, $Offset, $Segment);
		}
		if(in_array($ParentId, $GLOBALS['we']['lang']['FrontendLanguages'], true)){
			return self::getTypes($ParentId);
		}
		return self::getLanguages();
	}

	private static function getLanguages(){
		$Items = [];

		foreach(getWeFrontendLanguagesForBackend() as $Key => $Val){
			$Items[] = ['id' => $Key,
				'parentid' => 0,
				'text' => $Val,
				'typ' => 'group',
				'open' => 0,
				'disabled' => 0,
				'tooltip' => $Val,
				'offset' => 0,
				'published' => 1,
				'cmd' => "glossary_view_folder",
				'contenttype' => we_base_ContentTypes::FOLDER
			];
		}

		return $Items;
	}

	private static function getTypes($Language){

		$Items = [];

		$Types = [we_glossary_glossary::TYPE_ABBREVATION => g_l('modules_glossary', '[abbreviation]'),
			we_glossary_glossary::TYPE_ACRONYM => g_l('modules_glossary', '[acronym]'),
			we_glossary_glossary::TYPE_FOREIGNWORD => g_l('modules_glossary', '[foreignword]'),
			we_glossary_glossary::TYPE_LINK => g_l('modules_glossary', '[link]'),
			we_glossary_glossary::TYPE_TEXTREPLACE => g_l('modules_glossary', '[textreplacement]'),
		];

		foreach($Types as $Key => $Val){
			$Items[] = [
				'id' => $Language . '_' . $Key,
				'parentid' => $Language,
				'text' => $Val,
				'typ' => 'group',
				'open' => 0,
				'disabled' => 0,
				'tooltip' => $Val,
				'offset' => 0,
				'published' => 1,
				'cmd' => 'glossary_view_type',
				'contenttype' => we_base_ContentTypes::FOLDER
			];
		}

		if(we_base_permission::hasPerm(we_base_perm::GLOSSARY_EDIT_DICTIONARY)){
			$Items[] = [
				'id' => $Language . '_exception',
				'parentid' => $Language,
				'text' => g_l('modules_glossary', '[exception]'),
				'typ' => 'item',
				'open' => 0,
				'disabled' => 0,
				'tooltip' => g_l('modules_glossary', '[exception]'),
				'offset' => 0,
				'published' => 1,
				'cmd' => 'glossary_view_exception',
				'contenttype' => we_base_ContentTypes::GLOSSAR
			];
		}

		return $Items;
	}

	private static function getItemsFromDB($Language, $Type, $Offset = 0, $Segment = 500){
		$Db = new DB_WE();
		$Items = [];
		$PrevOffset = max(0, $Offset - $Segment);

		if($Offset && $Segment){
			$Item = [
				'id' => 'prev_' . $Language . '_' . $Type,
				'parentid' => $Language . "_" . $Type,
				'text' => 'display (' . $PrevOffset . '-' . $Offset . ')',
				"contenttype" => self::CT_UP,
				"table" => GLOSSARY_TABLE,
				"typ" => self::TYP_DOTS,
				"open" => 0,
				"disabled" => 0,
				"tooltip" => '',
				"offset" => $PrevOffset,
			];
			$Items[] = $Item;
		}

		$Db->query('SELECT ID,Type,Language,Text,Published,IsFolder FROM ' . GLOSSARY_TABLE . ' ' .
			' WHERE Language="' . $Db->escape($Language) . '" AND Type="' . $Db->escape($Type) . '"' .
			' ORDER BY (Text REGEXP "^[0-9]") DESC,abs(Text),Text' .
			($Segment ? ' LIMIT ' . intval($Offset) . ',' . intval($Segment) : ''));

		while($Db->next_record(MYSQLI_ASSOC)){
			$Item = ['id' => $Db->f('ID'),
				'parentid' => $Language . '_' . $Type,
				'text' => oldHtmlspecialchars($Db->f('Text')),
				'typ' => 'item',
				'open' => 0,
				'disabled' => 0,
				'tooltip' => intval($Db->f('ID')),
				'offset' => $Offset,
				'published' => ($Db->f('Published') > 0 ? 1 : 0),
				'contenttype' => ($Db->f('IsFolder') ? we_base_ContentTypes::FOLDER : we_base_ContentTypes::GLOSSAR),
			];

			switch($Type){
				case we_glossary_glossary::TYPE_ABBREVATION:
					$Item['cmd'] = 'glossary_edit_abbreviation';
					break;
				case we_glossary_glossary::TYPE_ACRONYM:
					$Item['cmd'] = "glossary_edit_acronym";
					break;
				case we_glossary_glossary::TYPE_FOREIGNWORD:
					$Item['cmd'] = "glossary_edit_foreignword";
					break;
				case we_glossary_glossary::TYPE_LINK:
					$Item['cmd'] = "glossary_edit_link";
					break;
				case we_glossary_glossary::TYPE_TEXTREPLACE:
					$Item['cmd'] = "glossary_edit_textreplacement";
					break;
			}

			$Items[] = $Item;
		}

		$Total = f('SELECT COUNT(1) FROM ' . $Db->escape(GLOSSARY_TABLE)/* . ' ' . $Where */, '', $Db);

		$NextOffset = $Offset + $Segment;
		if($Segment && ($Total > $NextOffset)){
			$Items[] = ['id' => 'next_' . $Language . '_' . $Type,
				'parentid' => $Language . '_' . $Type,
				'text' => 'display (' . $NextOffset . '-' . ($NextOffset + $Segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => GLOSSARY_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $NextOffset,
			];
		}
		return $Items;
	}

}
