<?php
/**
 * webEdition CMS
 *
 * $Rev: 15672 $
 * $Author: mokraemer $
 * $Date: 2018-08-30 01:14:31 +0200 (Thu, 30 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class
 * @desc    class for tag <we:listview type="banner">
 *
 */
class we_listview_banner extends we_listview_base{
	private $allviews = 0;
	private $allclicks = 0;
	private $UseFilter = 0;
	private $FilterDate = 0;
	private $FilterDateEnd = 0;
	private $docs;

	/**
	 * @desc    constructor of class
	 *
	 * @param   name          string - name of listview
	 * @param   rows          integer - number of rows to display per page
	 * @param   order         string - field name(s) to order by
	 * @param   bannerID      integer - ID of banner
	 * @param   UseFilter     integer - 0 or 1
	 * @param   FilterDate    integer - Unix Timestamp
	 * @param   FilterDateEnd integer - Unix Timestamp
	 *
	 */
	function __construct($name, $rows, $order, $bannerID, $UseFilter, $FilterDate, $FilterDateEnd){
		parent::__construct($name, $rows, 0, $order);

		$this->bannerID = $bannerID;
		$this->UseFilter = $UseFilter;
		$this->FilterDate = $FilterDate;
		$this->FilterDateEnd = $FilterDateEnd;
		$this->allviews = 0;
		$this->allclicks = 0;
		$this->count = $this->start;
		$this->docs = [];

		$tempArray = [];
		$tempArray2 = [];

		$ord = stripos($this->order, 'views') === 0 ? 'ORDER BY ' . $this->order : '';
		$this->DB_WE->query('SELECT DID,COUNT(1) AS views FROM ' . BANNER_VIEWS_TABLE . ' WHERE DID!=0 AND (Page="" OR page="0") AND ID=' . intval($this->bannerID) . ' ' . ($this->UseFilter ? ' AND (Timestamp BETWEEN ' . intval($this->FilterDate) . ' AND ' . intval($this->FilterDateEnd) . ')' : '') . ' GROUP  BY DID');
		while($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$tempArray[$this->DB_WE->f('DID')] = ['views' => $this->DB_WE->f('views')
			];
			$this->allviews += intval($this->DB_WE->f('views'));
		}

		$this->DB_WE->query('SELECT DID,COUNT(1) AS clicks FROM ' . BANNER_CLICKS_TABLE . ' WHERE DID!=0 AND (Page="" OR page="0") AND ID=' . intval($this->bannerID) . ' ' . ($this->UseFilter ? ' AND (Timestamp BETWEEN ' . intval($this->FilterDate) . ' AND ' . intval($this->FilterDateEnd) . ')' : '') . ' GROUP BY DID');
		while($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$tempArray[$this->DB_WE->f('DID')]['clicks'] = $this->DB_WE->f('clicks');
			$this->allclicks += intval($this->DB_WE->f('clicks'));
		}

		$this->DB_WE->query('SELECT Page,COUNT(1) AS views FROM ' . BANNER_VIEWS_TABLE . ' WHERE Page!="" AND Page!="0" AND ID=' . intval($this->bannerID) . ' ' . ($this->UseFilter ? ' AND (Timestamp BETWEEN ' . intval($this->FilterDate) . ' AND ' . intval($this->FilterDateEnd) . ')' : '') . ' GROUP  BY Page');
		while($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$tempArray2[$this->DB_WE->f('Page')] = ['views' => $this->DB_WE->f('views')];
			$this->allviews += intval($this->DB_WE->f('views'));
		}
		$this->DB_WE->query('SELECT Page,COUNT(1) AS clicks FROM ' . BANNER_CLICKS_TABLE . ' WHERE Page!="" AND Page!="0" AND ID=' . intval($this->bannerID) . ' ' . ($this->UseFilter ? ' AND (Timestamp BETWEEN ' . intval($this->FilterDate) . ' AND ' . intval($this->FilterDateEnd) . ')' : '') . ' GROUP BY Page');
		while($this->DB_WE->next_record(MYSQLI_ASSOC)){
			$tempArray2[$this->DB_WE->f('Page')]['clicks'] = $this->DB_WE->f('clicks');
			$this->allclicks += intval($this->DB_WE->f('clicks'));
		}

		// correct views entry on main banner table
		$allviews = f('SELECT COUNT(1) FROM ' . BANNER_VIEWS_TABLE . ' WHERE ID=' . intval($this->bannerID), '', $this->DB_WE);
		$this->DB_WE->query('UPDATE ' . BANNER_TABLE . ' SET views=' . $allviews . ' WHERE ID=' . intval($this->bannerID));


		foreach($tempArray as $did => $vals){
			$this->docs[] = [
				'did' => $did,
				'views' => (isset($vals['views']) ? $vals['views'] : 0),
				'clicks' => (isset($vals['clicks']) ? $vals['clicks'] : 0),
				'page' => ''
			];
		}

		foreach($tempArray2 as $page => $vals){
			$this->docs[] = [
				'did' => 0,
				'views' => isset($vals['views']) ? $vals['views'] : 0,
				'clicks' => isset($vals['clicks']) ? $vals['clicks'] : 0,
				'page' => $page
			];
		}

		if(stripos('path', $this->order) === 0){
			usort($this->docs, (preg_match('|^path +desc|i', $this->order) ? 'we_sort_banners_path_desc' : 'we_sort_banners_path'));
		} else if(stripos('clicks', $this->order) === 0){
			usort($this->docs, (preg_match('|^clicks +desc|i', $this->order) ? 'we_sort_banners_clicks_desc' : 'we_sort_banners_clicks'));
		} else if(stripos('views', $this->order) === 0){
			usort($this->docs, (preg_match('|^views +desc|i', $this->order) ? 'we_sort_banners_views_desc' : 'we_sort_banners_views'));
		} else if(stripos('rate', $this->order) === 0){
			usort($this->docs, (preg_match('|^rate +desc|i', $this->order) ? 'we_sort_banners_rate_desc' : 'we_sort_banners_rate'));
		}
		$this->anz_all = count($this->docs);
		$this->anz = min($this->rows, $this->anz_all - $this->start);
	}

	public function next_record(){
		if($this->count >= min($this->start + $this->rows, $this->anz_all)){
			return false;
		}
		$id = intval($this->docs[$this->count]['did']);
		$path = $id ? id_to_path($id, FILE_TABLE) : $this->docs[$this->count]['page'];
		$views = abs($this->docs[$this->count]['views']);
		$clicks = abs($this->docs[$this->count]['clicks']);
		$this->Record = [
			'id' => $id,
			'path' => $path,
			'views' => $views,
			'page' => $this->docs[$this->count]['page'],
			'clicks' => $clicks,
			'rate' => round($views ? (100 * ($clicks / $views)) : 0, 1)
		];

		$this->count++;
		return true;
	}

	public function f($key, $stripWE = true){
		return parent::f($key, $stripWE);
	}

	function getAllviews(){
		return intval($this->allviews);
	}

	function getAllclicks(){
		return intval($this->allclicks);
	}

	function getAllrate(){
		return round($this->getAllviews() ? (100 * ($this->getAllclicks() / $this->getAllviews())) : 0, 1);
	}

}

function we_sort_banners_path($a, $b){
	$aa = $a['did'] ? id_to_path($a['did'], FILE_TABLE) : $a['page'];
	$bb = $b['did'] ? id_to_path($b['did'], FILE_TABLE) : $b['page'];
	return strcmp($aa, $bb);
}

function we_sort_banners_path_desc($a, $b){
	return we_sort_banners_path($a, $b) * -1;
}

function we_sort_banners_clicks($a, $b){
	if(intval($a['clicks']) == intval($b['clicks'])){
		return 0;
	}
	return (intval($a['clicks']) > intval($b['clicks'])) ? 1 : -1;
}

function we_sort_banners_clicks_desc($a, $b){
	return we_sort_banners_clicks($a, $b) * -1;
}

function we_sort_banners_views($a, $b){
	if(intval($a['views']) === intval($b['views'])){
		return 0;
	}
	return (intval($a['views']) > intval($b['views'])) ? 1 : -1;
}

function we_sort_banners_views_desc($a, $b){
	return we_sort_banners_views($a, $b) * -1;
}

function we_sort_banners_rate($a, $b){
	$rate_a = round($a['views'] ? (100 * ($a['clicks'] / $a['views'])) : 0, 1);
	$rate_b = round($b['views'] ? (100 * ($b['clicks'] / $b['views'])) : 0, 1);
	if($rate_a == $rate_b){
		return 0;
	}
	return ($rate_a > $rate_b) ? 1 : -1;
}

function we_sort_banners_rate_desc($a, $b){
	return we_sort_banners_rate($a, $b) * -1;
}
