<?php
/**
 * webEdition CMS
 *
 * $Rev: 15735 $
 * $Author: mokraemer $
 * $Date: 2018-09-10 12:02:00 +0200 (Mon, 10 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Filter model class for navigation tool
 *
 */
class we_navigation_customerFilter extends we_customer_abstractFilter{
	private $useDocumentFilter = true;

	public function __construct($mode = self::OFF, array $specificCustomers = [], array $blackList = [], array $whiteList = [], array $filter = []){
		parent::__construct($mode, $specificCustomers, $blackList, $whiteList, $filter);
	}

	/**
	 * initialize object with a naviagtion model
	 *
	 * @param we_navigation_navigation $navModel
	 */
	public function initByNavModel(we_navigation_navigation $navModel){
		// convert navigation data into data the filter model needs
		$this->setMode($navModel->FilterMode);
		$this->setFilter($navModel->CustomerFilter);
		$this->setBlackList(empty($navModel->BlackList) ? [] : $navModel->BlackList);
		$this->setWhiteList(empty($navModel->WhiteList) ? [] : $navModel->WhiteList );
		$this->setSpecificCustomers(empty($navModel->Customers) ? [] : $navModel->Customers );
		$this->setUseDocumentFilter($navModel->UseDocumentFilter);
	}

	/**
	 * initialize object with a navigation item
	 *
	 * @param we_navigation_item $navItem
	 */
	function initByNavItem(we_navigation_item $navItem){
		$this->setMode($navItem->filtermode);
		$this->setFilter(empty($navItem->customers['filter']) ? [] : $navItem->customers['filter']);
		$this->setBlackList(empty($navItem->customers['blacklist']) ? [] : $navItem->customers['blacklist']);
		$this->setWhiteList(empty($navItem->customers['whitelist']) ? [] : $navItem->customers['whitelist']);
		$this->setSpecificCustomers(empty($navItem->customers['id']) ? [] : $navItem->customers['id']);
	}

	function getUseDocumentFilter(){
		return $this->useDocumentFilter;
	}

	function setUseDocumentFilter($useDocumentFilter){
		$this->useDocumentFilter = $useDocumentFilter;
	}

	static function getUseDocumentFilterFromRequest(){
		return we_base_request::_(we_base_request::BOOL, 'wecf_useDocumentFilter');
	}

	public static function updateByFilter(we_customer_documentFilter $filterObj, $id, $table){
		$GLOBALS['DB_WE']->query('UPDATE ' . NAVIGATION_TABLE . ' SET ' .
			we_database_base::arraySetter([
				'FilterMode' => $filterObj->getMode(),
				'Customers' => implode(',', $filterObj->getSpecificCustomers()),
				'CustomerFilter' => we_serialize($filterObj->getFilter(), SERIALIZE_JSON),
				'BlackList' => implode(',', $filterObj->getBlackList()),
				'WhiteList' => implode(',', $filterObj->getWhiteList())
			]) .
			' WHERE
	UseDocumentFilter=1 AND
	SelectionType="' . ($table === OBJECT_FILES_TABLE ? we_navigation_navigation::STYPE_OBJLINK : we_navigation_navigation::STYPE_DOCLINK) . '" AND
	LinkID=' . intval($id));
	}

}
