<?php

/**
 * webEdition CMS
 *
 * $Rev: 15093 $
 * $Author: mokraemer $
 * $Date: 2018-03-21 12:07:05 +0100 (Wed, 21 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @abstract This class executes commands and provides views depending on requesting client.
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpc_cmdShell{
	protected $Protocol;
	protected $Cmd;
	protected $View;
	protected $Response;
	protected $Status = we_rpc_cmd::STATUS_OK;

	public function __construct($protocol){
		$this->Protocol = $protocol;
		$this->Cmd = $this->createCmd();

		if(($view = we_base_request::_(we_base_request::STRING, 'view'))){
			if(!$this->isViewAllowed($view)){
				$this->Status = we_rpc_cmd::STATUS_NOT_ALLOWED_VIEW;
			}
		} else {
			$view = $this->CmdName;
		}
		if($this->Status == we_rpc_cmd::STATUS_OK){
			$this->View = $this->getView($view);
		}
	}

	private function createCmd(){
		$this->CmdName = we_base_request::_(we_base_request::STRING, 'cmd');
		$classname = 'we_rpcCmd_' . $this->CmdName;
		if(!class_exists($classname, true)){
			$this->Status = we_rpc_cmd::STATUS_NO_CMD;
			t_e('RPC command not found', $this->CmdName, $_REQUEST);
			return null;
		}

		$obj = new $classname($this);
		$obj->executeRpcCmd($this);
		$this->Status = $obj->getStatus();

		return $obj;
	}

	function getView($view){
		return new we_rpc_genericJSONView($this, $this->Protocol);
	}

	private function isViewAllowed($view){
		if($view == $this->CmdName){
			return true;
		}

		if($this->Cmd->ExtraViews){
			return in_array($view, $this->Cmd->ExtraViews);
		}

		return false;
	}

	function executeCommand(){
		$this->Response = $this->Cmd->execute();
	}

	function cleanUp(){
		$this->Cmd->cleanUp();
	}

	function getResponse(){
		return $this->View->getResponse($this->Response);
	}

	function executeInternalCmd(&$cmd){
		$cmd = $this->createCmd();
		return $cmd->execute();
	}

	function getInternalView($cmd){
		$View = $this->getView($cmd);
		return $View->getResponse($this->executeInternalCmd($cmd));
	}

	public function getErrorOut(){
		switch($this->Status){
			case we_rpc_cmd::STATUS_NO_CMD:
				return 'ERROR: No command defined!';
			case we_rpc_cmd::STATUS_NO_VIEW:
				return 'ERROR: No view defined!';
			case we_rpc_cmd::STATUS_NO_SESSION:
				return 'ERROR: No session exists!';
			default:
				return 'ERROR';
		}
	}

	function getStatus(){
		return $this->Status;
	}

}
