<?php

/**
 * webEdition CMS
 *
 * $Rev: 15394 $
 * $Author: mokraemer $
 * $Date: 2018-06-21 14:06:00 +0200 (Thu, 21 Jun 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_selector_delete extends we_selector_file{

	function __construct($id, $table = FILE_TABLE){
		parent::__construct($id, $table);
		$this->title = g_l('fileselector', '[delSelector][title]');
	}

	function printHTML($what = we_selector_file::FRAMESET, $withPreview = true){
		switch($what){
			case self::DEL:
				$this->printDoDelEntryHTML();
				break;
			default:
				parent::printHTML($what);
		}
	}

	protected function setFramesetJavaScriptOptions(){
		parent::setFramesetJavaScriptOptions();
		$this->jsoptions['options']['seemForOpenDelSelector'] = intval(isset($_SESSION['weS']['seemForOpenDelSelector']['ID']) ? $_SESSION['weS']['seemForOpenDelSelector']['ID'] : 0);
		unset($_SESSION['weS']['seemForOpenDelSelector']['ID']);
	}

	protected function printCmdHTML(we_base_jsCmd $weCmd){
		$weCmd->addCmd('setButtons', [['DelBut', $this->dir ? true : false]]);
		parent::printCmdHTML($weCmd);
	}

	function renameChildrenPath($id, we_database_base $db = null){
		$db = $db ?: new DB_WE();
		$parentPath = f('SELECT Path FROM ' . $db->escape($this->table) . ' WHERE ID=' . intval($id), "", $db);
		$db->query('UPDATE ' . $db->escape($this->table) . ' SET Path=CONCAT("' . $parentPath . '/",Text) WHERE ParentID=' . intval($id));
		$db->query('SELECT ID FROM ' . $db->escape($this->table) . ' WHERE IsFolder=1 AND ParentID=' . intval($id));
		$all = $db->getAll(true);
		foreach($all as $id){
			$this->renameChildrenPath($id, $db);
		}
	}

	function printDoDelEntryHTML(){
		$weCmd = new we_base_jsCmd();
		if(($del = we_base_request::_(we_base_request::INT, "todel"))){
			$_SESSION['weS']['fragDel'] = [
				'todel' => $del,
				'we_not_deleted_entries' => [],
				'we_go_seem_start' => false,
			];
			$weCmd->addCmd('we_cmd', ['del_frag', $del]);
			$weCmd->addCmd('close');
		}
		echo we_html_tools::getHtmlTop('', '', '', $weCmd->getCmds(), we_html_element::body());
	}

	protected function printFooterTable(){
		if($this->values['Text'] === '/'){
			$this->values['Text'] = '';
		}
		$okBut = we_html_button::create_button(we_html_button::DELETE, "javascript:if(document.we_form.fname.value==''){top.exit_close();}else{top.deleteEntry();}", '', "", true, false);

		$cancelbut = we_html_button::create_button(we_html_button::CANCEL, "javascript:top.exit_close();");

		return '
<table id="footer" class="defaultfont">
	<tr>
		<td class="description">' . g_l('fileselector', '[filename]') . '</td>
		<td style="text-align:left">' . we_html_tools::textInput("fname", $this->values['Text'], "", ['style' => "width:100%", 'readonly' => "readonly"]) . '</td>
	</tr>
</table><div id="footerButtons">' . ($okBut ? we_html_button::position_yes_no_cancel($okBut, null, $cancelbut) : $cancelbut) . '</div>';
	}

	protected function query(){
		$this->db->query('SELECT ' . $this->fields . ' FROM ' . $this->db->escape($this->table) . ' WHERE ParentID=' . intval($this->dir) . ' AND((1' . we_users_util::makeOwnersSql() . ')' .
			self::getWsQuery($this->table, false) . ')' . ($this->order ? (' ORDER BY IsFolder DESC,' . $this->order) : '')
		);
	}

}
