<?php

/**
 * webEdition CMS
 *
 * $Rev: 15595 $
 * $Author: mokraemer $
 * $Date: 2018-08-21 14:25:45 +0200 (Tue, 21 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_versions_search{
	private $db;
	public $searchFields = [];
	public $location = [];
	public $search = [];
	public $mode = 0;
	public $order = 'version DESC';
	public $anzahl = 25;
	public $searchstart = 0;
	public $height = 1;
	public $where;
	public $View;

	/**
	 *  Constructor for class 'weVersionsSearch'
	 */
	public function __construct($view = null){
		$this->db = new DB_WE();
		$this->View = $view ?: new we_versions_view();
	}

	/**
	 * @abstract make WHERE-Statement for mysql-SELECT
	 */
	public function getWhere($model){
		$where = '';
		$modConst = [];
		$currentSearch = $model->getProperty('currentSearch');
		$currentSearchFields = $model->getProperty('currentSearchFields');
		$currentLocation = $model->getProperty('currentLocation');

		if(($model->mode)){
			if(count($currentSearchFields)){
				foreach($currentSearchFields as $k => $v){
					switch($v){
						case 'modifierID':
							if(isset($currentSearch[$k])){
								$where .= ' AND ' . $v . '="' . escape_sql_query($currentSearch[$k]) . '"';
							}
							break;
						case 'status':
							if(isset($currentSearch[$k])){
								$where .= ' AND ' . $v . '="' . escape_sql_query($currentSearch[$k]) . '"';
							}
							break;
						case 'timestamp':
							if(isset($currentLocation[$k]) && isset($currentSearch[$k]) && $currentSearch[$k]){

								$date = explode('.', $currentSearch[$k]);
								$day = $date[0];
								$month = $date[1];
								$year = $date[2];
								$timestampStart = mktime(0, 0, 0, $month, $day, $year);
								$timestampEnd = mktime(23, 59, 59, $month, $day, $year);

								switch($currentLocation[$k]){
									case we_search_base::OP_IS:
										$where .= ' AND ' . $v . ' BETWEEN ' . intval($timestampStart) . " AND " . intval($timestampEnd);
										break;
									case we_search_base::OP_LO:
										$where .= ' AND ' . $v . '<' . intval($timestampStart);
										break;
									case we_search_base::OP_LEQ:
										$where .= ' AND ' . $v . '<=' . intval($timestampEnd);
										break;
									case we_search_base::OP_HI:
										$where .= ' AND ' . $v . '>' . intval($timestampEnd);
										break;
									case we_search_base::OP_HEQ:
										$where .= ' AND ' . $v . '>=' . intval($timestampStart);
										break;
								}
							} else {
								$where .= ' AND 1 ';
							}
							break;
						case 'allModsIn':
							if(isset($currentSearch[$k]) && array_key_exists($currentSearch[$k], we_versions_version::MOD_FIELDS)){
								$modConst[] = we_versions_version::MOD_FIELDS[$currentSearch[$k]];
							}
							break;
					}
				}
				if($modConst){
					//FIXME: this is absolute nonsense
					$ids = [];
					$this->db->query('SELECT ID,modifications FROM ' . VERSIONS_TABLE . ' WHERE modifications!=""');
					$modifications = array_map('makeArrayFromCSV', $this->db->getAllFirst(false));

					$m = 0;
					foreach($modConst as $k => $v){
						foreach($modifications as $key => $val){
							if(in_array($v, $modifications[$key])){
								$ids[$m][] = $key;
							}
						}$m++;
					}

					if(!empty($ids)){
						foreach($ids as $key => $val){
							$ids[] = $val;
						}
						$arr = [];
						if(!empty($ids[0])){
							//more then one field
							$mtof = false;
							foreach($ids as $k => $v){
								if($k != 0){
									$mtof = true;
									foreach($v as $key => $val){
										if(!in_array($val, $ids[0])){
											unset($ids[0][$val]);
										} else {
											$arr[] = $val;
										}
									}
								}
							}
							if($mtof){
								$where .= ' AND ID IN (' . implode(',', $arr) . ') ';
							} elseif(!empty($ids[0])){
								$where .= ' AND ID IN (' . implode(',', $ids[0]) . ') ';
							} else {
								$where .= ' AND 0';
							}
						}
					} else {
						$where .= ' AND 0';
					}
				}
			}
		}

		return $where;
	}

	/**
	 * @abstract get modification-fields for filter-SELECT
	 * @return array of modification-fields
	 */
	public function getModFields(){
		$modFields = [];

		foreach(we_versions_version::MOD_FIELDS as $k => $v){
			if($k != 'status'){
				$modFields[$k] = g_l('versions', '[' . $k . ']');
			}
		}

		return $modFields;
	}

	/**
	 * @abstract get filter categories for filter-SELECT
	 * @return array of filter categories
	 */
	public function getFields(){

		$tableFields = ['allModsIn' => g_l('versions', '[allModsIn]'),
			'timestamp' => g_l('versions', '[modTime]'),
			'modifierID' => g_l('versions', '[modUser]'),
			'status' => g_l('versions', '[status]')
		];

		return $tableFields;
	}

	/**
	 * @abstract get all user for filter-SELECT in category 'modifierID'
	 * @return array of users
	 */
	public function getUsers(){
		return $GLOBALS['DB_WE']->getAllFirstq('SELECT ID,username FROM ' . USER_TABLE . ' WHERE Type="user"' .
				(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ? '' : ' AND ParentID=' . $_SESSION['user']['Group']), false);
	}

	/**
	 * @abstract get status
	 * @return array of stats
	 */
	public function getStats(){
		return [
			'published' => g_l('versions', '[published]'),
			'unpublished' => g_l('versions', '[unpublished]'),
			'saved' => g_l('versions', '[saved]'),
			'deleted' => g_l('versions', '[deleted]'),
		];
	}

}
