<?php

/**
 * webEdition CMS
 *
 * $Rev: 15714 $
 * $Author: mokraemer $
 * $Date: 2018-09-05 15:22:25 +0200 (Wed, 05 Sep 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_input extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			return $foo;
		}

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$val = oldHtmlspecialchars($GLOBALS['we_doc']->issetElement($name) ? $GLOBALS['we_doc']->getElement($name) : $value);

		if($GLOBALS['we']['editmode']){
			//all edit-specific things
			$values = self::getAttribute('values', $attribs, '', we_base_request::RAW);
			$mode = self::getAttribute('mode', $attribs, '', we_base_request::STRING);
			$reload = self::getAttribute('reload', $attribs, false, we_base_request::BOOL);
			$seperator = self::getAttribute('seperator', $attribs, '|', we_base_request::RAW);
			return we_html_element::span(['class' => 'we we_input'], self::getEdit($type, $name, $reload, $val, $values, $seperator, $mode, $attribs));
		}
		//not-editmode
		switch($type){
			case 'date':
				return $GLOBALS['we_doc']->getField($attribs, 'date');
			case 'checkbox':
				return $GLOBALS['we_doc']->getElement($name);
			case 'country':
				$lang = self::getAttribute('outputlanguage', $attribs, '', we_base_request::STRING);
				if(!$lang){
					$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
					$doc = self::getDocForTag($docAttr);
					$lang = $doc->Language;
				}
				if(!$lang){
					$lang = explode('_', $GLOBALS['WE_LANGUAGE']);
					$langcode = array_search($lang[0], getWELangs());
				} else {
					$langcode = substr($lang, 0, 2);
				}
				if($GLOBALS['we_doc']->getElement($name) === '--'){
					return '';
				}
				return CheckAndConvertISOfrontend(we_base_country::getTranslation($GLOBALS['we_doc']->getElement($name), we_base_country::TERRITORY, $langcode));
			case 'language':
				$lang = self::getAttribute('outputlanguage', $attribs, '', we_base_request::STRING);
				if(!$lang){
					$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
					$doc = self::getDocForTag($docAttr);
					$lang = $doc->Language;
				}
				$langcode = substr($lang, 0, 2);
				if(!$lang){
					$lang = explode('_', $GLOBALS['WE_LANGUAGE']);
					$langcode = array_search($lang[0], getWELangs());
				}
				return CheckAndConvertISOfrontend(we_base_country::getTranslation($GLOBALS['we_doc']->getElement($name), we_base_country::LANGUAGE, $langcode));
			case 'choice':
				return $GLOBALS['we_doc']->getElement($name);
			case 'select':
				return $GLOBALS['we_doc']->getField($attribs, 'select');
			case 'text':
			default:
				return $GLOBALS['we_doc']->getField($attribs);
		}
	}

	private static function editDate($name, $currentdate, $format, $attribs){
		$d = abs($GLOBALS['we_doc']->getElement($name, we_contents_base::ETYPE_NUMBER));
		return we_html_tools::getDateInput('we_' . $GLOBALS['we_doc']->Name . '_date[' . $name . ']', $d ?: ($currentdate ? time() : 0), true, $format);
	}

	private static function editCcheckbox($name, $reload, $val, $attribs){
		$attribs = removeAttribs($attribs, ['name', 'value', 'type', '_name_orig', 'reload']);
		$attribs['type'] = 'checkbox';
		$attribs['name'] = 'we_' . $GLOBALS['we_doc']->Name . '_attrib_' . $name;
		$attribs['value'] = 1;
		$attribs['onclick'] = '_EditorFrame.setEditorIsHot(true);this.form.elements[\'we_' . $GLOBALS['we_doc']->Name . '_checkbox[' . $name . ']\'].value=(this.checked ? 1 : 0);' . ($reload ? (';setScrollTo();top.we_cmd(\'reload_editpage\');') : '');
		if($val){
			$attribs['checked'] = 'checked';
		}

		return we_html_element::hidden('we_' . $GLOBALS['we_doc']->Name . '_checkbox[' . $name . ']', $val) .
			getHtmlTag('input', $attribs);
	}

	private static function editCountry($name, $docAttr, $attribs){
		$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
			'choice', 'pure', 'rows', 'cols', 'maxlength', 'wysiwyg']);
		$newAtts['name'] = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']';
		$newAtts['onclick'] = '_EditorFrame.setEditorIsHot(true);';

		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = ($lang ? substr($lang, 0, 2) : array_search($GLOBALS['WE_LANGUAGE'], getWELangs()));

		$orgVal = $GLOBALS['we_doc']->getElement($name);
		$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		setlocale(LC_ALL, $oldLocale);
		$orgVal = $GLOBALS['we_doc']->getElement($name);
		$content = '';
		if(WE_COUNTRIES_DEFAULT != ''){
			$content .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
		}
		foreach($topCountries as $countrykey => &$countryvalue){
			$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue) . '</option>';
		}
		unset($countryvalue);
		$content .= '<option value="-" disabled="disabled">----</option>';
		foreach($shownCountries as $countrykey2 => &$countryvalue2){
			$content .= '<option value="' . $countrykey2 . '" ' . ($orgVal == $countrykey2 ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($countryvalue2) . '</option>';
		}
		unset($countryvalue2);

		return getHtmlTag('select', $newAtts, $content, true);
	}

	private static function editLanguage($name, $docAttr, $attribs){
		$newAtts = removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
			'choice', 'pure', 'rows', 'cols', 'maxlength', 'wysiwyg']);
		$newAtts['name'] = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']';
		$newAtts['onclick'] = '_EditorFrame.setEditorIsHot(true);';
		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = ($lang ? substr($lang, 0, 2) : array_search($GLOBALS['WE_LANGUAGE'], getWELangs()));

		$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		$frontendLL = [];
		foreach($frontendL as &$lcvalue){
			$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
		}

		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($frontendLL, SORT_LOCALE_STRING);
		setlocale(LC_ALL, $oldLocale);
		$content = '';
		$orgVal = $GLOBALS['we_doc']->getElement($name);
		foreach($frontendLL as $langkey => &$langvalue){
			$content .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . CheckAndConvertISOfrontend($langvalue) . '</option>';
		}
		unset($langvalue);
		return getHtmlTag('select', $newAtts, $content, true);
	}

	private static function editChoice($name, $val, $values, $seperator, $mode, $reload, $attribs){
		if($values){
			$tagname = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']';
			$vals = explode($seperator, $values);

			$onChange = ($mode === 'add' ?
				"this.form.elements['" . $tagname . "'].value += ((this.form.elements['" . $tagname . "'].value ? ' ' : '')+this.options[this.selectedIndex].text);" :
				"this.form.elements['" . $tagname . "'].value = this.options[this.selectedIndex].text;") .
				($reload ? 'setScrollTo();top.we_cmd(\'reload_editpage\');' : '');

			$sel = getHtmlTag('select', [
				'class' => 'defaultfont',
				'name' => 'we_choice_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']',
				'onchange' => $onChange . ';this.selectedIndex=0;_EditorFrame.setEditorIsHot(true);'
				], ($vals ? '<option disabled="disabled">' . g_l('button', '[select][value]') . '</option><option>' . implode('</option><option>', $vals) . '</option>' : ''), true);
		}

		$attribs['onchange'] = '_EditorFrame.setEditorIsHot(true);';
		$attribs['type'] = 'text';
		$attribs['name'] = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']';
		$attribs['value'] = $val;

		return getHtmlTag('input', removeAttribs($attribs, ['mode', 'values', '_name_orig'])) . '&nbsp;' . (isset($sel) ? $sel : '');
	}

	private static function editDefault($name, $val, $attribs){
		$attribs['onchange'] = '_EditorFrame.setEditorIsHot(true);';
		$attribs['class'] = "wetextinput";
		$attribs['type'] = "text";
		$attribs['name'] = 'we_' . $GLOBALS['we_doc']->Name . '_txt[' . $name . ']';
		$attribs['value'] = $val;
		return getHtmlTag('input', removeAttribs($attribs, ['mode', 'values', '_name_orig']));
	}

	private static function getEdit($type, $name, $reload, $val, $values, $seperator, $mode, $attribs){
		switch($type){
			case 'date':
				$format = self::getAttribute('format', $attribs, '', we_base_request::STRING);
				$currentdate = self::getAttribute('currentdate', $attribs, false, we_base_request::BOOL);
				return self::editDate($name, $currentdate, $format, $attribs);
			case 'checkbox':
				return self::editCcheckbox($name, $reload, $val, $attribs);
			case 'country':
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				return self::editCountry($name, $docAttr, $attribs);
			case 'language':
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				return self::editLanguage($name, $docAttr, $attribs);
			case 'choice':
				return self::editChoice($name, $val, $values, $seperator, $mode, $reload, $attribs);
			case 'select':
				//NOTE: this tag is for objects only
				return $GLOBALS['we_doc']->getField($attribs, 'select');
			case 'print':
				return $val;
			case 'text':
			default:
				return self::editDefault($name, $val, $attribs);
		}
	}

}

function we_tag_input(array $attribs){
	return we_tag_input::tag($attribs);
}
