<?php

/**
 * webEdition CMS
 *
 * $Rev: 15663 $
 * $Author: mokraemer $
 * $Date: 2018-08-28 20:01:53 +0200 (Tue, 28 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_listview extends we_weTag_tag{
	public static $lvArray = [];
	public static $current = null;

	public static function parse(array $attribs, $content){
		switch(self::getParserAttribute('type', $attribs)){
			default:
			case 'document':
			case 'search':
			case 'languagelink':
			case 'customer':
			case 'onlinemonitor':
			case 'order':
			case 'shopVariant':
			case 'category':
				break;
			case 'sql':
				$predefinedSQL = self::getParserAttribute('predefinedSQL', $attribs, '');
				if(!$predefinedSQL){
					if(($foo = self::attributeMissing($attribs, ['select' => false, 'from' => false, 'where' => false], __CLASS__))){
						return $foo;
					}
				}
				break;
			case 'object':
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					$predefinedSQL = self::getParserAttribute('predefinedSQL', $attribs, '');
					if($predefinedSQL){
						$attribs['type'] = 'sql';
						t_e('deprecated', 'use of deprecated predefinedSQL');
					} elseif(($foo = self::attributeMissing($attribs, 'classid', __CLASS__))){
						return $foo;
					}
				}
				break;
			case 'orderitem':
				if(we_modules_info::isActive(we_modules_info::SHOP) && ($foo = self::attributeMissing($attribs, 'orderid', __CLASS__))){
					return $foo;
				}
				break;
			case 'multiobject':
				if(we_modules_info::isActive(we_modules_info::OBJECT) && ($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
					return $foo;
				}
				break;
			case 'banner':
				if(we_modules_info::isActive(we_modules_info::BANNER) && ($foo = self::attributeMissing($attribs, 'path', __CLASS__))){
					return $foo;
				}
				break;
		}
		//setting global $lv is for backward compatibility
		return '<?php ' . (strpos($content, '$lv') !== false ? 'global $lv;' : '') .
			we_weTag_parser::printTag('listview', $attribs) . ';?>' . $content . '<?php we_weTag_tag::post_listview();?>';
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		$name = self::getAttribute('name', $attribs, 0, we_base_request::STRING);
		$doctype = self::getAttribute('doctype', $attribs, '', we_base_request::STRING);
		$classid = self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$we_lv_cats = we_base_request::_(we_base_request::WEFILELIST, 'we_lv_cats_' . $name, self::getAttribute('categories', $attribs, '', we_base_request::WEFILELIST));
		$categoryids = self::getAttribute('categoryids', $attribs, '', we_base_request::INTLIST);
		$we_lv_categoryids = we_base_request::_(we_base_request::INTLIST, 'we_lv_categoryids_' . $name, $categoryids);
		$we_lv_catOr = we_base_request::_(we_base_request::BOOL, 'we_lv_catOr_' . $name, self::getAttribute('catOr', $attribs, false, we_base_request::BOOL));

		$we_rows = intval(self::getAttribute('rows', $attribs, 100000000, we_base_request::INT));
		$order = self::getAttribute('order', $attribs, '', we_base_request::STRING);
		$we_lv_order = we_base_request::_(we_base_request::STRING, 'we_lv_order_' . $name, $order);

		$we_lv_numorder = we_base_request::_(we_base_request::BOOL, 'we_lv_numorder_' . $name, self::getAttribute('numorder', $attribs, false, we_base_request::BOOL));
		$id = self::getAttribute('id', $attribs, false, we_base_request::INTLIST);
		$cond = self::getAttribute('condition', $attribs, '', we_base_request::RAW) ?: (isset($GLOBALS['we_lv_condition']) ? $GLOBALS['we_lv_condition'] : '');
		if($cond && $cond{0} != '$' && isset($GLOBALS[$cond])){
			$cond = $GLOBALS[$cond];
		}
		$type = self::getAttribute('type', $attribs, 'document', we_base_request::STRING);
		$desc = self::getAttribute('desc', $attribs, false, we_base_request::BOOL);
		$we_lv_desc = we_base_request::_(we_base_request::BOOL, 'we_lv_desc_' . $name, $desc);

		$predefinedSQL = self::getAttribute('predefinedSQL', $attribs, '', we_base_request::RAW);
		$we_offset = intval(self::getAttribute('offset', $attribs, 0, we_base_request::INT));
		$workspaceID = self::getAttribute('workspaceID', $attribs, self::getAttribute('workspaceid', $attribs, [], we_base_request::INTLISTA, true), we_base_request::INTLISTA);
		$we_lv_ws = we_base_request::_(we_base_request::INTLISTA, 'we_lv_ws_' . $name, $workspaceID);

		$orderid = self::getAttribute('orderid', $attribs, 0, we_base_request::INT);

		$we_lv_languages = we_base_request::_(we_base_request::HTML, 'we_lv_languages_' . $name, self::getAttribute('languages', $attribs, '', we_base_request::STRING));
		$we_lv_pagelanguage = we_base_request::_(we_base_request::HTML, 'we_lv_pagelanguage_' . $name, self::getAttribute('pagelanguage', $attribs, '', we_base_request::STRING));
		$showself = self::getAttribute('showself', $attribs, false, we_base_request::BOOL);

		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$docid = self::getAttribute('docid', $attribs, 0, we_base_request::INT);
		$customers = self::getAttribute('customers', $attribs); // csv value of Ids
		$casesensitive = self::getAttribute('casesensitive', $attribs, false, we_base_request::BOOL);
		$customer = self::getAttribute('customer', $attribs, false, we_base_request::BOOL);
		$we_lv_ct = we_base_request::_(we_base_request::HTML, 'we_lv_ct_' . $name, self::getAttribute('contenttypes', $attribs, '', we_base_request::STRING));

		$cols = self::getAttribute('cols', $attribs, '', we_base_request::INT);
		$we_lv_se = we_base_request::_(we_base_request::BOOL, 'we_lv_se_' . $name, self::getAttribute('searchable', $attribs, true, we_base_request::BOOL));

		$seeMode = self::getAttribute('seeMode', $attribs, true, we_base_request::BOOL); //	backwards compatibility


		$calendar = self::getAttribute('calendar', $attribs, '', we_base_request::STRING);
		$datefield = self::getAttribute('datefield', $attribs, '', we_base_request::STRING);
		$date = self::getAttribute('date', $attribs, '', we_base_request::STRING);
		$weekstart = self::getAttribute('weekstart', $attribs, 'monday', we_base_request::STRING);

		// deprecated, because subfolders acts the other way arround as it should
		$subfolders = (isset($attribs['subfolders'])) ?
			!self::getAttribute('subfolders', $attribs, false, we_base_request::BOOL) :
			self::getAttribute('recursive', $attribs, true, we_base_request::BOOL);

		$we_lv_subfolders = isset($_REQUEST['we_lv_subfolders_' . $name]) ? (bool) $_REQUEST['we_lv_subfolders_' . $name] : $subfolders;

		$cfilter = self::getAttribute('cfilter', $attribs, false, we_base_request::BOOL);
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		$objectseourls = self::getAttribute('objectseourls', $attribs, TAGLINKS_OBJECTSEOURLS, we_base_request::BOOL);
		$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);

		switch($we_lv_languages){
			case 'self':
			case 'top':
				$we_lv_langguagesdoc = self::getDocForTag($we_lv_languages);
				$we_lv_languages = $we_lv_langguagesdoc->Language;
				unset($we_lv_langguagesdoc);
		}

		$we_lv_calendar = we_base_request::_(we_base_request::STRING, 'we_lv_calendar_' . $name, $calendar);
		$we_lv_datefield = we_base_request::_(we_base_request::STRING, 'we_lv_datefield_' . $name, $datefield);
		$we_lv_date = we_base_request::_(we_base_request::STRING, 'we_lv_date_' . $name, ($date ?: date('Y-m-d')));
		$we_lv_weekstart = we_base_request::_(we_base_request::STRING, 'we_lv_weekstart_' . $name, $weekstart);

		$we_lv_cats = ($we_lv_cats === 'we_doc' ? we_category_category::we_getCatsFromDoc($GLOBALS['we_doc'], ',', true, $GLOBALS['DB_WE']) : $we_lv_cats);
		//FIXME: here cats are retransformed to text, this should not happen.

		switch($type){
			case 'document':
				if($id === 0 || $id === '0'){
					break;
				}
				$lv = new we_listview_document($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $doctype, $we_lv_cats, $we_lv_catOr, $casesensitive, $we_lv_ws, $we_lv_ct, $cols, $we_lv_se, $cond, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $cfilter, $we_lv_subfolders, $customers, $id, $we_lv_languages, $we_lv_numorder, $hidedirindex, $triggerid);
				break;
			case 'search':
				$lv = new we_listview_search($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $doctype, $classid, $we_lv_cats, $we_lv_catOr, $casesensitive, $we_lv_ws, $triggerid, $cols, $cfilter, $we_lv_languages, $hidedirindex, $objectseourls);
				break;
			case 'object':
				if(!we_modules_info::isActive(we_modules_info::OBJECT)){
					echo self::modulMissingError('Object/DB', __CLASS__ . ' type="object"');
					return false;
				}
				if($id === 0 || $id === '0'){
					return '';
				}
				if(f('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classid))){
					$lv = new we_listview_object($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $classid, $we_lv_cats, $we_lv_catOr, $cond, $triggerid, $cols, $seeMode, $we_lv_se, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $we_lv_ws, $cfilter, $docid, $customers, $id, $predefinedSQL, $we_lv_languages, $hidedirindex, $objectseourls);
				} else {
					t_e('warning', 'Class with id=' . intval($classid) . ' does not exist');
					return false;
				}
				break;
			case 'languagelink':
				$we_lv_langguagesdoc = self::getDocForTag($we_lv_pagelanguage);
				$we_lv_ownlanguage = $we_lv_langguagesdoc->Language;

				switch(we_tag_listview::$current ? get_class(we_tag_listview::$current) : ''){
					case 'we_listview_object':
						$we_lv_pageID = we_tag_listview::$current->f(we_listview_base::ID_FIELD);
						$we_lv_linktype = OBJECT_FILES_TABLE;
						$we_lv_pagelanguage = $we_lv_pagelanguage === 'self' ? we_tag_listview::$current->f(we_listview_base::LANG_FIELD) : ($we_lv_pagelanguage === 'top' ? $we_lv_ownlanguage : $we_lv_pagelanguage);
						$we_lv_ownlanguage = we_tag_listview::$current->f(we_listview_base::LANG_FIELD);
						break;
					default:
						$we_lv_pagelanguage = $we_lv_pagelanguage === 'self' || $we_lv_pagelanguage === 'top' ? $we_lv_ownlanguage : self::getDocForTag($docAttr)->Language;

						/**
						 * Fix #9694
						 * attention: we can not check $we_lv_langguagesdoc instanceof we_objectFile
						 * $we_lv_langguagesdoc is always instance of webEditionDocument because
						 * we need an webEdition Document to show webEdition object detail pages
						 */
						$we_lv_pageID = isset($GLOBALS['we_obj']) ? $GLOBALS['we_obj']->ID : $we_lv_langguagesdoc->ID;
						$we_lv_linktype = isset($GLOBALS['we_obj']) ? OBJECT_FILES_TABLE : FILE_TABLE;
				}
				unset($we_lv_langguagesdoc);

				$lv = new we_listview_langlink($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $we_lv_linktype, $cols, $showself, $we_lv_pageID, $we_lv_pagelanguage, $we_lv_ownlanguage, $hidedirindex, $objectseourls, $we_lv_subfolders);
				break;
			case 'customer':
				if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
					echo self::modulMissingError('Customer', __CLASS__ . ' type="customer"');
					break;
				}
				$lv = new we_listview_customer($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $hidedirindex);
				break;
			case 'onlinemonitor':
				if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
					$id = we_base_request::_(we_base_request::INT, 'we_omid', 0);
					$lastaccesslimit = self::getAttribute('lastaccesslimit', $attribs, 300, we_base_request::INT);
					$lastloginlimit = self::getAttribute('lastloginlimit', $attribs, 0, we_base_request::INT);
					$lv = new we_listview_customerOnlineMonitor($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $lastaccesslimit, $lastloginlimit, $hidedirindex);
					break;
				}
				echo self::modulMissingError('Customer', __CLASS__ . ' type="onlinemonitor"');
				break;
			case 'order':
				if(!we_modules_info::isActive(we_modules_info::SHOP)){
					echo self::modulMissingError('Shop', __CLASS__ . ' type="order"');
					break;
				}
				$lv = new we_listview_shopOrder($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $hidedirindex);
				break;
			case 'orderitem':
				if(!we_modules_info::isActive(we_modules_info::SHOP)){
					echo self::modulMissingError('Shop', __CLASS__ . ' type="orderitem"');
					break;
				}
				$lv = new we_listview_shopOrderitem($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $cond, $cols, $docid, $orderid, $hidedirindex);
				break;
			case 'multiobject':
				if(!we_modules_info::isActive(we_modules_info::OBJECT)){
					echo self::modulMissingError('Object/DB', __CLASS__ . ' type="multiobject"');
					break;
				}
				$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
				$lv = new we_listview_multiobject($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $we_lv_cats, $we_lv_catOr, $cond, $triggerid, $cols, $seeMode, $we_lv_se, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $cfilter, $docid, $we_lv_languages, $hidedirindex, $objectseourls);
				break;
			case 'banner':
				if(!we_modules_info::isActive(we_modules_info::BANNER)){
					echo self::modulMissingError('Banner', __CLASS__ . ' type="banner"');
					break;
				}
				$usefilter = self::getAttribute('usefilter', $attribs, false, we_base_request::BOOL);
				$path = self::getAttribute('path', $attribs, '', we_base_request::FILE);
				$filterdatestart = self::getAttribute('filterdatestart', $attribs, -1, we_base_request::INT);
				$filterdateend = self::getAttribute('filterdateend', $attribs, -1, we_base_request::INT);
				$bannerid = f('SELECT ID FROM ' . BANNER_TABLE . ' WHERE PATH="' . $GLOBALS[DB_WE]->escape($path) . '"');
				if($customer && we_modules_info::isActive(we_modules_info::CUSTOMER) && !empty($_SESSION['webuser']['registered']) && (!we_banner_banner::customerOwnsBanner($_SESSION['webuser']['ID'], $bannerid, $GLOBALS['DB_WE']))){
					$bannerid = 0;
				}
				$lv = new we_listview_banner($name, $we_rows, $order, $bannerid, $usefilter, $filterdatestart, $filterdateend);
				break;
			case 'shopVariant': // TODO: Remove in webEdition 7 - for backwords compatibility since FR# 8556
				if(!we_modules_info::isActive(we_modules_info::SHOP)){
					echo self::modulMissingError('Shop', __CLASS__ . ' type="shopVariant"');
					break;
				}
			case 'variant':
				$defaultname = self::getAttribute('defaultname', $attribs, '', we_base_request::STRING);
				$docId = self::getAttribute('documentid', $attribs, 0, we_base_request::INT);
				$objectId = self::getAttribute('objectid', $attribs, 0, we_base_request::INT) ?: (we_tag_listview::$current ? intval(we_tag_listview::$current->f(we_listview_base::ID_FIELD)) : 0);

				$lv = new we_listview_variants($name, $we_rows, $defaultname, $docId, $objectId, $we_offset, $hidedirindex, $objectseourls, $triggerid);
				break;
			case 'category':
				$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
				$parentidname = self::getAttribute('parentidname', $attribs, '', we_base_request::STRING);
//$categoryids="' . $categoryids . '";
//$parentid="' . $parentid . '";
				$lv = new we_listview_category($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $parentid, $categoryids, $cols, ($parentidname ? $parentidname : ''), $hidedirindex);
				break;
			case 'collectionitems':
				/*
				 * priorities for $id:
				 * 1) $GLOBALS['we']['COLLECTION_ID']: coming from we_gallery plugin in tinymce
				 * 2) $GLOBALS['we_doc']->getElement($name): selection from tag <we:collectionSelect>
				 * 3) $id: attribute on listview: can thus be used as default from template
				 */
				$id = !empty($GLOBALS['we']['COLLECTION_ID']) ? $GLOBALS['we']['COLLECTION_ID'] :
					((we_tag_listview::$current && (($collectionID = intval(we_tag_listview::$current->f($name))) > 0)) ? $collectionID :
					((isset($GLOBALS['we_doc']) && ($collectionID = $GLOBALS['we_doc']->getElement($name, we_contents_base::ETYPE_NUMBER))) ? $collectionID : $id));

				$lv = new we_listview_collectionItems($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $doctype, $we_lv_cats, $we_lv_catOr, $casesensitive, $we_lv_ws, $we_lv_ct, $cols, $we_lv_se, $cond, $we_lv_calendar, $we_lv_datefield, $we_lv_date, $we_lv_weekstart, $we_lv_categoryids, $cfilter, $we_lv_subfolders, $customers, $id, $we_lv_languages, $we_lv_numorder, $hidedirindex, $triggerid);
				break;
			case 'sql':
				$select = self::getAttribute('select', $attribs, '', we_base_request::STRING);
				$from = self::getAttribute('from', $attribs, '', we_base_request::STRING);
				$where = self::getAttribute('where', $attribs, '', we_base_request::STRING);
				$lv = new we_listview_sql($name, $we_rows, $we_offset, $we_lv_order, $we_lv_desc, $select, $from, $where, $predefinedSQL);
				break;
			default:
				echo self::parseError('Unknown type ' . $type, __CLASS__);
				break;
		}

		self::pre_listview(isset($lv) ? $lv : new stdClass());
		return '';
	}

}

function we_tag_listview(array $attribs){
	return we_tag_listview::tag($attribs);
}
