<?php

/**
 * webEdition CMS
 *
 * $Rev: 15656 $
 * $Author: mokraemer $
 * $Date: 2018-08-28 01:19:43 +0200 (Tue, 28 Aug 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_shipping extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(($foo = self::attributeMissing($attribs, "sum", __CLASS__))){
			return $foo;
		}

		$sumName = self::getAttribute('sum', $attribs, '', we_base_request::STRING);

		// shipping depends on total value of basket
		$orderVal = we_tag_sum::getSum($sumName);
		if($orderVal === false){
			return 0;
		}

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);

		$weShippingControl = we_shop_shippingControl::getShippingControl();

		// check if user is registered
		$customer = (we_tag('ifRegisteredUser') ? $_SESSION['webuser'] : false);

		$shippingCost = $weShippingControl->getShippingCostByOrderValue($orderVal, $customer);

		// get calculated value if needed
		// if user must NOT pay vat always return net prices
		$mustPayVat = we_tag('ifShopPayVat'); // alayways return net prices

		switch($type){
			case 'net':
				//no difference if $mustPayVat or not
				if(!$weShippingControl->isNet){
					// y = x * (100/116)
					$shippingCost = $shippingCost * (100 / ((1 + ($weShippingControl->vatRate / 100)) * 100) );
				}
				break;
			case 'gros':
				if($weShippingControl->isNet){
					// y = x * (1.16)
					$shippingCost = ($mustPayVat ?
						$shippingCost * (1 + ($weShippingControl->vatRate / 100)) :
						//return net prices
						$shippingCost * (100 / ((1 + ($weShippingControl->vatRate / 100)) * 100) )
						);
				}
				break;
			case 'vat':
				$shippingCost = ($mustPayVat ?
					($weShippingControl->isNet ?
					// y = x * 0.16
					$shippingCost * ($weShippingControl->vatRate / 100) :
					// y = x /116 * 16
					$shippingCost / ( ((1 + ($weShippingControl->vatRate / 100)) * 100) ) * $weShippingControl->vatRate) :
					0);
				break;
			default:
		}

		return we_base_util::formatNumber($shippingCost, self::getAttribute('num_format', $attribs, '', we_base_request::STRING));
	}

}

function we_tag_shipping(array $attribs){
	return we_tag_shipping::tag($attribs);
}
