<?php
/**
 * webEdition CMS
 *
 * $Rev: 15093 $
 * $Author: mokraemer $
 * $Date: 2018-03-21 12:07:05 +0100 (Wed, 21 Mar 2018) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * WorkfFlow Document Task definition
 *
 * This class describe document task in workflow process
 *
 */
class we_workflow_documentTask extends we_workflow_base{
	const STATUS_NEW = 0;
	const STATUS_APPROVED = 1;
	const STATUS_CANCELED = 2;

	// workflow document task ID
	var $ID = 0;
	// workflow document step ID
	var $documentStepID = 0;
	// workflow task ID
	var $workflowTaskID = 0;
	// date when task is done
	var $Date = 0;
	// Status of document task
	var $Status = self::STATUS_NEW;

	/**
	 * Default Constructor
	 */
	function __construct($wfDocumentTask = 0){
		parent::__construct();
		$this->table = WORKFLOW_DOC_TASK_TABLE;
		$this->ClassName = __CLASS__;

		$this->persistents = [
			'ID' => we_base_request::INT,
			'documentStepID' => we_base_request::INT,
			'workflowTaskID' => we_base_request::INT,
			'Date' => we_base_request::INT,
			'Status' => we_base_request::INT,
		];

		if($wfDocumentTask){
			$this->ID = $wfDocumentTask;
			$this->load();
		}
	}

	public function approve(){
		$this->Status = self::STATUS_APPROVED;
		$this->Date = time();
	}

	public function decline(){
		$this->Status = self::STATUS_CANCELED;
		$this->Date = time();
	}


	//--------------------------------STATIC FUNCTIONS ------------------------------
	/**
	 * returns all tasks for workflow step
	 *
	 */
	static function getAllTasks($workflowDocumentStep){
		$db=$GLOBALS['DB_WE'];
		$all = $db->getAllq('SELECT ID FROM ' . WORKFLOW_DOC_TASK_TABLE . ' WHERE documentStepID=' . intval($workflowDocumentStep) . ' ORDER BY ID', true);

		$docTasks = [];

		foreach($all as $cur){
			$docTasks[] = new self($cur);
		}
		return $docTasks;
	}

	/**
	 * creates all tasks for workflow step
	 *
	 */
	public static function createAllTasks($workflowStepID, we_database_base $db){
		$all = $db->getAllq('SELECT ID FROM ' . WORKFLOW_TASK_TABLE . ' WHERE stepID=' . intval($workflowStepID) . ' ORDER BY ID', true);
		$docTasks = [];
		foreach($all as $cur){
			$docTasks[] = self::createTask($cur);
		}
		return $docTasks;
	}

	/**
	 * Create task
	 */
	private static function createTask($id){
		$docTask = new we_workflow_documentTask();
		$docTask->workflowTaskID = $id;
		return $docTask;
	}

}
